using System;

namespace Pybench.Aritmethic
{
	public class Chronometer
    {
        private DateTime ticks1, ticks2;
        private bool stopped;

        public void Start()
        {
            ticks1 = DateTime.Now;
            stopped = false;
        }
        public void Stop()
        {
            ticks2 = DateTime.Now;
            stopped = true;
        }

        private static int TicksToMicroSeconds(DateTime t1, DateTime t2)
        {
            return TicksToMiliSeconds(t1, t2) * 1000;
        }

        private static int TicksToMiliSeconds(DateTime t1, DateTime t2)
        {
            TimeSpan difference = t2.Subtract(t1);
            return (difference.Milliseconds + difference.Seconds * 1000 + difference.Minutes * 60000);
        }

        private static int TicksToSeconds(DateTime t1, DateTime t2)
        {
            TimeSpan difference = t2.Subtract(t1);
            return (difference.Seconds + difference.Minutes * 60);
        }

        public int GetMicroSeconds()
        {
            if (stopped)
                return TicksToMicroSeconds(ticks1, ticks2);
            return TicksToMicroSeconds(ticks1, DateTime.Now);
        }

        public int GetMiliSeconds()
        {
            if (stopped)
                return TicksToMiliSeconds(ticks1, ticks2);
            return TicksToMiliSeconds(ticks1, DateTime.Now);
        }

        public int GetSeconds()
        {
            if (stopped)
                return TicksToSeconds(ticks1, ticks2);
            return TicksToSeconds(ticks1, DateTime.Now);
        }
    }
    
	public class BenchMark 
	{
		private int iterations;
		protected int microSeconds;

		public BenchMark(int iterations) 
		{
			this.iterations = iterations;
		}

		public int run() 
		{
			BenchMark self = this;			
			for (int i = 0; i < iterations; i++)
				self.runOneIteration();
			return this.microSeconds;
		}

		virtual public object runOneIteration() { return null; }
	}
	
	public class ArithmethicBenchmark : BenchMark 
	{
		public ArithmethicBenchmark(int iterations) : base(iterations) { }	
		public override object runOneIteration() 
		{				
			Chronometer chronometer = new Chronometer();
			Test test = new IfThenElse();						
			chronometer.Start();				
			test.test();
			chronometer.Stop();			
			this.microSeconds = this.microSeconds + chronometer.GetMicroSeconds();
			return null;
		}
	}
	
	public abstract class Test {
        public abstract void test();
    }

	public class IfThenElse : Test
    {		
        public override void test()
        {
			dynamic one = 1;
			dynamic two = 2;
			dynamic three = 3;
            dynamic a = one;
			dynamic b = two;
			dynamic	c = three;
			dynamic i = 0;
            while (i < (150000 * 3))
            {				
                if (a == 1) {
                    if (b == 2) {
                        if (c != 3) {
                            c = 3;
                            b = 3;
                        }
                        else {
                            c = 2;
                        }
                    }
                    else {
                        if (b == 3) {
                            b = 2;
                            a = 2;
                        }
                    }
                }
                else {
                    if (a == 2)
                        a = 3;
                    else
                        a = 1;
                }
				
				if (a == 1) {
                    if (b == 2) {
                        if (c != 3) {
                            c = 3;
                            b = 3;
                        }
                        else {
                            c = 2;
                        }
                    }
                    else {
                        if (b == 3) {
                            b = 2;
                            a = 2;
                        }
                    }
                }
                else {
                    if (a == 2)
                        a = 3;
                    else
                        a = 1;
                }
				
				if (a == 1) {
                    if (b == 2) {
                        if (c != 3) {
                            c = 3;
                            b = 3;
                        }
                        else {
                            c = 2;
                        }
                    }
                    else {
                        if (b == 3) {
                            b = 2;
                            a = 2;
                        }
                    }
                }
                else {
                    if (a == 2)
                        a = 3;
                    else
                        a = 1;
                }
				
				if (a == 1) {
                    if (b == 2) {
                        if (c != 3) {
                            c = 3;
                            b = 3;
                        }
                        else {
                            c = 2;
                        }
                    }
                    else {
                        if (b == 3) {
                            b = 2;
                            a = 2;
                        }
                    }
                }
                else {
                    if (a == 2)
                        a = 3;
                    else
                        a = 1;
                }
				
				if (a == 1) {
                    if (b == 2) {
                        if (c != 3) {
                            c = 3;
                            b = 3;
                        }
                        else {
                            c = 2;
                        }
                    }
                    else {
                        if (b == 3) {
                            b = 2;
                            a = 2;
                        }
                    }
                }
                else {
                    if (a == 2)
                        a = 3;
                    else
                        a = 1;
                }
				i=i+1;
            }
        }
    } 
	
	public class Program 
	{
		public static void Main(string[] args) 
		{
			if (args.Length<1) {
				Console.Error.WriteLine("You must pass the number of thousands iterations.");
					System.Environment.Exit(-1);
			}
			int iterations = Convert.ToInt32(args[0]);
			ArithmethicBenchmark arith = new ArithmethicBenchmark(iterations);
			Console.WriteLine(arith.run());
		}
    }
}