using System;
using System.Collections;

namespace Pybench.Aritmethic
{
	public class Chronometer
    {
        private DateTime ticks1, ticks2;
        private bool stopped;

        public void Start()
        {
            ticks1 = DateTime.Now;
            stopped = false;
        }
        public void Stop()
        {
            ticks2 = DateTime.Now;
            stopped = true;
        }

        private static int TicksToMicroSeconds(DateTime t1, DateTime t2)
        {
            return TicksToMiliSeconds(t1, t2) * 1000;
        }

        private static int TicksToMiliSeconds(DateTime t1, DateTime t2)
        {
            TimeSpan difference = t2.Subtract(t1);
            return (difference.Milliseconds + difference.Seconds * 1000 + difference.Minutes * 60000);
        }

        private static int TicksToSeconds(DateTime t1, DateTime t2)
        {
            TimeSpan difference = t2.Subtract(t1);
            return (difference.Seconds + difference.Minutes * 60);
        }

        public int GetMicroSeconds()
        {
            if (stopped)
                return TicksToMicroSeconds(ticks1, ticks2);
            return TicksToMicroSeconds(ticks1, DateTime.Now);
        }

        public int GetMiliSeconds()
        {
            if (stopped)
                return TicksToMiliSeconds(ticks1, ticks2);
            return TicksToMiliSeconds(ticks1, DateTime.Now);
        }

        public int GetSeconds()
        {
            if (stopped)
                return TicksToSeconds(ticks1, ticks2);
            return TicksToSeconds(ticks1, DateTime.Now);
        }
    }
    
	public class BenchMark 
	{
		private int iterations;
		protected int microSeconds;

		public BenchMark(int iterations) 
		{
			this.iterations = iterations;
		}

		public int run() 
		{
			BenchMark self = this;			
			for (int i = 0; i < iterations; i++)
				self.runOneIteration();
			return this.microSeconds;
		}

		virtual public object runOneIteration() { return null; }
	}
	
	public class ArithmethicBenchmark : BenchMark 
	{
		public ArithmethicBenchmark(int iterations) : base(iterations) { }	
		public override object runOneIteration() 
		{				
			Chronometer chronometer = new Chronometer();
			Test test = new ListSlicing();						
			chronometer.Start();				
			test.test();
			chronometer.Stop();			
			this.microSeconds = this.microSeconds + chronometer.GetMicroSeconds();
			return null;
		}
	}
	
	public abstract class Test {
        public abstract void test();
    }

	public class ListSlicing : Test 
	{
        public override void test() 
		{
            dynamic n = new ArrayList(100);
            dynamic r = new ArrayList(25);

            for (dynamic i = 0; i < 100; i=i+1)
                n.Add(i);
            for (dynamic i = 0; i < 25; i=i+1)
                r.Add(i);

			dynamic l;
			dynamic m;			
            for(dynamic i = 0; i < 800 ; i=i+1)
            {
                l = new ArrayList(n.GetRange(0, 100));				
                for (dynamic j = 0; j < r.Count; j=j+1)
                {				
                    m = l.GetRange(50, 50);
                    m = l.GetRange(75, 25);
                    m = l.GetRange(50, 5);
                    l.AddRange(n.GetRange(3, 97));
                    m = l.GetRange(0, 99);
                    m = l.GetRange(1, 99);
                    l.RemoveAt(99);
                    l.AddRange(n.GetRange(0, 100));
                }				
            }
        }
    }
	
	public class Program 
	{
		public static void Main(string[] args) 
		{
			if (args.Length<1) {
				Console.Error.WriteLine("You must pass the number of thousands iterations.");
					System.Environment.Exit(-1);
			}
			int iterations = Convert.ToInt32(args[0]);
			ArithmethicBenchmark arith = new ArithmethicBenchmark(iterations);
			Console.WriteLine(arith.run());
		}
    }
}