using System;

namespace Points
{
	public class Chronometer
    {
        private DateTime ticks1, ticks2;
        private bool stopped;

        public void Start()
        {
            ticks1 = DateTime.Now;
            stopped = false;
        }
        public void Stop()
        {
            ticks2 = DateTime.Now;
            stopped = true;
        }

        private static int TicksToMicroSeconds(DateTime t1, DateTime t2)
        {
            return TicksToMiliSeconds(t1, t2) * 1000;
        }

        private static int TicksToMiliSeconds(DateTime t1, DateTime t2)
        {
            TimeSpan difference = t2.Subtract(t1);
            return (difference.Milliseconds + difference.Seconds * 1000 + difference.Minutes * 60000);
        }

        private static int TicksToSeconds(DateTime t1, DateTime t2)
        {
            TimeSpan difference = t2.Subtract(t1);
            return (difference.Seconds + difference.Minutes * 60);
        }

        public int GetMicroSeconds()
        {
            if (stopped)
                return TicksToMicroSeconds(ticks1, ticks2);
            return TicksToMicroSeconds(ticks1, DateTime.Now);
        }

        public int GetMiliSeconds()
        {
            if (stopped)
                return TicksToMiliSeconds(ticks1, ticks2);
            return TicksToMiliSeconds(ticks1, DateTime.Now);
        }

        public int GetSeconds()
        {
            if (stopped)
                return TicksToSeconds(ticks1, ticks2);
            return TicksToSeconds(ticks1, DateTime.Now);
        }
    }
    
	public class BenchMark 
	{
		private int iterations;
		protected int microSeconds;

		public BenchMark(int iterations) 
		{
			this.iterations = iterations;
		}

		public int run() 
		{
			BenchMark self = this;			
			for (int i = 0; i < iterations; i++)
				self.runOneIteration();
			return this.microSeconds;
		}

		virtual public object runOneIteration() { return null; }
	}
	
	public class ArithmethicBenchmark : BenchMark 
	{
		public ArithmethicBenchmark(int iterations) : base(iterations) { }	
		public override object runOneIteration() 
		{				
			Chronometer chronometer = new Chronometer();
			Test test = new PointsBench();						
			chronometer.Start();				
			test.test();
			chronometer.Stop();			
			this.microSeconds = this.microSeconds + chronometer.GetMicroSeconds();
			return null;
		}
	}
	
	public abstract class Test {
        public abstract void test();
    }

	
	public class Node
	{
		public dynamic data;
		public dynamic next;
		public Node(dynamic data, dynamic next)
		{
			this.data = data;
			this.next = next;
		}
	}
	
	public class Point3D
	{
		public dynamic x;
		public dynamic y;
		public dynamic z;		
		public dynamic dimensions;
		public Point3D(dynamic x, dynamic y, dynamic z, dynamic dimensions)
		{
			this.x = x;
			this.y = y;
			this.z = z;	
			this.dimensions = dimensions;			
		}
	}
	
	public class Point2D
	{		
		public dynamic x;
		public dynamic y;		
		public dynamic dimensions;
		public Point2D(dynamic x, dynamic y, dynamic dimensions)
		{
			this.x = x;
			this.y = y;			
			this.dimensions = dimensions;
		}
	}
	
	public class PointsBench : Test
	{	
		dynamic createPoint(dynamic dimensions, dynamic x, dynamic y, dynamic z) 
		{
			dynamic point;
			if (dimensions==2)
				point = new Point2D(x,y,dimensions);
			else
				point = new Point3D(x,y,z,3);
			return point;
		}

		
		dynamic createPoints(dynamic number) {
			dynamic i;
			dynamic list, point;

			i = 1;
			point = createPoint(3,0,0,0);
			list = new Node(point, 0); 
			while(i < number) 
			{
				point = createPoint(i % 2 + 2, number/2 - i, i, i);
				list = new Node(point, list);
				i = i + 1;
			}
			return list;
		}
		
		
		dynamic positiveX(dynamic list, dynamic n) {
			dynamic i;
			dynamic l, result;
			result = i = 0;
			l = list;
			while(i < n)
			{
				if(l.data.x >= 0)
					result = new Node(l.data,result);
				l = l.next;
				i = i + 1;
			}
			return result;
		}

		dynamic distance3D(dynamic point) 
		{
			dynamic value;
			value = 2147483647;				
			if (point.dimensions == 3)
				value = point.x*point.x + point.y*point.y + point.z*point.z;
			return value;
		}

		dynamic closestToOrigin3D(dynamic list, dynamic n) 
		{
			dynamic i, minDistance;
			dynamic l, point3D;
			
			point3D = createPoint(3,0,0,0);
			minDistance = 2147483647;
			l = list;
			i = 0;
			while(i<n) {  
				if(distance3D(l.data)<minDistance) 
				{
					minDistance = distance3D(l.data);
					point3D = l.data;
				}
				l = l.next;
				i = i + 1;
			}
			return point3D;  
		}
	
        public override void test() 
		{          			
            dynamic numberOfPoints;
			dynamic list, positive, point;					
			numberOfPoints = 10000;
			list = createPoints(numberOfPoints);			
			positive = positiveX(list, numberOfPoints);  
			point = closestToOrigin3D(list, numberOfPoints);
        }
    }
	
	public class Program 
	{
		public static void Main(string[] args) 
		{
			if (args.Length<1) {
				Console.Error.WriteLine("You must pass the number of thousands iterations.");
					System.Environment.Exit(-1);
			}
			int iterations = Convert.ToInt32(args[0]);
			ArithmethicBenchmark arith = new ArithmethicBenchmark(iterations);
			Console.WriteLine(arith.run());
		}
    }
}