public class Chronometer 
{
  private Int ticks1;
  private Int ticks2;
  private Bool stopped;
  
  public Void start() 
  {
    this.ticks1 = Duration.nowTicks();
    this.stopped = false;
  }

  public Void stop() {
    this.ticks2 = Duration.nowTicks();
    this.stopped = true;
  }
  
  public Int getMicroSeconds() {
    return (this.ticks2 - this.ticks1)/1000;
  } 
}

class Random
{
  public Int next()
  {
    return Int.random;
  }
  
  public Float nextFloat()
  {
    return Float.random;
  }
}

public abstract class Benchmark 
{
  private Int iterations;
  protected Int microSeconds;
  
  new make(Int iterations) 
  {
    this.iterations = iterations;
  } 
  
  public Int run() {
    for(i := 0 ; i < this.iterations ; i++)
      this.runOneIteration();
    return this.microSeconds;
  }
  
  public virtual Obj? runOneIteration() {
    return null;
  }   
}


class ArithmethicBenchmark : Benchmark 
{
  new make(Int iterations): super(iterations) {}

  public override Obj? runOneIteration() 
  {
    chronometer := Chronometer(); 
    
    test := JGFFFTBench();
    
    chronometer.start();
    test.test();
    chronometer.stop();   
    this.microSeconds = this.microSeconds + chronometer.getMicroSeconds();
    return null;
  }
}

public abstract class TestPyBench {
        public abstract Void test();
    }


 public class JGFFFTBench : TestPyBench
  {
    Obj? size;
    Obj?[]  datasizes;
    Obj? R;
    FFT fft;
    
    new make()
    {
      datasizes = Int?[,];
      datasizes->size = 3; 
      datasizes->set(0, 65536);      
      datasizes->set(1, 8388608);
      datasizes->set(2, 16777216);
      fft = FFT();
      
    }
    
    public override Void test() 
    {   
      JGFsetsize(0);
      JGFinitialise();
      JGFkernel();
      JGFvalidate();     
    }
    
    public Void JGFsetsize(Obj? size)
    {
      this.size = size;
    }
    
    public Void JGFinitialise()
    {      
      R = Random();     
    }
    
    private static Obj? RandomVector(Obj? N, Obj? R)
    {
      Obj? a;
      a = Float?[,];
      a->size = N;
      for (Obj? i := 0; i < N; i = i->plus(1))
        a->set(i, R->nextFloat()->mult(0.000001f));                
      return a;
    }
  
    public Void JGFkernel()
    {       
      Obj? x;
      x = JGFFFTBench.RandomVector((datasizes->get(size))->mult(2), R);
      fft.transform(x);      
      fft.inverse(x); 
    }
  
    public Void JGFvalidate()
    {
      Obj? refval;
      refval = Float?[,];
      refval->size = 3;
      refval->set(0, 1.726962988395339f);
      refval->set(1, 6.907851953579193f);
      refval->set(2, 13.815703907167297f);
      Obj? refvali;
      refvali = Float?[,];
      refvali->size = 3;
      refvali->set(0, 2.0974756152524314f);
      refvali->set(1, 8.389142211032294f);
      refvali->set(2, 16.778094422092604f);
      
      Obj? dev;
      dev = fft.JDKtotal->minus(refval->get(size));
      dev = (dev >= 0.0f ? dev : dev->mult(-1.0f));
      if (dev > 0.000000000001f)
      {
        //echo("Validation failed");
        //echo("FFT.JDKtotal = " + FFT.JDKtotal + "  " + dev + "  " + size);
      }
      
      Obj? devi;
      devi = fft.JDKtotali->minus(refvali->get(size));
      devi = (devi >= 0.0f ? devi : devi->mult(-1.0f));
      if (devi > 0.000000000001f)
      {
        //echo("Validation failed");
        //echo("FFT.JDKtotali = " + FFT.JDKtotali + "  " + devi + "  " + size);
      }
    }    
  }
  
  public class FFT
  {
    public Obj? JDKtotal := 0.0f;
    public Obj? JDKtotali := 0.0f;
    
    public static Obj? log2(Obj? n)
    {
      Obj? log;
      log = 0;
      Obj? k;
      k = 1;        
      while(k < n)
      {                 
        k = k->mult(2);                
        log = log->plus(1);                
      }     
      if (n != (1.shiftl(log)))
        echo("FFT: Data Length is not a power of 2!: " + n);
      return log;       
    }
    
    public static Obj? makeRandom(Obj? n)
    {
      Obj? rand;
      rand = Random();
      Obj? nd;
      nd = n->mult(2);
      Obj? data;
      data = Float?[,];
      data->size = nd;
      for (Obj? i := 0; i < nd; i = i->plus(1))
        data->set(i, rand->nextFloat());                
      return data;
    }    
    
    public static Void bitreverse(Obj? data)
    {     
      Obj? n;
      n = data->size->div(2);
      Obj? j;
      j = 0;
      Obj? i;
      i = 0;          
      while(i < (n->minus(1)))
      {     
        Obj? ii;
        ii = i->mult(2);
        Obj? jj;
        jj = j->mult(2);
        Obj? k;
        k = n->div(2);
        if (i < j)
        {                     
            Obj? tmp_real;
            tmp_real = data->get(ii);                    
            Obj? tmp_imag;
            tmp_imag = data->get(ii->plus(1));
            data->set(ii,data->get(jj));
            data->set(ii->plus(1),data->get(jj->plus(1)));
            data->set(jj,tmp_real);
            data->set(jj->plus(1),tmp_imag);        
        }             
        while (k <= j)
        {                   
            j = j->minus(k);
            k = k->div(2);        
        }       
        j = j->plus(k);
        i = i->plus(1);                        
      }           
    }
    
    public static Void transform_internal(Obj? data, Obj? direction)
    {     
      Obj? n;
      n = data->size->div(2);
      if (n == 1) 
        return;     
      Obj? logn;
      logn = log2(n);     
      bitreverse(data);             
      Obj? dual;
      dual = 1; 
      Obj? bit;
      bit = 0;
      while(bit < logn)            
      {                            
          Obj? w_real;
          w_real = 1.0f;
          Obj? w_imag;
          w_imag = 0.0f;
          Obj? theta;
          theta = direction->mult(2.0f)->mult(3.14159265f)->div(dual->toFloat()->mult(2.0f));     
          Obj? s;
          s = theta->sin;
          Obj? t;
          t = (theta->div(2.0f))->sin;      
          Obj? s2;
          s2 = t->mult(t)->mult(2.0f);
          Obj? iterations;
          iterations = 0;
          while (iterations < n)
          {                    
            Obj? i;
            i= iterations->mult(2);
            Obj? j;
            j = dual->plus(iterations)->mult(2);
            Obj? wd_real;
            wd_real = data->get(j);
            Obj? wd_imag;
            wd_imag = data->get(j->plus(1));                    
            data->set(j,data->get(i)->minus(wd_real));
            data->set(j->plus(1), data->get(i->plus(1))->minus(wd_imag));
            data->set(i,data->get(i)->plus(wd_real));
            data->set(i->plus(1),data->get(i->plus(1))->plus(wd_imag));                    
            iterations = iterations->plus(dual->mult(2));
          }             
          iterations = 1;
        while (iterations < dual)
        {
          Obj? tmp_real;
          tmp_real = w_real->minus(s->mult(w_imag))->minus(s2->mult(w_real));
          Obj? tmp_imag;
          tmp_imag = w_imag->plus(s->mult(w_real))->minus(s2->mult(w_imag));
          w_real = tmp_real;
          w_imag = tmp_imag;
        
          Obj? b;
          b = 0;
          while (b < n)
          {                        
            Obj? i;
            i = b->plus(iterations)->mult(2);
            Obj? j;
            j = b->plus(iterations)->plus(dual)->mult(2);
        
            Obj? z1_real;
            z1_real = data->get(j);
            Obj? z1_imag;
            z1_imag = data->get(j->plus(1));
        
            Obj? wd_real;
            wd_real = w_real->mult(z1_real)->minus(w_imag->mult(z1_imag));
            Obj? wd_imag;
            wd_imag = w_real->mult(z1_imag)->plus(w_imag->mult(z1_real));
        
            data->set(j, data->get(i)->minus(wd_real));
            data->set(j->plus(1), data->get(i->plus(1))->minus(wd_imag));
            data->set(i, data->get(i)->plus(wd_real));
            data->set(i->plus(1), data->get(i->plus(1))->plus(wd_imag));
            b = b->plus((dual->mult(2)));
          }                    
          iterations = iterations->plus(1);
        }
          dual = dual->mult(2);
          bit = bit->plus(1);
      }
    }
    
    public Void transform(Obj? data)
    {
        Obj? JDKrange;
        FFT.transform_internal(data, -1f);
        JDKrange = data->size;
        for (Obj? i := 0; i < JDKrange; i = i->plus(1))          
          this.JDKtotal = data->get(i)->plus(this.JDKtotal);            
    }
    
    public Void inverse(Obj? data)
    {
        FFT.transform_internal(data, 1f);
        Obj? nd;
        nd = data->size;
        Obj? n;
        n = nd->div(2);
        Obj? norm;            
        norm = 1.0f.div(n->toFloat());   
        for (Obj? i := 0; i < nd; i = i->plus(1))  
            data->set(i, data->get(i)->mult(norm));    
        for (Obj? i := 0; i < nd; i = i->plus(1))  
            this.JDKtotali = data->get(i)->plus(this.JDKtotali);
    }    
  }

public class Main
{
  public static Void main(Str[] args)
  {   
    if (args.size < 1)
    {   
      echo("You must pass the number of thousands iterations.");
    }
    else
    {   
      iterations := Int.fromStr(args[0]);   
      arith := ArithmethicBenchmark(iterations)
      echo(arith.run());
    }    
  } 
}