public class Chronometer 
{
  private Int ticks1;
  private Int ticks2;
  private Bool stopped;
  
  public Void start() 
  {
    this.ticks1 = Duration.nowTicks();
    this.stopped = false;
  }

  public Void stop() {
    this.ticks2 = Duration.nowTicks();
    this.stopped = true;
  }
  
  public Int getMicroSeconds() {
    return (this.ticks2 - this.ticks1)/1000;
  } 
}

public abstract class Benchmark 
{
  private Int iterations;
  protected Int microSeconds;
  
  new make(Int iterations) 
  {
    this.iterations = iterations;
  } 
  
  public Int run() {
    for(i := 0 ; i < this.iterations ; i++)
      this.runOneIteration();
    return this.microSeconds;
  }
  
  public virtual Obj? runOneIteration() {
    return null;
  }   
}

class Random
{
  public Int next()
  {
    return Int.random;
  }
  
  public Float nextFloat()
  {
    return Float.random;
  }
}

class MathmulBenchmark : Benchmark 
{
  new make(Int iterations): super(iterations) {}

  public override Obj? runOneIteration() 
  {
    chronometer := Chronometer(); 
    
    test := JGFHeapSortBench();
    
    chronometer.start();
    test.test();
    chronometer.stop();   
    this.microSeconds = this.microSeconds + chronometer.getMicroSeconds();
    return null;
  }
}

public abstract class NumericSortTest {
  public Obj? array_rows;
  public Obj?[] TestArray; 

  public abstract Void test();
  
  new make()
  {
    TestArray = Int?[,];
  }
  
  public Void buildTestData()
  {
    TestArray->size = array_rows;
    Obj? rndnum;
    rndnum = Random(); //Set seed = 1729??? 
    Obj? i;
    i = 0;
    while(i < array_rows)
    {
      TestArray->set(i, rndnum->next());
      i = i->plus(1);
    }     
  }
  
  public Void Do()
  {
      NumHeapSort();
  }

  private Void NumSift(Obj? min, Obj? max)    
  {   
    while (min->plus(min) <= max)
    {                
      Obj? inc;
      inc = false;
      if (min->plus(min) < max && TestArray->get(min->plus(min)) < TestArray->get(min->plus(min)->plus(1)))
        inc = true;
      Obj? k;
      k = inc ? min->plus(min)->plus(1) : min->plus(min);                
      if (TestArray->get(min) < TestArray->get(k))
      {                   
        Obj? temp;
        temp = TestArray->get(k);
        TestArray->set(k, TestArray->get(min));
        TestArray->set(min, temp);                 
        min = k;                    
      }
      else
        min = max->plus(1);
    }   
  }

  private Void NumHeapSort()
  {
    Obj? temp;         
    Obj? top;
    top = array_rows->minus(1);     
    Obj? i;
    i = top->div(2);
    while( i > 0)
    {               
      NumSift(i, top);
      i = i->minus(1);
    } 
    i = top;  
    while( i > 0)
    {
      NumSift(0, i);
      temp = TestArray->get(0);
      TestArray->set(0, TestArray->get(i));
      TestArray->set(i, temp);
      i = i->minus(1);
    }     
  } 
}


 public class JGFHeapSortBench : NumericSortTest
  {
    Obj? size;
    Obj?[]  datasizes;
    
    new make()
    {
      datasizes = Int?[,];
      datasizes->size = 3; 
      datasizes->set(0, 1000000);
      datasizes->set(1, 5000000);
      datasizes->set(2, 25000000);	  
    }
    
    public override Void test() {
      JGFsetsize(0);
      JGFinitialise();
      JGFkernel();
      JGFvalidate();
    }
    
    public Void JGFsetsize(Obj? size)
    {
      this.size = size;
    }

    public Void JGFinitialise()
    {
      array_rows = datasizes->get(size);
      buildTestData();
    }
    
    public Void JGFkernel()
    {
      Do();
    }
    
    public Void JGFvalidate()
    {
      Obj? error;
      error = false;
      Obj? i;
      i = 1;
      while (i < array_rows)
      {
        error = (TestArray->get(i) < TestArray->get(i->minus(1)));        
        if (error)
        {
          //echo("Validation failed");
          //echo("Item " + i + " = " + TestArray[i]);
          //echo("Item " + (i - 1) + " = " + TestArray[i - 1]);          
        }
        i = i->plus(1);
      }
    }  
  }

public class Main
{
  public static Void main(Str[] args)
  {   
    if (args.size < 1)
    {   
      echo("You must pass the number of thousands iterations.");
    }
    else
    {   
      iterations := Int.fromStr(args[0]);   
      arith := MathmulBenchmark(iterations)
      echo(arith.run());
    }    
  } 
}