public class Chronometer 
{
  private Int ticks1;
  private Int ticks2;
  private Bool stopped;
  
  public Void start() 
  {
    this.ticks1 = Duration.nowTicks();
    this.stopped = false;
  }

  public Void stop() {
    this.ticks2 = Duration.nowTicks();
    this.stopped = true;
  }
  
  public Int getMicroSeconds() {
    return (this.ticks2 - this.ticks1)/1000;
  } 
}

public abstract class Benchmark 
{
  private Int iterations;
  protected Int microSeconds;
  
  new make(Int iterations) 
  {
    this.iterations = iterations;
  } 
  
  public Int run() {
    for(i := 0 ; i < this.iterations ; i++)
      this.runOneIteration();
    return this.microSeconds;
  }
  
  public virtual Obj? runOneIteration() {
    return null;
  }   
}


class ArithmethicBenchmark : Benchmark 
{
  new make(Int iterations): super(iterations) {}

  public override Obj? runOneIteration() 
  {
    chronometer := Chronometer(); 
    test := JGFRayTracerBench();
    chronometer.start();
    test.Test();
    chronometer.stop();   
    this.microSeconds = this.microSeconds + chronometer.getMicroSeconds();
    return null;
  }
  
  
}

public class Vec
{
    public Obj? x;
    public Obj? y;
    public Obj? z;
  
    new make()
    {
      x = 0.0f;
      y = 0.0f;
      z = 0.0f;      
    }
  
    new makeComponents(Obj? a, Obj? b, Obj? c)
    {
      x = a;
      y = b;
      z = c;
    }
  
    public Void add1(Obj? a)
    {
      x = a->x->plus(this.x);
      y = a->y->plus(this.y);
      z = a->z->plus(this.z);
    }
  
    public static Obj? addsStatic(Obj? s, Obj? a, Obj? b)
    {
      return Vec(s->mult(a->x)->plus(b->x), s->mult(a->y)->plus(b->y), s->mult(a->z)->plus(b->z));
    }
  
    public Void adds(Obj? s, Obj? b)
    {
      this.x = this.x->plus(s->mult(b->x));
      this.y = this.y->plus(s->mult(b->y));
      this.z = this.z->plus(s->mult(b->z));
    }
  
    public static Obj? sub1(Obj? a, Obj? b)
    {             
      return Vec(a->x->minus(b->x), a->y->minus(b->y), a->z->minus(b->z));                    
    }
  
    public Void sub2(Obj? a, Obj? b)
    {     
      this.x = a->x->minus(b->x);
      this.y = a->y->minus(b->y);
      this.z = a->z->minus(b->z);
    }
  
    public static Obj? mult1(Obj? a, Obj? b)
    {
      return Vec(a->x->mult(b->x), a->ymult(b->y), a->zmult(b->z));
    }
  
    public static Obj? cross(Obj? a, Obj? b)
    {           
      return Vec(a->y->mult(b->z)->minus(a->z->mult(b->y)), a->z->mult(b->x)->minus(a->x->mult(b->z)), a->x->mult(b->y)->minus(a->y->mult(b->x)));            
    }
  
    public static Obj? dot(Obj? a, Obj? b)
    {     
      return a->x->mult(b->x)->plus(a->y->mult(b->y))->plus(a->z->mult(b->z));
    }
  
    public static Obj? comb(Obj? a, Obj? A, Obj? b, Obj? B)
    {     
      return Vec(a->mult(A->x)->plus(b->mult(B->x)), a->mult(A->y)->plus(b->mult(B->y)), a->mult(A->z)->plus(b->mult(B->z)));
    }
  
    public Void comb2(Obj? a, Obj? A, Obj? b, Obj? B)
    {
      x = a->mult(A->x)->plus(b->mult(B->x));
      y = a->mult(A->y)->plus(b->mult(B->y));
      z = a->mult(A->z)->plus(b->mult(B->z));
    }
  
    public Void scale(Obj? t)
    {     
      x = t->mult(x);
      y = t->mult(y);
      z = t->mult(z);
    }
  
    public Void negate()
    {
      x = x->mult(-1.0f);
      y = y->mult(-1.0f);
      z = z->mult(-1.0f);
    }
  
    public Obj? normalize()
    {                 
      Obj? len;
      len = x->mult(x)->plus(y->mult(y))->plus(z->mult(z))->sqrt;    
      while (len > 0.0f)
      {
        x = x->div(len);
        y = y->div(len);
        z = z->div(len);
        len = 0.0f;
      }         
      return len;
    }
  
    public override Str toStr()
    {
      return "<" + x + "," + y + "," + z + ">";
    }   
}

public class View
{
    public Obj? fromVec;
    public Obj? atVec;
    public Obj? upVec;
    public Obj? dist;
    public Obj? angle;
    public Obj? aspect;
  
    new make(Obj? fromVec, Obj? atVec, Obj? upVec, Obj? dist, Obj? angle, Obj? aspect)
    {
      this.fromVec = fromVec;
      this.atVec = atVec;
      this.upVec = upVec;
      this.dist = dist;
      this.angle = angle;
      this.aspect = aspect;
    }
}

public class Ray
{
    public Obj? P;
    public Obj? D;

    new makeRay(Obj? pnt, Obj? dir)
    {                 
      P = Vec();      
      P->add1(pnt);
      D = Vec();      
      D->add1(dir);
      D->normalize();
    }

    new make()
    {
      P = Vec();
      D = Vec();
    }

    public Obj? point(Obj? t)
    {
      return Vec(P->x->plus(D->x->mult(t)), P->y->plus(D->y->mult(t)), P->z->plus(D->z->mult(t)));
    }

    public override Str toStr()
    {
      return "{" + P->toStr() + " -> " + D->toStr() + "}";
    }
}

public class Interval
{
    public Obj? number;
    public Obj? width;
    public Obj? height;
    public Obj? yfrom;
    public Obj? yto;
    public Obj? total;

    new make(Obj? number, Obj? width, Obj? height, Obj? yfrom, Obj? yto, Obj? total)
    {
        this->number = number;
        this->width = width;
        this->height = height;
        this->yfrom = yfrom;
        this->yto = yto;
        this->total = total;
    }
}
  
public class Isect
{
    public Obj? t;
    public Obj? enter;
    public Obj? prim;
    public Obj? surf;
  
    public override Str toStr()
    {
      return "{" + t->toStr() + "," + enter->toStr() + "," + prim->toStr() + "," + surf->toStr() + "}";
    }
}
  
public class Light
{
    public Obj? pos;
    public Obj? brightness;

    new make() {}

    new makeComponents(Obj? x, Obj? y, Obj? z, Obj? brightness)
    {
        this->pos = Vec(x,y,z);
        this->brightness = brightness;
    }
}
  
public class Surface
{
    public Obj? color;
    public Obj? kd;
    public Obj? ks;
    public Obj? shine;
    public Obj? kt;
    public Obj? ior;

    new make()
    {
        color = Vec(1.0f, 0.0f, 0.0f);
        kd = 1.0f;
        ks = 0.0f;
        shine = 0.0f;
        kt = 0.0f;
        ior = 1.0f;
    }

    public override Str toStr()
    {
        return "Surface { color=" + color->toStr() + "}";
    }
}
  
public abstract class Primitive
{
    public Obj? surf;

    new make()
    {
      surf = Surface();
    }

    public Void setColor(Obj? r, Obj? g, Obj? b)
    {
        surf->color = Vec(r, g, b);
    }

    public abstract Obj? intersect(Obj? ry);    
}


public class Sphere : Primitive
{
    Obj? c;
    Obj? r;
    Obj? r2;
    Obj? v;
    Obj? b;

    new make(Obj? center, Obj? radius):super()
    {     
        c = center;
        r = radius;
        r2 = radius->mult(radius);
        v = Vec();
        b = Vec();
    }

    public override Obj? intersect(Obj? ry) 
    {   
        Obj? p;
        p = ry->P;
        Obj? d;
        d = ry->D;            
        v->sub2(c, p);          
        Obj? b;
        b = Vec.dot(v, d);
        Obj? dotVV;
        dotVV = Vec.dot(v, v);
        Obj? disc;
        disc = b->mult(b)->minus(dotVV)->plus(r2);        
        if (disc < 0.0f)                  
          return null;                                  
        disc = disc->sqrt();                              
        Obj? t;
        t = (b->minus(disc) < 0.000001f) ? b->plus(disc) : b->minus(disc);            
        if (t < 0.000001f)            
          return null;          
        Obj? ip;
        ip = Isect();
        ip->t = t;            
        ip->enter = dotVV > r2->plus(1000001f) ? 1.0f : 0.0f;
        ip->prim = this;
        ip->surf = this->surf;
        return ip;          
    }

    public Obj? normal(Obj? p)
    {
        Obj? r;
        r = Vec.sub1(p, c);
        r->normalize();
        return r;
    }

    public override Str toStr()
    {
        return "Sphere {" + c->toStr() + "," + r + "}";
    }

    public Obj? getCenter()
    {
        return c;
    }
    public Void setCenter(Obj? c)
    {
        this->c = c;
    }
}

public class Scene
{
    public Obj? lights;
    public Obj? objects;
    private Obj? view;

    new make()
    {
        this->lights = [,];
        this->objects = [,];
    }

    public Void addLight(Obj? l)
    {
        this->lights->add(l);
    }

    public Void addObject(Obj? myobject)
    {
        this->objects->add(myobject);
    }

    public Void setView(Obj? view)
    {
        this->view = view;
    }

    public Obj? getView()
    {
        return this->view;
    }

    public Obj? getLight(Obj? number)
    {
        return this->lights->get(number);
    }

    public Obj? getObject(Obj? number)
    {
        return objects->get(number);
    }

    public Obj? getLights()
    {
        return this->lights->size;
    }

    public Obj? getObjects()
    {
        return this->objects->size;
    }

    public Void setObject(Obj? myobject, Obj? pos)
    {
        this->objects->set(pos, myobject);
    }
}


  public class RayTracer
    {
        public Obj? scene;
        public Obj? lights;
        public Obj? prim;
        public Obj? view;
        public Obj? tRay;
        public Obj? alpha;
        public Obj? voidVec;
        public Obj? L;
        public Obj? inter;
        public Obj? height;
        public Obj? width;
        public Obj? datasizes;
        public Obj? checksum;
        public Obj? size;
        public Obj? numobjects;       
      
        new make()
        {
          datasizes = Int?[,];
          datasizes->size = 10;
          datasizes->set(0, 5);  
          datasizes->set(1, 10);        
          datasizes->set(2, 15);            
          datasizes->set(3, 20);        
          datasizes->set(4, 25);
          datasizes->set(5, 30);  
          datasizes->set(6, 35);        
          datasizes->set(7, 40);            
          datasizes->set(8, 45);        
          datasizes->set(9, 50);
          
          alpha = 255.shiftl(24);
          voidVec = Vec();
          
          tRay = Ray();
          
          L = Vec();
          inter = Isect();
          checksum = 0;
        }
  
    public Obj? createScene()
    {
        Obj? x;
        x = 0.0f;
        Obj? y;
        y = 0.0f;
        Obj? scene;
        scene = Scene();
        Obj? p;
        Obj? nx;
        nx = 4.0f;
        Obj? ny;
        ny = 4.0f;
        Obj? nz;
        nz = 4.0f;
        Obj? i;
        i = 0.0f;     
        while (i < nx)
        {
            Obj? j;
            j = 0.0f;
            while (j < ny)
            {
                Obj? k;
                k = 0.0f;
                while (k < nz)
                {
                    Obj? xx;
                    xx = 20.0f->div(nx->minus(1.0f))->mult(i)->minus(10.0f);
                    Obj? yy;
                    yy = 20.0f->div(ny->minus(1.0f))->mult(j)->minus(10.0f);
                    Obj? zz;
                    zz = 20.0f->div(nz->minus(1.0f))->mult(k)->minus(10.0f);                         
                    p = Sphere(Vec(xx, yy, zz), 3.0f);            
                    p->setColor(0.0f, 0.0f, (((Float)(i->plus(j))) / ((Float)(nx->plus(ny)->minus(2.0f)))));
                    p->surf->shine = 15.0f;
                    p->surf->ks = 1.5f - 1.0f;
                    p->surf->kt = 1.5f - 1.0f;
                    scene->addObject(p);            
                    k = k->plus(1.0f);
                }
                j = j->plus(1.0f);
            }
            i = i->plus(1.0f);
        }
        scene->addLight(Light(100.0f, 100.0f, -50.0f, 1.0f));
        scene->addLight(Light(-100.0f, 100.0f, -50.0f, 1.0f));
        scene->addLight(Light(100.0f, -100.0f, -50.0f, 1.0f));
        scene->addLight(Light(-100.0f, -100.0f, -50.0f, 1.0f));
        scene->addLight(Light(200.0f, 200.0f, 0.0f, 1.0f));
        Obj? v;
        v = View(Vec(x, 20.0f, -30.0f), Vec(x, y, 0.0f), Vec(0.0f, 1.0f, 0.0f), 1.0f, 35.0f * 3.14159265f / 180.0f, 1.0f);
        scene->setView(v);
        return scene;
    }
  
    public Void setScene(Obj? scene)
    {     
      Obj? nLights;
      nLights = scene->getLights();
      Obj? nObjects;
      nObjects = scene->getObjects();
      //lights = Light[nLights];
      lights = [,];
      lights->size = nLights;
      //prim = Primitive[nObjects];  
      prim = [,];
      prim->size = nObjects;
      Obj? l;
      l = 0;      
      while(l < nLights)
      {
        lights->set(l, scene->getLight(l));     
        l = l->plus(1);
      }
      Obj? o;
      o = 0;
      while (o < nObjects)
      {
        prim->set(o, scene->getObject(o));        
        o = o->plus(1);
      }
      view = scene->getView();      
    } 
   
    public Obj? intersect(Obj? r, Obj? tmax)
    {     
      Obj? nhits;
      nhits = 0;
      Obj? temp;
      temp = Isect();             
      temp->t = 1e9f;  
      this->inter = temp;         
      Obj? i;
      i = 0;
      while (i < this->prim->size)
      {
        Obj? tp;
        tp = this->prim->get(i)->intersect(r);                             
        Obj? sw;
        sw = (tp != null);         
        if(sw)
        {                   
          Obj? isect;
          isect = Isect();              
          isect->t = tp->t;                             
          isect->prim = tp->prim;
          isect->surf = tp->surf;
          isect->enter = tp->enter;                           
          this->inter = isect;              
          nhits = nhits->plus(1);                      
        }
        i = i->plus(1);
      }
      return nhits > 0 ? true : false;
    }


    public Obj? trace(Obj? level, Obj? weight, Obj? r)
    {                        
      if (level > 6)
        return Vec();

      Obj? hit;
      hit = intersect(r, 1e6f);
      if (hit)
      {
        Obj? P;
        P = r->point(inter->t);
        Obj? N;
        N = inter->prim->normal(P);
        if (Vec.dot(r->D, N) >= 0.0f)
          N->negate();
        //return shade(level, weight, P, N, r->D, inter);
      }
      return voidVec;
    }
    
    public Obj? SpecularDirection(Obj? I, Obj? N)
    {
      Obj? dotIN;
      dotIN = Vec.dot(I, N);
      Obj? abs;
      abs = dotIN > 0 ? dotIN : ((-1)->mult(dotIN));
      Obj? r;
      r = Vec.comb(1.0f->div(abs), I, 2.0f, N);
      r->normalize();
      return r;
    }

    public Obj? TransDir(Obj? m1, Obj? m2, Obj? I, Obj? N)
    {            
      Obj? n1;
      n1 = m1 == null ? 1.0f : m1->ior;
      Obj? n2;
      n2 = m2 == null ? 1.0f : m2->ior;
      Obj? eta;
      eta = n1->div(n2);
      Obj? c1;
      c1 = (Vec.dot(I, N))->mult(-1);
      Obj? cs2;
      cs2 = 1.0f->minus(eta->mult(eta)->mult(1.0f->minus(c1->mult(c1))));
      if (cs2 < 0.0f)
          return null;
      Obj? r;
      r = Vec.comb(eta, I, eta->mult(c1)->minus(cs2->sqrt()), N);
      r->normalize();
      return r;
    }
    
    public Obj? Shadow(Obj? r, Obj? tmax)
    {
      if (intersect(r,tmax))
          return 0;
      return 1;
    }
    
    public Void render(Obj? interval)
    {     
      Obj? localView;
      localView = this->view;     
      Obj? viewVec;
      viewVec = Vec.sub1(localView->atVec, localView->fromVec);                      
      viewVec->normalize();     
      Obj? tmpVec;
      tmpVec = Vec(viewVec->x, viewVec->y, viewVec->z);     
      Obj? dotValue;
      dotValue = Vec.dot(localView->upVec, viewVec);     
      tmpVec->scale(dotValue);      
      Obj? upVec;
      upVec = Vec.sub1(localView->upVec, tmpVec);
      upVec->normalize();
      Obj? leftVec;
      leftVec = Vec.cross(localView->upVec, viewVec);
      leftVec->normalize();
      Obj? frustrumwidth;
      frustrumwidth = localView->dist->mult(localView->angle->tan());            
      upVec->scale(frustrumwidth->mult(-1.0f));     	  
      Obj? scaleValue;
      scaleValue = localView->aspect->mult(frustrumwidth);
      leftVec->scale(scaleValue);   	    
      Obj? r;
      r = Ray(localView->fromVec, this.voidVec);	  
      Obj? y;
      y = interval->yfrom;      
      Obj? nhits;
      nhits = 0;      
      while (y < interval->yto)
      {       			 
        Obj? ylen;
        ylen = 2.0f->mult(y->toFloat())->div(interval->width->toFloat())->minus(1.0f);				
        Obj? x;
        x = 0;
        while (x < interval->width)
        {               
          Obj? xlen;
          xlen = (2.0f->mult(x->toFloat()))->div(interval->width->toFloat())->minus(1.0f);         		  
          r->D = Vec.comb(xlen, leftVec, ylen, upVec);               
          r->D->add1(viewVec);                  
          r->D->normalize();                                
          Obj? hit;
          hit = this->intersect(r, 1e6f);                    
          if(hit)
            nhits = nhits->plus(1);
          x = x->plus(1);
        }
          y = y->plus(1);
      }     
      checksum = checksum->plus(nhits);      
    }  
}
    
    
public class JGFRayTracerBench : RayTracer
{
    public Void JGFsetsize(Obj? size)
    {
        this.size = size;
    }

    public Void JGFinitialise()
    {
        width = height = datasizes->get(size);
        scene = createScene();
        setScene(scene);
        numobjects = scene->getObjects();
    }

    public Void JGFapplication()
    {
        Obj? interval;
        interval = Interval(0, width, height, 0, height, 1);
        render(interval);
    }

    public Void JGFvalidate()
    {
        Obj? refval;
        refval = Int?[,];
        refval->size = 10;
        refval->set(0, 12);
        refval->set(1, 40);            
        refval->set(2, 104);            
        refval->set(3, 170);     
        refval->set(4, 278);            
        refval->set(5, 390);
        refval->set(6, 552);            
        refval->set(7, 692);            
        refval->set(8, 894);     
        refval->set(9, 1084);            
        Obj? dev;
        dev = checksum->minus(refval->get(this.size));
        if (dev != 0)
        {
            echo("Validation failed");
            echo("Pixel checksum = " + checksum);
            echo("Reference value = " + refval->get(size));
          }
      }

      public Void Test()
      {
          JGFsetsize(4);
          JGFinitialise();
          JGFapplication();
          JGFvalidate();            
      }
}  

public class Main
{
  public static Void main(Str[] args)
  {   
    if (args.size < 1)
    {   
      echo("You must pass the number of thousands iterations.");
    }
    else
    {   
      iterations := Int.fromStr(args[0]);   
      arith := ArithmethicBenchmark(iterations)
      echo(arith.run());
    }    
  } 
}