public class Chronometer 
{
  private Int ticks1;
  private Int ticks2;
  private Bool stopped;
  
  public Void start() 
  {
    this.ticks1 = Duration.nowTicks();
    this.stopped = false;
  }

  public Void stop() {
    this.ticks2 = Duration.nowTicks();
    this.stopped = true;
  }
  
  public Int getMicroSeconds() {
    return (this.ticks2 - this.ticks1)/1000;
  } 
}

public abstract class Benchmark 
{
  private Int iterations;
  protected Int microSeconds;
  
  new make(Int iterations) 
  {
    this.iterations = iterations;
  } 
  
  public Int run() {
    for(i := 0 ; i < this.iterations ; i++)
      this.runOneIteration();
    return this.microSeconds;
  }
  
  public virtual Obj? runOneIteration() {
    return null;
  }   
}

class Random
{
  public Int next()
  {
    return Int.random;
  }
  
  public Float nextFloat()
  {
    return Float.random;
  }
}

class ArithmethicBenchmark : Benchmark 
{
  new make(Int iterations): super(iterations) {}

  public override Obj? runOneIteration() 
  {
    chronometer := Chronometer(); 
    
    test := JGFSparseMatmultBench();
    
    chronometer.start();
    test.test();
    chronometer.stop();   
    this.microSeconds = this.microSeconds + chronometer.getMicroSeconds();
    return null;
  }
}

public abstract class TestPyBench {
        public abstract Void test();
    }


 public class JGFSparseMatmultBench : TestPyBench
  {
    Obj? size;
    
    //Atributos de clase con inicializacion estatica
    Obj?[]  datasizes_M;
    Obj?[]  datasizes_N;
    Obj?[]  datasizes_nz;
    Obj? SPARSE_NUM_ITER;
         
    //Atributos de instancia
    Obj? R;
    
    Obj? x;
    Obj? y;
    
    Obj? val;
    Obj? col;
    Obj? row;
    
    Obj? ytotal;
    
    new make()
    {
      ytotal = 0.0f;
      datasizes_M = Int?[,];
      datasizes_M->size = 3; 
      datasizes_M->set(0, 50000);
      datasizes_M->set(1, 100000);
      datasizes_M->set(2, 500000);
      datasizes_N = Int?[,];
      datasizes_N->size = 3;      
      datasizes_N->set(0, 50000);
      datasizes_N->set(1, 100000);
      datasizes_N->set(2, 500000);
      datasizes_nz = Int?[,];
      datasizes_nz->size = 3;
      datasizes_nz->set(0, 250000);
      datasizes_nz->set(1, 500000);
      datasizes_nz->set(2, 2500000);
      SPARSE_NUM_ITER = 10;
    }
    
    public override Void test() 
    {   
      JGFsetsize(0);
      JGFinitialise();
      JGFkernel();
      JGFvalidate();     
    }
    
    public Void JGFsetsize(Obj? size)
    {
      this.size = size;
    }
    
    public Void JGFinitialise()
    {      
      R = Random();

      x = RandomVector(datasizes_N->get(size), R);
      y = Float?[,];
      y->size = datasizes_M->get(size);
      for (i := 0; i < datasizes_M->get(size); i++)
      {
        y->set(i,0.0f);
      }
      val = Float?[,];
      val->size = datasizes_nz->get(size);
      col = Int?[,];
      col->size = datasizes_nz->get(size);
      row = Int?[,];
      row->size = datasizes_nz->get(size);
      
      for (Obj? i := 0; i < datasizes_nz->get(size); i = i->plus(1))
      {
        Obj? temp1;
        temp1 = R->next()->mod(datasizes_M->get(size));
        Obj? temp2;
        temp2 = R->next()->mod(datasizes_N->get(size));                            
        row->set(i, temp1 >= 0 ? temp1 : temp1->mult(-1));
        col->set(i, temp2 >= 0 ? temp2 : temp2->mult(-1));
        val->set(i, R->nextFloat());                   
      }
    }
    
    private static Obj? RandomVector(Obj? N, Obj? R)
    {
      Obj? a;
      a = Float?[,];
      a->size = N;
      for (Obj? i := 0; i < N; i = i->plus(1))
        a->set(i, R->nextFloat()->mult(0.000001f));                
      return a;
    }
  
    public Void JGFkernel()
    {       
      for (Obj? reps := 0; reps < SPARSE_NUM_ITER; reps = reps->plus(1))
      {      
        for (Obj? i := 0; i < val->size ; i = i->plus(1))
        {    
          y->set(row->get(i), y->get(row->get(i))->plus(x->get(col->get(i))->mult(val->get(i))));          
        }
      }          
      for (Obj? i := 0; i < val->size ; i = i->plus(1))
      {
        ytotal = ytotal->plus(y->get(row->get(i)));                    
      }     
    }
  
    public Void JGFvalidate()
    {
      Obj? refval;
      refval = Float?[,];
      refval->size = 3;
      refval->set(0, 75.02484945753453f);
      refval->set(1, 150.0130719633895f);
      refval->set(2, 749.5245870753752f);
      Obj? dev;
      dev = ytotal->minus(refval->get(size));
      dev = (dev >= 0.0f ? dev : dev->mult(-1.0f));
      if (dev > 0.000000000001f)
      {
        //echo("Validation failed");
        //echo("ytotal = " + ytotal + "  " + dev + "  " + size);
      }
    }    
  }

public class Main
{
  public static Void main(Str[] args)
  {   
    if (args.size < 1)
    {   
      echo("You must pass the number of thousands iterations.");
    }
    else
    {   
      iterations := Int.fromStr(args[0]);   
      arith := ArithmethicBenchmark(iterations)
      echo(arith.run());
    }    
  } 
}