public class Chronometer 
{
  private Int ticks1;
  private Int ticks2;
  private Bool stopped;
  
  public Void start() 
  {
    this.ticks1 = Duration.nowTicks();
    this.stopped = false;
  }

  public Void stop() {
    this.ticks2 = Duration.nowTicks();
    this.stopped = true;
  }
  
  public Int getMicroSeconds() {
    return (this.ticks2 - this.ticks1)/1000;
  } 
}

public abstract class Benchmark 
{
  private Int iterations;
  protected Int microSeconds;
  
  new make(Int iterations) 
  {
    this.iterations = iterations;
  } 
  
  public Int run() {
    for(i := 0 ; i < this.iterations ; i++)
      this.runOneIteration();
    return this.microSeconds;
  }
  
  public virtual Obj? runOneIteration() {
    return null;
  }   
}


class ArithmethicBenchmark : Benchmark 
{
  new make(Int iterations): super(iterations) {}

  public override Obj? runOneIteration() 
  {
    chronometer := Chronometer();   
    test := SimpleFloatArithmetic();
      
    chronometer.start();
    test.test();
    chronometer.stop();   
    this.microSeconds = this.microSeconds + chronometer.getMicroSeconds();
    return null;
  }
}

public abstract class TestPyBench {
	public abstract Void test();
}

public class SimpleFloatArithmetic : TestPyBench 
{		
	public override Void test() 
	{
		Obj? a := 2.1f;
		Obj? b := 3.3332f;
		Obj? c := 3.14159f;
		for (i := 0; i < 120000; i++)
		{
			c = a->plus(b);
			c = b->plus(c);
			c = c->plus(a);
			c = a->plus(b);
			c = b->plus(c);

			c = c->minus(a);
			c = a->minus(b);
			c = b->minus(c);
			c = c->minus(a);
			c = b->minus(c);

			c = a->div(b);
			c = b->div(a);
			c = c->div(b);

			c = a->mult(b);
			c = b->mult(a);
			c = c->mult(b);

			c = a->div(b);
			c = b->div(a);
			c = c->div(b);

			a = 2.1f;
			b = 3.3332f;
			c = 3.14159f;

			c = a->plus(b);
			c = b->plus(c);
			c = c->plus(a);
			c = a->plus(b);
			c = b->plus(c);

			c = c->minus(a);
			c = a->minus(b);
			c = b->minus(c);
			c = c->minus(a);
			c = b->minus(c);

			c = a->div(b);
			c = b->div(a);
			c = c->div(b);

			c = a->mult(b);
			c = b->mult(a);
			c = c->mult(b);

			c = a->div(b);
			c = b->div(a);
			c = c->div(b);

			a = 2.1f;
			b = 3.3332f;
			c = 3.14159f;

			c = a->plus(b);
			c = b->plus(c);
			c = c->plus(a);
			c = a->plus(b);
			c = b->plus(c);

			c = c->minus(a);
			c = a->minus(b);
			c = b->minus(c);
			c = c->minus(a);
			c = b->minus(c);

			c = a->div(b);
			c = b->div(a);
			c = c->div(b);

			c = a->mult(b);
			c = b->mult(a);
			c = c->mult(b);

			c = a->div(b);
			c = b->div(a);
			c = c->div(b);

			a = 2.1f;
			b = 3.3332f;
			c = 3.14159f;

			c = a->plus(b);
			c = b->plus(c);
			c = c->plus(a);
			c = a->plus(b);
			c = b->plus(c);

			c = c->minus(a);
			c = a->minus(b);
			c = b->minus(c);
			c = c->minus(a);
			c = b->minus(c);

			c = a->div(b);
			c = b->div(a);
			c = c->div(b);

			c = a->mult(b);
			c = b->mult(a);
			c = c->mult(b);

			c = a->div(b);
			c = b->div(a);
			c = c->div(b);

			a = 2.1f;
			b = 3.3332f;
			c = 3.14159f;

			c = a->plus(b);
			c = b->plus(c);
			c = c->plus(a);
			c = a->plus(b);
			c = b->plus(c);

			c = c->minus(a);
			c = a->minus(b);
			c = b->minus(c);
			c = c->minus(a);
			c = b->minus(c);

			c = a->div(b);
			c = b->div(a);
			c = c->div(b);

			c = a->mult(b);
			c = b->mult(a);
			c = c->mult(b);

			c = a->div(b);
			c = b->div(a);
			c = c->div(b);
		}
	}
}
        
    


public class Main
{
  public static Void main(Str[] args)
  {   
    if (args.size < 1)
    {   
      echo("You must pass the number of thousands iterations.");
    }
    else
    {   
      iterations := Int.fromStr(args[0]);   
      arith := ArithmethicBenchmark(iterations)
      echo(arith.run());
    }    
  } 
}