public class Chronometer 
{
  private Int ticks1;
  private Int ticks2;
  private Bool stopped;
  
  public Void start() 
  {
    this.ticks1 = Duration.nowTicks();
    this.stopped = false;
  }

  public Void stop() {
    this.ticks2 = Duration.nowTicks();
    this.stopped = true;
  }
  
  public Int getMicroSeconds() {
    return (this.ticks2 - this.ticks1)/1000;
  } 
}

public abstract class Benchmark 
{
  private Int iterations;
  protected Int microSeconds;
  
  new make(Int iterations) 
  {
    this.iterations = iterations;
  } 
  
  public Int run() {
    for(i := 0 ; i < this.iterations ; i++)
      this.runOneIteration();
    return this.microSeconds;
  }
  
  public virtual Obj? runOneIteration() {
    return null;
  }   
}


class ArithmethicBenchmark : Benchmark 
{
  new make(Int iterations): super(iterations) {}

  public override Obj? runOneIteration() 
  {
    chronometer := Chronometer(); 
    
    test := PythonFunctionCalls();
    
    chronometer.start();
    test.test();
    chronometer.stop();   
    this.microSeconds = this.microSeconds + chronometer.getMicroSeconds();
    return null;
  }
}

public abstract class TestPyBench {
        public abstract Void test();
    }



public class PythonFunctionCalls : TestPyBench 
{   
  Void f() { }

  Void f1(Obj? x) { }

  Obj? g(Obj? a, Obj? b, Obj? c)
  {    
    Obj?[] result := Obj?[3];
    result[0] = a;
    result[0] = b;
    result[0] = c;
    return  result;
  }

  Obj? h(Obj? a, Obj? b, Obj? c, Obj? d, Obj? e, Obj? f)
  {
    Obj?[] result := Obj?[3];
    result[0] = d;
    result[0] = e;
    result[0] = f;
    return  result;
  }
    
  public override Void test() 
	{    
    Obj? two := 2;
    Obj? three := 3;        
    for ( i := 0; i < 60000; i++) 
    {       
      f();
      f1(i);
      f1(i);
      f1(i);
      f1(i);
      g(i, i, i);
      g(i, i, i);
      g(i, i, i);
      g(i, i, i);
      h(i, i, three, i, i, i);
      h(i, i, i, two, i, three);

      f();
      f1(i);
      f1(i);
      f1(i);
      f1(i);
      g(i, i, i);
      g(i, i, i);
      g(i, i, i);
      g(i, i, i);
      h(i, i, three, i, i, i);
      h(i, i, i, two, i, three);

      f();
      f1(i);
      f1(i);
      f1(i);
      f1(i);
      g(i, i, i);
      g(i, i, i);
      g(i, i, i);
      g(i, i, i);
      h(i, i, three, i, i, i);
      h(i, i, i, two, i, three);

      f();
      f1(i);
      f1(i);
      f1(i);
      f1(i);
      g(i, i, i);
      g(i, i, i);
      g(i, i, i);
      g(i, i, i);
      h(i, i, three, i, i, i);
      h(i, i, i, two, i, three);

      f();
      f1(i);
      f1(i);
      f1(i);
      f1(i);
      g(i, i, i);
      g(i, i, i);
      g(i, i, i);
      g(i, i, i);
      h(i, i, three, i, i, i);
      h(i, i, i, two, i, three);
    }
  }
}



public class Main
{
  public static Void main(Str[] args)
  {   
    if (args.size < 1)
    {   
      echo("You must pass the number of thousands iterations.");
    }
    else
    {   
      iterations := Int.fromStr(args[0]);   
      arith := ArithmethicBenchmark(iterations)
      echo(arith.run());
    }    
  } 
}