public class Chronometer 
{
  private Int ticks1;
  private Int ticks2;
  private Bool stopped;
  
  public Void start() 
  {
    this.ticks1 = Duration.nowTicks();
    this.stopped = false;
  }

  public Void stop() {
    this.ticks2 = Duration.nowTicks();
    this.stopped = true;
  }
  
  public Int getMicroSeconds() {
    return (this.ticks2 - this.ticks1)/1000;
  } 
}

public abstract class Benchmark 
{
  private Int iterations;
  protected Int microSeconds;
  
  new make(Int iterations) 
  {
    this.iterations = iterations;
  } 
  
  public Int run() {
    for(i := 0 ; i < this.iterations ; i++)
      this.runOneIteration();
    return this.microSeconds;
  }
  
  public virtual Obj? runOneIteration() {
    return null;
  }   
}


class ArithmethicBenchmark : Benchmark 
{
  new make(Int iterations): super(iterations) {}

  public override Obj? runOneIteration() 
  {
    chronometer := Chronometer(); 
    
    test := SimpleDictManipulation();
    
    chronometer.start();
    test.test();
    chronometer.stop();   
    this.microSeconds = this.microSeconds + chronometer.getMicroSeconds();
    return null;
  }
}

public abstract class TestPyBench {
        public abstract Void test();
    }
    
    public class SimpleDictManipulation : TestPyBench
    {   
      
      private Obj? HasKey(Obj? d, Obj? key)
      {
          if (key >= d->size)
              return false;
          return d->containsKey(key);
      }
      
      public override Void test() 
      { 
            Obj? temp;
            Obj? zero;
            Obj? one;
            Obj? two;
            Obj? three;
            Obj? four;
            Obj? five;
            Obj? six;
            Obj? eight;
            Obj? ten;
        
            zero = 0;
            one = 1;
            two = 2;
            three = 3;
            four = 4;
            five = 5;
            six = 6;
            eight = 8
            ten = 10
            
            Obj? d;            
            d = [:];
            
            for (i := 0; i < 100000; i = i->plus(1))
            {       
              d->set(0,three);
              d->set(1,four);
              d->set(2,five);
              d->set(3,three);
              d->set(4,four);
              d->set(5,five);
              
              temp = d->get(0);
              temp = d->get(1);
              temp = d->get(2);
              temp = d->get(3);
              temp = d->get(4);
              temp = d->get(5);
        
              HasKey(d,zero);
              HasKey(d,two);
              HasKey(d,four);
              HasKey(d,six);
              HasKey(d,eight);
              HasKey(d,ten);
              
              d->set(0,zero);
              d->set(1,zero);
              d->set(2,zero);
              d->set(3,zero);
              d->set(4,zero);
              d->set(5,zero);              

              d->set(0,three);
              d->set(1,four);
              d->set(2,five);
              d->set(3,three);
              d->set(4,four);
              d->set(5,five);
              
              temp = d->get(0);
              temp = d->get(1);
              temp = d->get(2);
              temp = d->get(3);
              temp = d->get(4);
              temp = d->get(5);
        
              HasKey(d,zero);
              HasKey(d,two);
              HasKey(d,four);
              HasKey(d,six);
              HasKey(d,eight);
              HasKey(d,ten);
              
              d->set(0,zero);
              d->set(1,zero);
              d->set(2,zero);
              d->set(3,zero);
              d->set(4,zero);
              d->set(5,zero);              
              
              d->set(0,three);
              d->set(1,four);
              d->set(2,five);
              d->set(3,three);
              d->set(4,four);
              d->set(5,five);
              
              temp = d->get(0);
              temp = d->get(1);
              temp = d->get(2);
              temp = d->get(3);
              temp = d->get(4);
              temp = d->get(5);
        
              HasKey(d,zero);
              HasKey(d,two);
              HasKey(d,four);
              HasKey(d,six);
              HasKey(d,eight);
              HasKey(d,ten);
              
              d->set(0,zero);
              d->set(1,zero);
              d->set(2,zero);
              d->set(3,zero);
              d->set(4,zero);
              d->set(5,zero);              
              
              d->set(0,three);
              d->set(1,four);
              d->set(2,five);
              d->set(3,three);
              d->set(4,four);
              d->set(5,five);
              
              temp = d->get(0);
              temp = d->get(1);
              temp = d->get(2);
              temp = d->get(3);
              temp = d->get(4);
              temp = d->get(5);
        
              HasKey(d,zero);
              HasKey(d,two);
              HasKey(d,four);
              HasKey(d,six);
              HasKey(d,eight);
              HasKey(d,ten);
              
              d->set(0,zero);
              d->set(1,zero);
              d->set(2,zero);
              d->set(3,zero);
              d->set(4,zero);
              d->set(5,zero);              
              
              d->set(0,three);
              d->set(1,four);
              d->set(2,five);
              d->set(3,three);
              d->set(4,four);
              d->set(5,five);
              
              temp = d->get(0);
              temp = d->get(1);
              temp = d->get(2);
              temp = d->get(3);
              temp = d->get(4);
              temp = d->get(5);
        
              HasKey(d,zero);
              HasKey(d,two);
              HasKey(d,four);
              HasKey(d,six);
              HasKey(d,eight);
              HasKey(d,ten);      
              
              d->set(0,zero);
              d->set(1,zero);
              d->set(2,zero);
              d->set(3,zero);
              d->set(4,zero);
              d->set(5,zero);                            
            }
       }
    }    

public class Main
{
  public static Void main(Str[] args)
  {   
    if (args.size < 1)
    {   
      echo("You must pass the number of thousands iterations.");
    }
    else
    {   
      iterations := Int.fromStr(args[0]);   
      arith := ArithmethicBenchmark(iterations)
      echo(arith.run());
    }    
  } 
}