public class Chronometer 
{
  private Int ticks1;
  private Int ticks2;
  private Bool stopped;
  
  public Void start() 
  {
    this.ticks1 = Duration.nowTicks();
    this.stopped = false;
  }

  public Void stop() {
    this.ticks2 = Duration.nowTicks();
    this.stopped = true;
  }
  
  public Int getMicroSeconds() {
    return (this.ticks2 - this.ticks1)/1000;
  } 
}

public abstract class Benchmark 
{
  private Int iterations;
  protected Int microSeconds;
  
  new make(Int iterations) 
  {
    this.iterations = iterations;
  } 
  
  public Int run() {
    for(i := 0 ; i < this.iterations ; i++)
      this.runOneIteration();
    return this.microSeconds;
  }
  
  public virtual Obj? runOneIteration() {
    return null;
  }   
}


class ArithmethicBenchmark : Benchmark 
{
  new make(Int iterations): super(iterations) {}

  public override Obj? runOneIteration() 
  {
    chronometer := Chronometer(); 
    
    test := ListSlicing();
    
    chronometer.start();
    test.test();
    chronometer.stop();   
    this.microSeconds = this.microSeconds + chronometer.getMicroSeconds();
    return null;
  }
}

public abstract class TestPyBench {
        public abstract Void test();
    }

    
    public class ListSlicing : TestPyBench
    {   
      
               
      public override Void test() 
      { 
          Obj? n;
          n = [,];
          for(i := 0 ; i < 100; i=i->plus(1))          
            n->add(i);          
          Obj? r;
          r = [,];
          for(i := 0 ; i < 25; i=i->plus(1))          
            r->add(i);
          Obj? l;
          Obj? m;
          for(i := 0 ; i < 800; i=i->plus(1))
          {
                l = n->getRange(0..99);     
                for(j := 0; j < r->size(); j=j->plus(1))
                {                       
                      m = l->getRange(50..99);
                      m = l->getRange(75..99);   
                      m = l->getRange(50..55);                  
                      l->addAll(n->getRange(3..99));
                      m = l->getRange(0..98);
                      m = l->getRange(1..98);
                      l->removeAt(99);
                      l->addAll(n->getRange(0..99));                  
                }
          }            
      }
    }    

public class Main
{
  public static Void main(Str[] args)
  {   
    if (args.size < 1)
    {   
      echo("You must pass the number of thousands iterations.");
    }
    else
    {   
      iterations := Int.fromStr(args[0]);   
      arith := ArithmethicBenchmark(iterations)
      echo(arith.run());
    }    
  } 
}