public class Chronometer 
{
  private Int ticks1;
  private Int ticks2;
  private Bool stopped;
  
  public Void start() 
  {
    this.ticks1 = Duration.nowTicks();
    this.stopped = false;
  }

  public Void stop() {
    this.ticks2 = Duration.nowTicks();
    this.stopped = true;
  }
  
  public Int getMicroSeconds() {
    return (this.ticks2 - this.ticks1)/1000;
  } 
}

public abstract class Benchmark 
{
  private Int iterations;
  protected Int microSeconds;
  
  new make(Int iterations) 
  {
    this.iterations = iterations;
  } 
  
  public Int run() {
    for(i := 0 ; i < this.iterations ; i++)
      this.runOneIteration();
    return this.microSeconds;
  }
  
  public virtual Obj? runOneIteration() {
    return null;
  }   
}


class ArithmethicBenchmark : Benchmark 
{
  new make(Int iterations): super(iterations) {}

  public override Obj? runOneIteration() 
  {
    chronometer := Chronometer(); 
    
    test := StringPredicates();
    
    chronometer.start();
    test.test();
    chronometer.stop();   
    this.microSeconds = this.microSeconds->plus(chronometer.getMicroSeconds());
    return null;
  }
}

public abstract class TestPyBench {
        public abstract Void test();
    }

    
    public class StringPredicates : TestPyBench
    {           
      public override Void test() 
      {             
            Obj? s;                       
            s = "";            
            for (i := 0; i < 100;  i=i+1)
                s = s->plus("" + i.toStr);
            
            Obj? zero;
            Obj? one;
            Obj? two;
            Obj? five;
            Obj? eleven;
            Obj? twentyfive;
            Obj? fifty;
            Obj? fiftyfive;
            
            zero = 0;
            one = 1;
            two = 2;
            five = 5;
            eleven = 11;
            twentyfive = 25;
            fifty = 50;    
            fiftyfive = 55;
        
            Obj? b;
            for (Obj? i := 0; i < 160000; i=i->plus(1))
            {                              
              b = s->getRange(fifty..s->size()->minus(one));
              b = s->getRange(zero..twentyfive);
              b = s->getRange(fifty..fiftyfive);
              b = s->getRange((s->size()->minus(two))..s->size()->minus(one));
              b = s->getRange(zero..one);              
              b = s->getRange(two..s->size()->minus(two));
              b = s->getRange(eleven..s->size()->minus(eleven));
              
              b = s->getRange(fifty..s->size()->minus(one));
              b = s->getRange(zero..twentyfive);
              b = s->getRange(fifty..fiftyfive);
              b = s->getRange((s->size()->minus(two))..s->size()->minus(one));
              b = s->getRange(zero..one);              
              b = s->getRange(two..s->size()->minus(two));
              b = s->getRange(eleven..s->size()->minus(eleven));
              
              b = s->getRange(fifty..s->size()->minus(one));
              b = s->getRange(zero..twentyfive);
              b = s->getRange(fifty..fiftyfive);
              b = s->getRange((s->size()->minus(two))..s->size()->minus(one));
              b = s->getRange(zero..one);              
              b = s->getRange(two..s->size()->minus(two));
              b = s->getRange(eleven..s->size()->minus(eleven));
              
              b = s->getRange(fifty..s->size()->minus(one));
              b = s->getRange(zero..twentyfive);
              b = s->getRange(fifty..fiftyfive);
              b = s->getRange((s->size()->minus(two))..s->size()->minus(one));
              b = s->getRange(zero..one);              
              b = s->getRange(two..s->size()->minus(two));
              b = s->getRange(eleven..s->size()->minus(eleven));
              
              b = s->getRange(fifty..s->size()->minus(one));
              b = s->getRange(zero..twentyfive);
              b = s->getRange(fifty..fiftyfive);
              b = s->getRange((s->size()->minus(two))..s->size()->minus(one));
              b = s->getRange(zero..one);              
              b = s->getRange(two..s->size()->minus(two));
              b = s->getRange(eleven..s->size()->minus(eleven));                           
            }
       }
    }    

public class Main
{
  public static Void main(Str[] args)
  {   
    if (args.size < 1)
    {   
      echo("You must pass the number of thousands iterations.");
    }
    else
    {   
      iterations := Int.fromStr(args[0]);   
      arith := ArithmethicBenchmark(iterations)
      echo(arith.run());
    }    
  } 
}