public class Chronometer 
{
  private Int ticks1;
  private Int ticks2;
  private Bool stopped;
  
  public Void start() 
  {
    this.ticks1 = Duration.nowTicks();
    this.stopped = false;
  }

  public Void stop() {
    this.ticks2 = Duration.nowTicks();
    this.stopped = true;
  }
  
  public Int getMicroSeconds() {
    return (this.ticks2 - this.ticks1)/1000;
  } 
}

public abstract class Benchmark 
{
  private Int iterations;
  protected Int microSeconds;
  
  new make(Int iterations) 
  {
    this.iterations = iterations;
  } 
  
  public Int run() {
    for(i := 0 ; i < this.iterations ; i++)
      this.runOneIteration();
    return this.microSeconds;
  }
  
  public virtual Obj? runOneIteration() {
    return null;
  }   
}



public class Node
{
  public Obj? data;
  public Obj? next;
  
  new make(Obj? data, Obj? next)
  {
    this.data = data;
    this.next = next;
  }
}

public class Point3D
{
  public Obj? x;
  public Obj? y;
  public Obj? z;   
  public Obj? dimensions;
  
  new make(Obj? x, Obj? y, Obj? z, Obj? dimensions)
  {
    this.x = x;
    this.y = y;
    this.z = z; 
    this.dimensions = dimensions;     
  }
}

public class Point2D
{   
  public Obj? x;
  public Obj? y;   
  public Obj? dimensions;
  
  new make(Obj? x, Obj? y, Obj? dimensions)
  {
    this.x = x;
    this.y = y;     
    this.dimensions = dimensions;
  }
}


class PointsBench : Benchmark 
{
  new make(Int iterations): super(iterations) {}

  public override Obj? runOneIteration() 
  {
    chronometer := Chronometer(); 
    test := PointsTest();
    chronometer.start();
    test.Test();
    chronometer.stop();   
    this.microSeconds = this.microSeconds + chronometer.getMicroSeconds();
    return null;
  }
}

public class PointsTest
{ 
  Obj? createPoint(Obj? dimensions, Obj? x, Obj? y, Obj? z) 
  {
    Obj? point;
    if (dimensions==2)
      point = Point2D(x,y,dimensions);
    else
      point = Point3D(x,y,z,3);
    return point;
  }

  
  Obj? createPoints(Obj? number) {
    Obj? i;
    Obj? list;
    Obj? point;

    i = 1;
    point = createPoint(3,0,0,0);
    list = Node(point, 0); 
    while(i < number) 
    {
      point = createPoint(i->mod(2)->plus(2), number->div(2)->minus(i), i, i);
      list = Node(point, list);
      i = i->plus(1);
    }
    return list;
  }
  
  
  Obj? positiveX(Obj? list, Obj? n) {
    Obj? i;
    Obj? l;
    Obj? result;
    result = i = 0;
    l = list;
    while(i < n)
    {
      if(l->data->x >= 0)
        result = Node(l->data,result);
      l = l->next;
      i = i->plus(1);
    }
    return result;
  }

  Obj? distance3D(Obj? point) 
  {
    Obj? value;
    value = 2147483647;       
    if (point->dimensions == 3)
      value = point->x->mult(point->x)->plus(point->y->mult(point->y))->plus(point->z->mult(point->z));
    return value;
  }

  Obj? closestToOrigin3D(Obj? list, Obj? n) 
  {
    Obj? i; Obj? minDistance;
    Obj? l; Obj? point3D;
    
    point3D = createPoint(3,0,0,0);
    minDistance = 2147483647;
    l = list;
    i = 0;
    while(i<n) {  
      if(distance3D(l->data)<minDistance) 
      {
        minDistance = distance3D(l->data);
        point3D = l->data;
      }
      l = l->next;
      i = i->plus(1);
    }
    return point3D;  
  }

  public Void Test() 
  {               
    Obj? numberOfPoints;
    Obj? list; Obj? positive; Obj? point;          
    numberOfPoints = 10000;
    list = createPoints(numberOfPoints);      
    positive = positiveX(list, numberOfPoints);  
    point = closestToOrigin3D(list, numberOfPoints);
  }
}


public class Main
{
  public static Void main(Str[] args)
  {   
    if (args.size < 1)
    {   
      echo("You must pass the number of thousands iterations.");
    }
    else
    {   
      iterations := Int.fromStr(args[0]);   
      pointsBench := PointsBench(iterations)
      echo(pointsBench.run());
    }    
  } 
}
