public class Chronometer 
{
  private Int ticks1;
  private Int ticks2;
  private Bool stopped;
  
  public Void start() 
  {
    this.ticks1 = Duration.nowTicks();
    this.stopped = false;
  }

  public Void stop() {
    this.ticks2 = Duration.nowTicks();
    this.stopped = true;
  }
  
  public Int getMicroSeconds() {
    return (this.ticks2 - this.ticks1)/1000;
  } 
}

public abstract class Benchmark 
{
  private Int iterations;
  protected Int microSeconds;
  
  new make(Int iterations) 
  {
    this.iterations = iterations;
  } 
  
  public Int run() {
    for(i := 0 ; i < this.iterations ; i++)
      this.runOneIteration();
    return this.microSeconds;
  }
  
  public virtual Obj? runOneIteration() {
    return null;
  }   
}


class ArithmethicBenchmark : Benchmark 
{
  new make(Int iterations): super(iterations) {}

  public override Obj? runOneIteration() 
  {
    chronometer := Chronometer();   
    pystone := Pystone();    
    chronometer.start();
    value := pystone.Proc0(5000);
    chronometer.stop();   
    this.microSeconds = this.microSeconds + chronometer.getMicroSeconds();
    return value;
  }
}


public class Idents
{
  public static const Obj? Ident1 := 1;
  public static const Obj? Ident2 := 2;
  public static const Obj? Ident3 := 3;
  public static const Obj? Ident4 := 4;
  public static const Obj? Ident5 := 5;
}
    
public class Record
{
    public Obj? PtrComp;        
    public Obj? Discr;
    public Obj? EnumComp;
    public Obj? IntComp;
    public Obj? StringComp;

    new make(Obj? Discr, Obj? EnumComp, Obj? IntComp, Obj? StringComp)
    {            
        this.Discr = Discr;  
        this.EnumComp = EnumComp;
        this.IntComp = IntComp;
        this.StringComp = StringComp;
    }        

    public Obj? copy()
    {
        record := Record(this.Discr, this.EnumComp, this.IntComp, this.StringComp);
        record->PtrComp = this.PtrComp;
        return record;
    }
}
    
public class Pystone
{
    public Obj? IntGlob;
    public Obj? BoolGlob;
    public Obj? Char1Glob;
    public Obj? Char2Glob;
    public Obj?[] Array1Glob;
    public Obj?[] Array2Glob;
    public Obj? PtrGlb;
    public Obj? PtrGlbNext;    

    new make()
    {     
      this.IntGlob = 0;
      this.BoolGlob = false;
      this.Char1Glob = ' ';
      this.Char2Glob = ' ';
      this.Array1Glob = List(51);
      for (Int i := 0; i < 51; i++)
        this.Array1Glob.add(0);
      this.Array2Glob = List(51);
      for (Int i := 0; i < 51; i++)
      {
        row := List(51)
        for (Int j := 0; j < 51; j++)
          row.add(0);
        this.Array2Glob.add(row);
      }        
      this.PtrGlb = null;
      this.PtrGlbNext = null;
    }
              
    public Obj? pystones(Obj? loops)
    {            
      return this.Proc0(loops);
    }  
  
    public Obj? Proc0(Obj? loops)
    {
      benchtime := 0;
      nulltime := 0;
      String1Loc := "";
         
      chronometer := Chronometer(); 
      chronometer.start();
      for (Int i := 0; i < loops; i++)
        continue;
      chronometer.stop();
      nulltime = chronometer.getMicroSeconds();
      
      
      this.PtrGlbNext = Record(0, Idents.Ident1, 0, "");
      this.PtrGlb = Record(0, Idents.Ident1, 0, "");
      this.PtrGlb->PtrComp = this.PtrGlbNext;
      this.PtrGlb->Discr = Idents.Ident1;
      this.PtrGlb->EnumComp = Idents.Ident3;
      this.PtrGlb->IntComp = 40;
      this.PtrGlb->StringComp = "DHRYSTONE PROGRAM, SOME STRING";
      String1Loc = "DHRYSTONE PROGRAM, 1'ST STRING";
      this.Array2Glob->get(8)->set(7,10);
            
      chronometer.start();      
      for (Int i := 0; i < loops; i++)            
        this.DoWork();     
      chronometer.stop();
      benchtime = chronometer.getMicroSeconds();
      
      result := List(2);
      result.add(benchtime);
            
      loopsPerBenchtime := 0
      if(benchtime > 0)
        loopsPerBenchtime = ((Int)loops) / benchtime;      
      result.add(loopsPerBenchtime);     
      return result;
    }

   private Void DoWork()
    {
      Obj? EnumLoc := 0;            
      Obj? String1Loc := "DHRYSTONE PROGRAM, 1'ST STRING";
      Obj? String2Loc := "";
      Obj? IntLoc1 := 0;
      Obj? IntLoc2 := 0;
      Obj? IntLoc3 := 0;
      Obj? CharIndex := 'A';      
      
      Proc5();
      Proc4();
      IntLoc1 = 2;
      IntLoc2 = 3;
      String2Loc = "DHRYSTONE PROGRAM, 2'ND STRING";
      EnumLoc = Idents.Ident2;
      BoolGlob = !Func2(String1Loc, String2Loc);
      while (IntLoc1.compare(IntLoc2) < 0)
      {
        IntLoc3 = IntLoc1->mult(5)->minus(IntLoc2);
        IntLoc3 = Proc7(IntLoc1, IntLoc2);
        IntLoc1 = IntLoc1->plus(1);
      }
      Proc8(Array1Glob, Array2Glob, IntLoc1, IntLoc3);                
      PtrGlb = Proc1(PtrGlb);             
      CharIndex = 'A';              
      while (CharIndex.compare(Char2Glob) <= 0) {                                              
        if (EnumLoc == Func1(CharIndex, 'C')){            
          EnumLoc = Proc6(Idents.Ident1);
        }       
        CharIndex = CharIndex->plus(1);
      }     
      IntLoc3 = IntLoc2->mult(IntLoc1);
      IntLoc2 = IntLoc3->div(IntLoc1);
      IntLoc2 = (IntLoc3->minus(IntLoc2))->mult(7)->minus(IntLoc1);        
      IntLoc1 = Proc2(IntLoc1);
    }
  
    public Obj? Proc1(Obj? PtrParIn)
    {     
        Obj? NextRecord := PtrGlb->copy();   
        PtrParIn->PtrComp = NextRecord;      
        PtrParIn->IntComp = 5;     
        NextRecord->IntComp = PtrParIn->IntComp;      
        NextRecord->PtrComp = PtrParIn->PtrComp;      
        NextRecord->PtrComp = Proc3(NextRecord->PtrComp);     
        if (NextRecord->Discr == Idents.Ident1)
        {       
            NextRecord->IntComp = 6;       
            NextRecord->EnumComp = Proc6(PtrParIn->EnumComp);       
            NextRecord->PtrComp = this.PtrGlb->PtrComp;
            NextRecord->IntComp = Proc7(NextRecord->IntComp, 10);
        }
        else 
          PtrParIn = NextRecord->copy();
        NextRecord->PtrComp = null;
        return PtrParIn;
    }
  
    public Obj? Proc2(Obj? IntParIO) 
    {        
        Obj? EnumLoc := Idents.Ident5;
        Obj? IntLoc := IntParIO->plus(10);            
        while (true) 
        {
            if (this.Char1Glob == 'A') 
            {
                IntLoc = IntLoc->minus(1);
                IntParIO = IntLoc->minus(this.IntGlob);
                EnumLoc = Idents.Ident1;
            }
            if (EnumLoc == Idents.Ident1)
              return IntParIO;
        }
        return IntParIO;
    }
  
    public Obj? Proc3(Obj? PtrParOut)
    {
        if (PtrGlb != null)        
            PtrParOut = PtrGlb->PtrComp;        
        else 
          IntGlob = 100;
        PtrGlb->IntComp = Proc7(10, IntGlob);
        return PtrParOut;
    }  
  
    public Void Proc4()
    {
        Obj? BoolLoc := (Char1Glob == 'A');
        BoolLoc = (BoolLoc || BoolGlob);
        Char2Glob = 'B';
    }  
  
    public Void Proc5()
    {
      Char1Glob = 'A';
      BoolGlob = false;
    }
  
    public Obj? Proc6(Obj? EnumParIn)
    {        
        Obj? EnumParOut := EnumParIn;
        if (!Func3(EnumParIn))
            EnumParOut = Idents.Ident4;
        if (EnumParIn == Idents.Ident1)
            EnumParOut = Idents.Ident1;
        else
        {
            if (EnumParIn == Idents.Ident2)
            {
                if (IntGlob > 100) 
                  EnumParOut = Idents.Ident1;
                else
                  EnumParOut = Idents.Ident4;
            }
            else
            {
                if (EnumParIn == Idents.Ident3)
                    EnumParOut = Idents.Ident2;
                else
                {
                    if (EnumParIn == Idents.Ident4)
                    {
                    }
                    else
                    {
                        if (EnumParIn == Idents.Ident5)
                            EnumParOut = Idents.Ident3;
                    }
                }
            }
        }
        return EnumParOut;
    }
  
    public Obj? Proc7(Obj? IntParI1, Obj? IntParI2)
    {     
        Obj? IntLoc := IntParI1->plus(2);
        Obj? IntParOut := IntParI2->plus(IntLoc);
        return IntParOut;
    }
  
    public Void Proc8(Obj? Array1Par, Obj? Array2Par, Obj? IntParI1, Obj? IntParI2)
    {
      Obj? IntLoc := IntParI1->plus(5);
      Array1Par->set(IntLoc,IntParI2);
      Array1Par->set(IntLoc->plus(1),Array1Par->get(IntLoc));
      Array1Par->set(IntLoc->plus(30),IntLoc);
      for (Int IntIndex := IntLoc ; IntIndex < IntLoc->plus(2); IntIndex++)
          Array2Par->get(IntLoc)->set(IntIndex, IntLoc);  
      Array2Par->get(IntLoc)->set(IntLoc->minus(1),Array2Par->get(IntLoc)->get(IntLoc->minus(1))->plus(1));
      Array2Par->get(IntLoc->plus(20))->set(IntLoc,Array1Par->get(IntLoc));      
      this.IntGlob = 5;
    }  
  
    public static Obj? Func1(Obj? CharPar1, Obj? CharPar2)
    {                             
      Obj? CharLoc1 := CharPar1;
      Obj? CharLoc2 := CharLoc1;      
      if (CharLoc2 != CharPar2)
        return Idents.Ident1;
      else 
        return Idents.Ident2;
    }
  
    public Obj? Func2(Obj? StrParI1, Obj? StrParI2)
    {
      Obj? CharLoc := ' ';
      Obj? IntLoc := 1;
      while(IntLoc.compare(1) <= 0)
      {
          if (Func1(StrParI1->get(IntLoc), StrParI2->get(IntLoc->plus(1))) == Idents.Ident1)
          {
              CharLoc = 'A';
              IntLoc = IntLoc->plus(1);
          }
      }
      if ((CharLoc.compare('W') >= 0) && (CharLoc.compare('Z') <= 0))
          IntLoc = 7;
      if (CharLoc == 'X')
              return true;
      else
      {
          if (StrParI1.compare(StrParI2) > 0)
          {
              IntLoc = IntLoc->plus(7);
              return true;
          }
          else 
            return false;
      }         
    }
  
  
    public Obj? Func3(Obj? EnumParIn)
    {        
        Obj? EnumLoc := EnumParIn;
        if (EnumLoc == Idents.Ident3) return true;
        return false;
    }
}


public class Main
{
  public static Void main(Str[] args)
  {   
    if (args.size < 1)
    {   
      echo("You must pass the number of thousands iterations.");
    }
    else
    {   
      iterations := Int.fromStr(args[0]);   
      arith := ArithmethicBenchmark(iterations)
      echo(arith.run());
    }    
  } 
}