import sys, time, operator, os, random
from time import clock
from System import Array

TRUE = 1
FALSE = 0

class Chronometer:
	ticks1 = 0
	ticks2 = 0
	stopped = TRUE
	
	def start(self):
		self.ticks1 = time.time()
		self.stopped = FALSE

	def stop(self):
		self.ticks2 = time.time()
		self.stopped = TRUE

	def getSeconds(self):
		if self.stopped:
			return self.ticks2 - self.ticks1
		return time.time() - self.ticks1
		
	def getMicroSeconds(self):
		return int(self.getSeconds() * 1000000)

class BenchMark:
	microSeconds = 0
	def __init__(self,iterations):		
		self.iterations = iterations
	
	def run(self):		
		for i in range(self.iterations):
			self.runOneIteration()
		return self.microSeconds
	
	def runOneIteration(self):
		pass	

class ArithmethicBenchmark(BenchMark):
	def __init__(self,iterations):
		self.iterations = iterations
		
	def runOneIteration(self):				
		chronometer = Chronometer()				
		test = JGSparseMatmultBench()		
		chronometer.start()
		test.test()
		chronometer.stop()		
		self.microSeconds = self.microSeconds + chronometer.getMicroSeconds()
		return None
		
class Test:
	pass

class JGSparseMatmultBench(Test):
	datasizes_M = None
	datasizes_N = None
	datasizes_nz = None
	SPARSE_NUM_ITER = None
	ytotal = None
	
	def __init__(self):		
		JGSparseMatmultBench.datasizes_M = Array.CreateInstance(int, 3)
		JGSparseMatmultBench.datasizes_M[0] = 50000
		JGSparseMatmultBench.datasizes_M[1] = 100000
		JGSparseMatmultBench.datasizes_M[2] = 500000

		JGSparseMatmultBench.datasizes_N = Array.CreateInstance(int, 3)
		JGSparseMatmultBench.datasizes_N[0] = 50000
		JGSparseMatmultBench.datasizes_N[1] = 100000
		JGSparseMatmultBench.datasizes_N[2] = 500000

		JGSparseMatmultBench.datasizes_nz = Array.CreateInstance(int, 3)
		JGSparseMatmultBench.datasizes_nz[0] = 250000
		JGSparseMatmultBench.datasizes_nz[1] = 500000
		JGSparseMatmultBench.datasizes_nz[2] = 2500000
		
		JGSparseMatmultBench.SPARSE_NUM_ITER = 10
		JGSparseMatmultBench.ytotal = 0.0
		
		self.size = None
		self.R = None
		self.x = None
		self.y = None
		self.val = None
		self.col = None
		self.row = None
	
	def JGsetsize(self,size):
		self.size = size  

	def JGinitialise(self):
		self.R = random.Random(1010)
		
		self.x = self.RandomVector(JGSparseMatmultBench.datasizes_N[self.size], self.R)
		self.y = Array.CreateInstance(float, JGSparseMatmultBench.datasizes_M[self.size])
		
		self.val = Array.CreateInstance(float,JGSparseMatmultBench.datasizes_nz[self.size])
		self.col = Array.CreateInstance(int,JGSparseMatmultBench.datasizes_nz[self.size])
		self.row = Array.CreateInstance(int,JGSparseMatmultBench.datasizes_nz[self.size])

		for i in range(JGSparseMatmultBench.datasizes_nz[self.size]):
			temp1 = self.R.randint(0,int.MaxValue) % JGSparseMatmultBench.datasizes_M[self.size]
			temp2 = self.R.randint(0,int.MaxValue) % JGSparseMatmultBench.datasizes_N[self.size]
			self.row[i] = temp1 if temp1 >= 0 else -1 * temp1
			self.col[i] = temp2 if temp2 >= 0 else -1 * temp2
			self.val[i] = self.R.random()
			
	def JGFkernel(self):
		for reps in range(JGSparseMatmultBench.SPARSE_NUM_ITER):
			for i in range(self.val.Length):
				self.y[self.row[i]] += self.x[self.col[i]] * self.val[i]
		for i in range(self.val.Length):
			JGSparseMatmultBench.ytotal += self.y[self.row[i]]               

	def JGFvalidate(self):        
		refval = Array.CreateInstance(float, 3)
		refval[0] = 75.02484945753453
		refval[1] = 150.0130719633895
		refval[2] = 749.5245870753752
		dev = JGSparseMatmultBench.ytotal - refval[self.size]
		dev = dev if dev >= 0 else -1 * dev		
		if dev > 1.0e-12:
			'''print "Validation failed"'''
			'''print "ytotal = " + str(JGSparseMatmultBench.ytotal) + "  " + str(dev) + "  " + str(self.size)'''
       	
	def RandomVector(self,N,R):
		a = Array.CreateInstance(float, N)
		for i in range(N):		
			a[i] = R.random() * 1e-6
		return a
				
	def test(self):
		self.JGsetsize(0)
		self.JGinitialise()
		self.JGFkernel()
		self.JGFvalidate()		

if __name__ == '__main__':	
	if len(sys.argv) < 2: 
		print ("You must pass the number of thousands iterations.")
		sys.exit(-1)
	iterations = int(sys.argv[1])	
	arith = ArithmethicBenchmark(iterations)
	print (str(arith.run()))