import sys, time
from time import clock
TRUE = 1
FALSE = 0

class Node:	
	def __init__(self,data, next):
		self.data = data
		self.next = next
		
class Point3D:
	def __init__(self,x ,y,z,dimensions):
		self.x = x
		self.y = y
		self.z = z	
		self.dimensions = dimensions
		
class Point2D:
	def __init__(self,x,y,dimensions):
		self.x = x
		self.y = y		
		self.dimensions = dimensions
		
class PointsBech():
	def createPoint(self,dimensions, x, y, z):		
		if (dimensions == 2):
			point = Point2D(x,y,dimensions)
		else:
			point = Point3D(x,y,z,3)
		return point

	def createPoints(self,number):
		i = 1
		point = self.createPoint(3,0,0,0)
		list = Node(point, 0)
		while(i < number): 		
			point = self.createPoint(i % 2 + 2, number/2 - i, i, i)
			list = Node(point, list)
			i = i + 1
		return list

	def positiveX(self,list, n):		
		result = 0
		i = 0
		l = list
		while(i < n):			
			if(l.data.x >= 0):
				result = Node(l.data,result)
			l = l.next
			i = i + 1			
		return result
	
	def distance3D(self,point):		
		value = 2147483647
		if (point.dimensions == 3):
			value = point.x*point.x + point.y*point.y + point.z*point.z
		return value
		
	def closestToOrigin3D(self,list, n):		
		point3D = self.createPoint(3,0,0,0)
		minDistance = 2147483647
		l = list
		i = 0
		while(i < n):
			if(self.distance3D(l.data)<minDistance):				
				minDistance = self.distance3D(l.data)
				point3D = l.data				
			l = l.next
			i = i + 1			
		return point3D
		
	def test(self):
		numberOfPoints = 10000
		list = self.createPoints(numberOfPoints)			
		positive = self.positiveX(list, numberOfPoints)  
		point = self.closestToOrigin3D(list, numberOfPoints)		
	
class Chronometer:
	ticks1 = 0
	ticks2 = 0
	stopped = TRUE
	
	def start(self):
		self.ticks1 = time.time()
		self.stopped = FALSE

	def stop(self):
		self.ticks2 = time.time()
		self.stopped = TRUE

	def getSeconds(self):
		if self.stopped:
			return self.ticks2 - self.ticks1
		return time.time() - self.ticks1		
	def getMiliSeconds(self):
		return int(self.getSeconds() * 1000)
	def getMicroSeconds(self):
		return int(self.getSeconds() * 1000000)
	
class BenchMark:		
	def __init__(self,iterations):
		self.microSeconds = 0
		self.iterations = iterations

	def runOneIteration(self): 
		chronometer = Chronometer()							
		chronometer.start()				
		PointsBech().test()
		chronometer.stop()		
		self.microSeconds = self.microSeconds + chronometer.getMicroSeconds()		
		

	def run(self): 
		for i in range(self.iterations):
			self.runOneIteration()
		return self.microSeconds
		
if __name__ == '__main__':
	import sys
	if len(sys.argv) < 2: 
		print ("You must pass the number of thousands iterations.")
		sys.exit(-1)  
	iterations = int(sys.argv[1])
	bench = BenchMark(iterations)	
	print(str(int(bench.run())).replace(".",","))