function JGFFFTBench() {   
	this.datasizes = new Array(3);            
	this.datasizes[0] = 65536;
	this.datasizes[1] = 8388608;
	this.datasizes[2] = 16777216;	
}

JGFFFTBench.prototype.JGFsetsize = function(size) {
	this.size = size;
}		

JGFFFTBench.prototype.JGFinitialise = function() {
	this.R = Math;
}		

JGFFFTBench.prototype.RandomVector = function(N, R) {        
	var a = new Array(N);            
	for (var i = 0; i < N; i = i + 1)
		a[i] = R.random() * 1e-6;        
	return a;
}

JGFFFTBench.prototype.JGFkernel = function() {
 	var x = this.RandomVector(2 * (this.datasizes[this.size]), this.R);	
	var fft = new FFT();
	fft.transform(x);			
    fft.inverse(x);	
}

JGFFFTBench.prototype.JGFvalidate = function() {
	var refval = new Array(3);
	refval[0] = 1.726962988395339;
	refval[1] = 6.907851953579193;
	refval[2] = 13.815703907167297;
	var refvali = new Array(3);
	refvali[0] = 2.0974756152524314;
	refvali[1] = 8.389142211032294;
	refvali[2] = 16.778094422092604;		
	var dev = FFT.JDKtotal - refval[this.size];
	dev = dev>=0?dev:(-1*dev);
	var devi = FFT.JDKtotali - refvali[this.size];
	devi = devi>=0?devi:(-1*devi);
	if (dev > 1.0e-12)
	{
		//print("Validation failed");
		//print("JDKtotal = " + FFT.JDKtotal + " " + dev + " " +this.size);
	}
	if (devi > 1.0e-12)
	{
		//print("Validation failed");
		//print("JDKtotalinverse = " + FFT.JDKtotali + " " + devi + " " + this.size);
	}
}

JGFFFTBench.prototype.test = function() {
	this.JGFsetsize(0);
	this.JGFinitialise();
	this.JGFkernel();
	this.JGFvalidate();    	
}

function FFT() {   
	FFT.JDKtotal = 0.0;
	FFT.JDKtotali = 0.0;
}

FFT.prototype.transform = function(data) {        
	var JDKrange;
	this.transform_internal(data, -1);
	JDKrange = data.length;
	for (var i = 0; i < JDKrange; i = i + 1)            
		FFT.JDKtotal = FFT.JDKtotal + data[i];   					
}

FFT.prototype.inverse = function(data) {        
	this.transform_internal(data, +1);
	var nd = data.length;
	var n = nd / 2;
	var norm = 1 / n;
	for (var i = 0; i < nd; i = i + 1)
		data[i] = data[i] * norm;
	for (var i = 0; i < nd; i = i + 1)
		FFT.JDKtotali = FFT.JDKtotali + data[i];
}

FFT.prototype.makeRandom = function(n) {  
	var rand = Math;
	var nd = 2 * n;
	var data = new Array(nd);
	for (var i = 0; i < nd; i = i + 1)
		data[i] = rand.random();
	return data;
}

FFT.prototype.log2 = function(n) {  
	var log = 0;
	var k = 1; 				
	while(k < n)
	{									
		k = k * 2;								
		log = log + 1;								
	}			
	if (n != (1 << log)){		
		print("FFT: Data Length is not a power of 2!: " + n);
	}
	return log;
}

FFT.prototype.transform_internal = function(data,direction) {  
	var n = data.length / 2;
	if (n == 1) return;			
	var logn = this.log2(n);			
	this.bitreverse(data);							
	var dual = 1;	
	var bit = 0;
	while(bit < logn)            
	{                            
		var w_real = 1.0;
		var w_imag = 0.0;
		var theta = 2.0 * direction * 3.14159265 / (2.0 * dual);
		var s = Math.sin(theta);
		var t = Math.sin(theta / 2.0);
		var s2 = 2.0 * t * t;
		var iterations = 0;											
		while (iterations < n)
		{                    
			var i = 2 * iterations;
			var j = 2 * (iterations + dual);
			var wd_real = data[j];
			var wd_imag = data[j + 1];                    
			data[j] = data[i] - wd_real;
			data[j + 1] = data[i + 1] - wd_imag;
			data[i] = data[i] + wd_real;
			data[i + 1] = data[i + 1] + wd_imag;                    
			iterations = iterations + (2*dual);
		}             
		iterations = 1;
		while (iterations < dual)
		{
			var tmp_real = w_real - s * w_imag - s2 * w_real;
			var tmp_imag = w_imag + s * w_real - s2 * w_imag;
			w_real = tmp_real;
			w_imag = tmp_imag;

			var b = 0;
			while (b < n)
			{                        
				var i = 2 * (b + iterations);
				var j = 2 * (b + iterations + dual);

				var z1_real = data[j];
				var z1_imag = data[j + 1];

				var wd_real = w_real * z1_real - w_imag * z1_imag;
				var wd_imag = w_real * z1_imag + w_imag * z1_real;

				data[j] = data[i] - wd_real;
				data[j + 1] = data[i + 1] - wd_imag;
				data[i] = data[i] + wd_real;
				data[i + 1] = data[i + 1] + wd_imag;
				b = b + (2 * dual);
			}                    
			iterations = iterations + 1;
		}                
		dual = dual * 2;
		bit = bit + 1;
	}
}

FFT.prototype.bitreverse = function(data) {  
	var n = data.length / 2;
	var j = 0;
	var i = 0;					
	while(i < (n - 1))
	{			
		var ii = 2 * i;
		var jj = 2 * j;
		var k = n / 2;
		if (i < j)
		{											
			var tmp_real = data[ii];										
			var tmp_imag = data[ii + 1];
			data[ii] = data[jj];
			data[ii + 1] = data[jj + 1];
			data[jj] = tmp_real;
			data[jj + 1] = tmp_imag;				
		}							
		while (k <= j)
		{										
			j = j - k;
			k = k / 2;				
		}				
		j = j + k;
		i = i + 1;												
	}						
}

function BenchMark(iterations) {
    this.microSeconds = 0;
	this.iterations = iterations;
}

BenchMark.prototype.runOneIteration = function() {		
	var before = new Date();
	new JGFFFTBench().test();
	var after = new Date();	
	this.microSeconds =  this.microSeconds + ((after-before) * 1000);
}

BenchMark.prototype.run = function() {	
	for(var i = 0 ; i < this.iterations ; i++)
		this.runOneIteration();	
	return this.microSeconds;
}	

function range(n) {
	var result = new Array();
	for(var i=0; i<n; i++)
		result[i] = i;
	return result;
}

if (scriptArgs.length == 1) {		
	var bench = new BenchMark(scriptArgs[0]);
	print(bench.run());
}
else {
	print("I need the name of the test.");
	quit();
}