function Vec(a,b,c) { 
	a = typeof a !== 'undefined' ? a : 0;
	b = typeof b !== 'undefined' ? b : 0;
	c = typeof c !== 'undefined' ? c : 0;
	this.x = a;
	this.y = b;
	this.z = c;
}

Vec.prototype.add1 = function(a) {
	this.x = a.x + this.x;
	this.y = a.y + this.y;
	this.z = a.z + this.z;	
}

Vec.adds = function(s,a,b) {
	return new Vec(s * a.x + b.x, s * a.y + b.y, s * a.z + b.z);
}

Vec.prototype.adds = function(s,b) {
	this.x = this.x + (s * b.x);
	this.y = this.y + (s * b.y);
	this.z = this.z + (s * b.z);
}

Vec.sub1 = function(a,b) {
	return new Vec(a.x - b.x, a.y - b.y, a.z - b.z);
}

Vec.prototype.sub2 = function(a,b) {
	this.x = a.x - b.x;
	this.y = a.y - b.y;
	this.z = a.z - b.z;
}

Vec.mult1 = function(a,b) {
	return new Vec(a.x * b.x, a.y * b.y, a.z * b.z);
}

Vec.cross = function(a,b) {
	return new Vec(a.y * b.z - a.z * b.y, a.z * b.x - a.x * b.z, a.x * b.y - a.y * b.x);
}

Vec.dot = function(a,b) {
	return a.x * b.x + a.y * b.y + a.z * b.z;
}

Vec.comb = function(a,A,b,B) {
	return new Vec(a * A.x + b * B.x, a * A.y + b * B.y, a * A.z + b * B.z);
}

Vec.prototype.comb2 = function(a,A,b,B) {
	this.x = a * A.x + b * B.x;
	this.y = a * A.y + b * B.y;
	this.z = a * A.z + b * B.z;
}

Vec.prototype.scale = function(t) {
	this.x = t * this.x;
	this.y = t * this.y;
	this.z = t * this.z;
}

Vec.prototype.negate = function() {
	this.x = -1.0 * this.x;
	this.y = -1.0 * this.y;
	this.z = -1.0 * this.z;
}

Vec.prototype.normalize = function() {
	var len = Math.sqrt(this.x * this.x + this.y * this.y + this.z * this.z);		
	while (len > 0.0)
	{
		this.x = (this.x / len);
		this.y = (this.y / len);
		this.z = (this.z / len);
		len = 0.0;
	}					
	return len;
}

Vec.prototype.toString = function() {
	return "<" + this.x + "," + this.y + "," + this.z + ">";
}

function View(fromVec, atVec, upVec, dist, angle, aspect) { 
	this.fromVec = fromVec;
	this.atVec = atVec;
	this.upVec = upVec;
	this.dist = dist;
	this.angle = angle;
	this.aspect = aspect;
}

function Ray(pnt, dir) { 	
	this.P = new Vec();			
	this.D = new Vec();	
	if(typeof pnt !== 'undefined'){
		this.P.add1(pnt);
		this.D.add1(dir);
		this.D.normalize();
	}
}

Ray.prototype.point = function(t) {
	return new Vec(this.P.x + this.D.x * t, this.P.y + this.D.y * t, this.P.z + this.D.z * t);
}

Ray.prototype.toString = function() {
	return "{" + this.P.toString() + " -> " + this.D.toString() + "}";
}

function Interval(number, width, height, yfrom, yto, total) { 
	this.number = number;
	this.width = width;
	this.height = height;
	this.yfrom = yfrom;
	this.yto = yto;
	this.total = total;
}

function Isect() { 	
}

Isect.prototype.toString = function() {
	return "{" + this.t.toString() + "," + this.enter.toString() + "," + this.prim.toString() + "," + this.surf.toString() + "}";
}

function Light(x, y, z, brightness) { 
	if(typeof x !== 'undefined'){
		this.pos = new Vec(x,y,z);
		this.brightness = brightness;
	}
}

function Surface() { 
	this.color = new Vec(1.0, 0.0, 0.0);
	this.kd = 1.0;
	this.ks = 0.0;
	this.shine = 0.0;
	this.kt = 0.0;
	this.ior = 1.0;	
}

Surface.prototype.toString = function() {
	return "Surface { color=" + this.color.toString() + "}";
}

function Sphere(center, radius) { 
	this.surf = new Surface();
	this.c = center;
	this.r = radius;
	this.r2 = radius * radius;
	this.v = new Vec();
	this.b = new Vec();	
}

Sphere.prototype.setColor = function(r,g,b) {
	this.surf.color = new Vec(r, g, b);
}

Sphere.prototype.getCenter = function() {
	return this.c;
}

Sphere.prototype.setCenter = function(c) {
	this.c = c;
}

Sphere.prototype.intersect = function(ry) {
	var p = ry.P;
	var d = ry.D;       			
	this.v.sub2(this.c, p);					
	var b = Vec.dot(this.v, d);									
	var dotVV = Vec.dot(this.v, this.v);
	var disc = b * b - dotVV + this.r2;				
	if (disc < 0.0)            			
		return null;            												
	disc = Math.sqrt(disc);									            	
	var t = (b - disc < 1e-6) ? b + disc : b - disc;							
	if (t < 1e-6)            
		return null;    			
	var ip = new Isect();
	ip.t = t;						
	ip.enter = dotVV > this.r2 + 1e-6 ? 1.0 : 0.0;
	ip.prim = this;
	ip.surf = this.surf;
	return ip;	
}

Sphere.prototype.normal = function(p) {
    var r = Vec.sub1(p, this.c);
	r.normalize();
	return r;	
}

Sphere.prototype.toString = function() {
	return "Sphere {" + this.c.toString() + "," + this.r + "}";
}

function Scene() { 
	this.lights = new Array();
	this.objects = new Array();	
}

Scene.prototype.addLight = function(l) {
	this.lights.push(l);   
}

Scene.prototype.addObject = function(myobject) {
	this.objects.push(myobject);   
}

Scene.prototype.getView = function() {
	return this.view;
}

Scene.prototype.setView = function(view) {
	this.view = view;
}

Scene.prototype.getLight = function(number) {
	return this.lights[number];
}

Scene.prototype.getObject = function(number) {
	return this.objects[number];
}

Scene.prototype.getLights = function() {
	return this.lights.length;
}

Scene.prototype.getObjects = function() {
	return this.objects.length;
}

Scene.prototype.setObject = function(myobject,pos) {
	this.objects[pos] = myobject;
}

function RayTracer() {
    this.datasizes = new Array(10);
	this.datasizes[0] = 5;  
	this.datasizes[1] = 10;  			
	this.datasizes[2] = 15;            
	this.datasizes[3] = 20;  			
	this.datasizes[4] = 25;
	this.datasizes[5] = 30;  
	this.datasizes[6] = 35;  			
	this.datasizes[7] = 40;            
	this.datasizes[8] = 45;  			
	this.datasizes[9] = 50;
	RayTracer.alpha = 255 << 24;
	this.tRay = new Ray();
	RayTracer.voidVec = new Vec();
	this.L = new Vec();
	this.inter = new Isect();
	this.checksum = 0;
}

RayTracer.prototype.createScene = function() {
	var x = 0.0;
	var y = 0.0;
	var scene = new Scene();
	var p;
	var nx = 4.0;
	var ny = 4.0;
	var nz = 4.0;
	var i = 0.0;			
	while (i < nx)
	{
		var j = 0.0;
		while (j < ny)
		{
			var k = 0.0;
			while (k < nz)
			{
				var xx = 20.0 / (nx - 1.0) * i - 10.0;						
				var yy = 20.0 / (ny - 1.0) * j - 10.0;
				var zz = 20.0 / (nz - 1.0) * k - 10.0;												
				p = new Sphere(new Vec(xx, yy, zz), 3.0);						
				p.setColor(0.0, 0.0, ((i + j) / (nx + ny - 2.0)));
				p.surf.shine = 15.0;
				p.surf.ks = 1.5 - 1.0;
				p.surf.kt = 1.5 - 1.0;						
				scene.addObject(p);						
				k = k + 1.0;
			}
			j = j + 1.0;
		}
		i = i + 1.0;
	}

	scene.addLight(new Light(100.0, 100.0, -50.0, 1.0));
	scene.addLight(new Light(-100.0, 100.0, -50.0, 1.0));
	scene.addLight(new Light(100.0, -100.0, -50.0, 1.0));
	scene.addLight(new Light(-100.0, -100.0, -50.0, 1.0));
	scene.addLight(new Light(200.0, 200.0, 0.0, 1.0));
	var v = new View(new Vec(x, 20.0, -30.0), new Vec(x, y, 0.0), new Vec(0.0, 1.0, 0.0), 1.0, 35.0 * 3.14159265 / 180.0, 1.0);
	scene.setView(v);
	return scene;
}

RayTracer.prototype.setScene = function(scene) {
	var nLights = scene.getLights();
	var nObjects = scene.getObjects();
	this.lights = new Array(nLights);
	this.prim = new Array(nObjects);			
	var l = 0;			
	while(l < nLights)
	{
		this.lights[l] = scene.getLight(l);			
		l = l + 1;
	}
	var o = 0;
	while (o < nObjects)
	{
		this.prim[o] = scene.getObject(o);				
		o = o + 1;
	}
	this.view = scene.getView();		
}

RayTracer.prototype.intersect = function(r,tmax) {
	var nhits = 0;
	var temp = new Isect();							
	temp.t = 1e9;	
	this.inter = temp;					
	var i = 0;											
	while (i < this.prim.length)
	{						
		var tp = this.prim[i].intersect(r);			
		var sw = (tp != null);		
		if(sw)
		{										
			var isect = new Isect();							
			isect.t = tp.t;															
			isect.prim = tp.prim;
			isect.surf = tp.surf;
			isect.enter = tp.enter;														
			this.inter = isect;							
			nhits = nhits + 1;											
		}
		i = i + 1;
	}	
	return nhits > 0 ? true : false;
}

RayTracer.prototype.trace = function(level, weight, r) {
	if (level > 6)
		return new Vec();

	var hit = this.intersect(r, 1e6);
	if (hit)
	{
		var P = r.point(inter.t);
		var N = this.inter.prim.normal(P);
		if (Vec.dot(r.D, N) >= 0.0)
			N.negate();		
	}
	return RayTracer.voidVec;
}

RayTracer.prototype.SpecularDirection = function(I,N) {
	var dotIN = Vec.dot(I, N);
	var abs = dotIN>0?dotIN:(-1*dotIN);
	var r = Vec.comb(1.0 / abs, I, 2.0, N);
	r.normalize();
	return r;
}

RayTracer.prototype.TransDir = function(m1,m2,I,N) {	
	var n1 = m1 == null ? 1.0 : m1.ior;
	var n2 = m2 == null ? 1.0 : m2.ior;
	var eta = n1 / n2;
	var c1 = -Vec.dot(I, N);
	var cs2 = 1.0 - eta * eta * (1.0 - c1 * c1);
	if (cs2 < 0.0)
		return null;
	var r = Vec.comb(eta, I, eta * c1 - Math.sqrt(cs2), N);
	r.normalize();
	return r;
}

RayTracer.prototype.Shadow = function(r,tmax) {	
	if (this.intersect(r,tmax))
		return 0;
	return 1;
}

RayTracer.prototype.render = function(interval) {	
	var localView = this.view;			
	var viewVec = Vec.sub1(localView.atVec, localView.fromVec);											
	viewVec.normalize();			
	var tmpVec = new Vec(viewVec.x, viewVec.y, viewVec.z);			
	var dotValue = Vec.dot(localView.upVec, viewVec);			
	tmpVec.scale(dotValue);			
	var upVec = Vec.sub1(localView.upVec, tmpVec);
	upVec.normalize();
	var leftVec = Vec.cross(localView.upVec, viewVec);
	leftVec.normalize();
	var frustrumwidth = localView.dist * Math.tan(localView.angle);						
	upVec.scale(-frustrumwidth);			
	var scaleValue = localView.aspect * frustrumwidth;
	leftVec.scale(scaleValue);            			
	var r = new Ray(localView.fromVec, RayTracer.voidVec);
	var y = interval.yfrom;			
	var nhits = 0;		
	while (y < interval.yto)
	{								
		var ylen = (2.0 * y) / interval.width - 1.0;
		var x = 0;
		while (x < interval.width)
		{								
			var xlen = (2.0 * x) / interval.width - 1.0;					
			r.D = Vec.comb(xlen, leftVec, ylen, upVec);     					
			r.D.add1(viewVec); 									
			r.D.normalize();					
			var hit = this.intersect(r, 1e6);							
			if(hit)
				nhits = nhits + 1;			
			x = x + 1;
		}
		y = y + 1;
	}			
	this.checksum = this.checksum + nhits;		
}

RayTracer.prototype.JGFsetsize = function(size) {
	this.size = size;
}		

RayTracer.prototype.JGFinitialise = function() {
    this.width = this.height = this.datasizes[this.size];
	this.scene = this.createScene();
	this.setScene(this.scene);
	this.numobjects = this.scene.getObjects();	
}

RayTracer.prototype.JGFkernel = function() {
	var interval = new Interval(0, this.width, this.height, 0, this.height, 1);
	this.render(interval);
}

RayTracer.prototype.JGFvalidate = function() {
	var refval = new Array(10);
	refval[0] = 12;
	refval[1] = 40;            
	refval[2] = 104;            
	refval[3] = 170;     
	refval[4] = 278;            
	refval[5] = 390;
	refval[6] = 552;            
	refval[7] = 692;            
	refval[8] = 894;     
	refval[9] = 1084;            
	var dev = this.checksum - refval[this.size];
	if (dev != 0)
	{
		print("Validation failed");
		print("Pixel checksum = " + this.checksum);
		print("Reference value = " + refval[this.size]);
	}	
}

RayTracer.prototype.test = function() {
	this.JGFsetsize(4);
	this.JGFinitialise();
	this.JGFkernel();
	this.JGFvalidate();   	
}

function BenchMark(iterations) {
    this.microSeconds = 0;
	this.iterations = iterations;
}

BenchMark.prototype.runOneIteration = function() {		
	var before = new Date();
	new RayTracer().test();
	var after = new Date();	
	this.microSeconds =  this.microSeconds + ((after-before) * 1000);
}

BenchMark.prototype.run = function() {	
	for(var i = 0 ; i < this.iterations ; i++)
		this.runOneIteration();	
	return this.microSeconds;
}	

function range(n) {
	var result = new Array();
	for(var i=0; i<n; i++)
		result[i] = i;
	return result;
}

if (scriptArgs.length == 1) {		
	var bench = new BenchMark(scriptArgs[0]);
	print(bench.run());
}
else {
	print("I need the name of the test.");
	quit();
}