using System;

namespace Pybench.Aritmethic
{
	public class Chronometer
    {
        private DateTime ticks1, ticks2;
        private bool stopped;

        public void Start()
        {
            ticks1 = DateTime.Now;
            stopped = false;
        }
        public void Stop()
        {
            ticks2 = DateTime.Now;
            stopped = true;
        }

        private static int TicksToMicroSeconds(DateTime t1, DateTime t2)
        {
            return TicksToMiliSeconds(t1, t2) * 1000;
        }

        private static int TicksToMiliSeconds(DateTime t1, DateTime t2)
        {
            TimeSpan difference = t2.Subtract(t1);
            return (difference.Milliseconds + difference.Seconds * 1000 + difference.Minutes * 60000);
        }

        private static int TicksToSeconds(DateTime t1, DateTime t2)
        {
            TimeSpan difference = t2.Subtract(t1);
            return (difference.Seconds + difference.Minutes * 60);
        }

        public int GetMicroSeconds()
        {
            if (stopped)
                return TicksToMicroSeconds(ticks1, ticks2);
            return TicksToMicroSeconds(ticks1, DateTime.Now);
        }

        public int GetMiliSeconds()
        {
            if (stopped)
                return TicksToMiliSeconds(ticks1, ticks2);
            return TicksToMiliSeconds(ticks1, DateTime.Now);
        }

        public int GetSeconds()
        {
            if (stopped)
                return TicksToSeconds(ticks1, ticks2);
            return TicksToSeconds(ticks1, DateTime.Now);
        }
    }
    
	public class BenchMark 
	{
		private int iterations;
		protected int microSeconds;

		public BenchMark(int iterations) 
		{
			this.iterations = iterations;
		}

		public int run() 
		{
			BenchMark self = this;			
			for (int i = 0; i < iterations; i++)
				self.runOneIteration();
			return this.microSeconds;
		}

		virtual public object runOneIteration() { return null; }
	}
	
	public class ArithmethicBenchmark : BenchMark 
	{
		public ArithmethicBenchmark(int iterations) : base(iterations) { }	
		public override object runOneIteration() 
		{				
			Chronometer chronometer = new Chronometer();
			Test test = new DictCreation();						
			chronometer.Start();				
			test.test();
			chronometer.Stop();			
			this.microSeconds = this.microSeconds + chronometer.GetMicroSeconds();
			return null;
		}
	}
	
	public abstract class Test {
        public abstract void test();
    }

	 public class DictCreation : Test
    {
        public override void test()
        {
            var i = 0;

            var two = 2;
            var three = 3;
            var four = 4;
            var five = 5;
            var six = 6;
            var seven = 7;
            var eight = 8;
            var nine = 9;
            var ten = 10;
            var eleven = 11;

            while (i < 80000)
            {
                var d1 = new var[11];
                var d2 = new var[11];
                var d3 = new var[11];
                var d4 = new var[11];
                var d5 = new var[11];

                d1[1] = two;
                d1[3] = four;
                d1[5] = six;

                d2[2] = three;
                d2[4] = five;
                d2[6] = seven;

                d3[3] = four;
                d3[5] = six;
                d3[7] = eight;

                d4[4] = five;
                d4[6] = seven;
                d4[8] = nine;

                d5[6] = seven;
                d5[8] = nine;
                d5[10] = eleven;

                d1 = new var[11];
                d2 = new var[11];
                d3 = new var[11];
                d4 = new var[11];
                d5 = new var[11];

                d1[1] = two;
                d1[3] = four;
                d1[5] = six;

                d2[2] = three;
                d2[4] = five;
                d2[6] = seven;

                d3[3] = four;
                d3[5] = six;
                d3[7] = eight;

                d4[4] = five;
                d4[6] = seven;
                d4[8] = nine;

                d5[6] = seven;
                d5[8] = nine;
                d5[10] = eleven;

                d1 = new var[11];
                d2 = new var[11];
                d3 = new var[11];
                d4 = new var[11];
                d5 = new var[11];

                d1[1] = two;
                d1[3] = four;
                d1[5] = six;

                d2[2] = three;
                d2[4] = five;
                d2[6] = seven;

                d3[3] = four;
                d3[5] = six;
                d3[7] = eight;

                d4[4] = five;
                d4[6] = seven;
                d4[8] = nine;

                d5[6] = seven;
                d5[8] = nine;
                d5[10] = eleven;

                d1 = new var[11];
                d2 = new var[11];
                d3 = new var[11];
                d4 = new var[11];
                d5 = new var[11];

                d1[1] = two;
                d1[3] = four;
                d1[5] = six;

                d2[2] = three;
                d2[4] = five;
                d2[6] = seven;

                d3[3] = four;
                d3[5] = six;
                d3[7] = eight;

                d4[4] = five;
                d4[6] = seven;
                d4[8] = nine;

                d5[6] = seven;
                d5[8] = nine;
                d5[10] = eleven;

                d1 = new var[11];
                d2 = new var[11];
                d3 = new var[11];
                d4 = new var[11];
                d5 = new var[11];

                d1[1] = two;
                d1[3] = four;
                d1[5] = six;

                d2[2] = three;
                d2[4] = five;
                d2[6] = seven;

                d3[3] = four;
                d3[5] = six;
                d3[7] = eight;

                d4[4] = five;
                d4[6] = seven;
                d4[8] = nine;

                d5[6] = seven;
                d5[8] = nine;
                d5[10] = eleven;

                i = i + 1;
            }
        }
    }
	
	public class Program 
	{
		public static void Main(string[] args) 
		{
			if (args.Length<1) {
				Console.Error.WriteLine("You must pass the number of thousands iterations.");
					System.Environment.Exit(-1);
			}
			int iterations = Convert.ToInt32(args[0]);
			ArithmethicBenchmark arith = new ArithmethicBenchmark(iterations);
			Console.WriteLine(arith.run());
		}
    }
}