Imports System.Collections
Namespace Pybench.Aritmethic
    Public Class Chronometer
        Private ticks1 As DateTime, ticks2 As DateTime
        Private stopped As Boolean

        Public Sub Start()
            ticks1 = DateTime.Now
            stopped = False
        End Sub
        Public Sub [Stop]()
            ticks2 = DateTime.Now
            stopped = True
        End Sub

        Private Shared Function TicksToMicroSeconds(t1 As DateTime, t2 As DateTime) As Integer
            Return TicksToMiliSeconds(t1, t2) * 1000
        End Function

        Private Shared Function TicksToMiliSeconds(t1 As DateTime, t2 As DateTime) As Integer
            Dim difference As TimeSpan = t2.Subtract(t1)
            Return (difference.Milliseconds + difference.Seconds * 1000 + difference.Minutes * 60000)
        End Function

        Private Shared Function TicksToSeconds(t1 As DateTime, t2 As DateTime) As Integer
            Dim difference As TimeSpan = t2.Subtract(t1)
            Return (difference.Seconds + difference.Minutes * 60)
        End Function

        Public Function GetMicroSeconds() As Integer
            If stopped Then
                Return TicksToMicroSeconds(ticks1, ticks2)
            End If
            Return TicksToMicroSeconds(ticks1, DateTime.Now)
        End Function

        Public Function GetMiliSeconds() As Integer
            If stopped Then
                Return TicksToMiliSeconds(ticks1, ticks2)
            End If
            Return TicksToMiliSeconds(ticks1, DateTime.Now)
        End Function

        Public Function GetSeconds() As Integer
            If stopped Then
                Return TicksToSeconds(ticks1, ticks2)
            End If
            Return TicksToSeconds(ticks1, DateTime.Now)
        End Function
    End Class

    Public Class BenchMark
        Private iterations As Integer
        Protected microSeconds As Integer

        Public Sub New(iterations As Integer)
            Me.iterations = iterations
        End Sub

        Public Function run() As Integer
            Dim self As BenchMark = Me
            For i As Integer = 0 To iterations - 1
                self.runOneIteration()
            Next
            Return Me.microSeconds
        End Function

        Public Overridable Function runOneIteration() As Object
            Return Nothing
        End Function
    End Class

    Public Class ArithmethicBenchmark
        Inherits BenchMark
        Public Sub New(iterations As Integer)
            MyBase.New(iterations)
        End Sub
        Public Overrides Function runOneIteration() As Object
            Dim chronometer As New Chronometer()
            Dim test As Test = New JGFFFTBench()
            chronometer.Start()
            test.test()
            chronometer.[Stop]()
            Me.microSeconds = Me.microSeconds + chronometer.GetMicroSeconds()
            Return Nothing
        End Function
    End Class

    Public MustInherit Class Test
        Public MustOverride Sub test()
    End Class

    Public Class JGFFFTBench
        Inherits Test

        Private size
        Private datasizes
        Private R

        Public Sub New()
            datasizes = New Integer(2) {}
            'datasizes[0] = 2097152;
            datasizes(0) = 65536
            datasizes(1) = 8388608
            datasizes(2) = 16777216
        End Sub

        Public Sub JGFsetsize(size)
            Me.size = size
        End Sub

        Public Sub JGFinitialise()
            R = New Random(1010)
        End Sub

        Public Sub JGFkernel()
            Dim x
            x = JGFFFTBench.RandomVector(2 * (datasizes(size)), R)
            FFT.transform(x)
            FFT.inverse(x)
        End Sub

        Private Shared Function RandomVector(N, R)
            Dim a
            a = New Double(N - 1) {}
            For i = 0 To N - 1
                a(i) = R.NextDouble() * 0.000001
            Next
            Return a
        End Function

        Public Sub JGFvalidate()
            Dim refval
            refval = New Double(2) {}
            refval(0) = 1.72696298839534
            refval(1) = 6.90785195357919
            refval(2) = 13.8157039071673
            Dim refvali
            refvali = New Double(2) {}
            refvali(0) = 2.09747561525243
            refvali(1) = 8.38914221103229
            refvali(2) = 16.7780944220926
            Dim dev
            dev = FFT.JDKtotal - refval(size)
            dev = If(dev >= 0, dev, (-1 * dev))
            Dim devi
            devi = FFT.JDKtotali - refvali(size)
            devi = If(devi >= 0, devi, (-1 * devi))            
            If dev > 0.000000000001 Then
                'Console.WriteLine("Validation failed");
                'Console.WriteLine("JDKtotal = {0} {1} {2}",FFT.JDKtotal,dev,size);
            End If            
            If devi > 0.000000000001 Then
                'Console.WriteLine("Validation failed");
                'Console.WriteLine("JDKtotalinverse = {0} {1} {2}",FFT.JDKtotali,devi,size);
            End If
        End Sub

        Public Overrides Sub test()
            JGFsetsize(0)
            JGFinitialise()
            JGFkernel()
            JGFvalidate()
        End Sub
    End Class

    Public Class FFT
        Public Shared JDKtotal = 0.0
        Public Shared JDKtotali = 0.0

        Public Shared Sub transform(data)
            Dim JDKrange            
            transform_internal(data, -1)
            JDKrange = data.Length
            For i = 0 To JDKrange - 1                
                JDKtotal = JDKtotal + data(i)
            Next
        End Sub

        Public Shared Sub inverse(data)
            transform_internal(data, 1)
            Dim nd
            nd = data.Length
            Dim n
            n = nd / 2
            Dim norm
            norm = 1 / CDbl(n)            
            For i = 0 To nd - 1
                data(i) = data(i) * norm
            Next
            For i = 0 To nd - 1
                JDKtotali = JDKtotali + data(i)                
            Next
        End Sub

        Public Shared Function makeRandom(n)
            Dim rand
            rand = New Random()
            Dim nd
            nd = 2 * n
            Dim data
            data = New Double(nd - 1) {}
            For i = 0 To nd - 1
                data(i) = rand.NextDouble()
            Next
            Return data
        End Function

        Protected Shared Function log2(n)
            Dim log
            log = 0
            Dim k
            k = 1
            While k < n
                k = k * 2
                log = log + 1
            End While
            If n <> (1 << log) Then
                Console.[Error].WriteLine("FFT: Data Length is not a power of 2!: " & Convert.ToString(n))
            End If
            Return log
        End Function

        Protected Shared Sub transform_internal(data, direction)
            Dim n
            n = data.Length / 2
            If n = 1 Then
                Return
            End If
            Dim logn
            logn = log2(n)
            bitreverse(data)
            Dim dual
            dual = 1
            Dim bit
            bit = 0
            While bit < logn
                Dim w_real
                w_real = 1.0
                Dim w_imag
                w_imag = 0.0
                Dim theta
                theta = 2.0 * direction * 3.14159265 / (2.0 * dual)
                Dim s
                s = Math.Sin(theta)
                Dim t
                t = Math.Sin(theta / 2.0)
                Dim s2
                s2 = 2.0 * t * t
                Dim iterations
                iterations = 0
                While iterations < n
                    Dim i
                    i = 2 * iterations
                    Dim j
                    j = 2 * (iterations + dual)
                    Dim wd_real
                    wd_real = data(j)
                    Dim wd_imag
                    wd_imag = data(j + 1)
                    data(j) = data(i) - wd_real
                    data(j + 1) = data(i + 1) - wd_imag
                    data(i) = data(i) + wd_real
                    data(i + 1) = data(i + 1) + wd_imag
                    iterations = iterations + (2 * dual)
                End While
                iterations = 1
                While iterations < dual
                    Dim tmp_real
                    tmp_real = w_real - s * w_imag - s2 * w_real
                    Dim tmp_imag
                    tmp_imag = w_imag + s * w_real - s2 * w_imag
                    w_real = tmp_real
                    w_imag = tmp_imag

                    Dim b
                    b = 0
                    While b < n
                        Dim i
                        i = 2 * (b + iterations)
                        Dim j
                        j = 2 * (b + iterations + dual)

                        Dim z1_real
                        z1_real = data(j)
                        Dim z1_imag
                        z1_imag = data(j + 1)

                        Dim wd_real
                        wd_real = w_real * z1_real - w_imag * z1_imag
                        Dim wd_imag
                        wd_imag = w_real * z1_imag + w_imag * z1_real

                        data(j) = data(i) - wd_real
                        data(j + 1) = data(i + 1) - wd_imag
                        data(i) = data(i) + wd_real
                        data(i + 1) = data(i + 1) + wd_imag
                        b = b + (2 * dual)
                    End While
                    iterations = iterations + 1
                End While
                dual = dual * 2
                bit = bit + 1
            End While
        End Sub

        Protected Shared Sub bitreverse(data)
            Dim n
            n = data.Length / 2
            Dim j
            j = 0
            Dim i
            i = 0
            While i < (n - 1)
                Dim ii
                ii = 2 * i
                Dim jj
                jj = 2 * j
                Dim k
                k = n / 2
                If i < j Then
                    Dim tmp_real
                    tmp_real = data(ii)
                    Dim tmp_imag
                    tmp_imag = data(ii + 1)
                    data(ii) = data(jj)
                    data(ii + 1) = data(jj + 1)
                    data(jj) = tmp_real
                    data(jj + 1) = tmp_imag
                End If
                While k <= j
                    j = j - k
                    k = k / 2
                End While
                j = j + k
                i = i + 1
            End While
        End Sub
    End Class

    Public Class Program
        Public Shared Sub Main(args As String())
            If args.Length < 1 Then
                Console.[Error].WriteLine("You must pass the number of thousands iterations.")
                System.Environment.[Exit](-1)
            End If
            Dim iterations As Integer = Convert.ToInt32(args(0))
            Dim arith As New ArithmethicBenchmark(iterations)
            Console.WriteLine(arith.run())
        End Sub
    End Class
End Namespace
