Namespace Pybench.Aritmethic
    Public Class Chronometer
        Private ticks1 As DateTime, ticks2 As DateTime
        Private stopped As Boolean

        Public Sub Start()
            ticks1 = DateTime.Now
            stopped = False
        End Sub
        Public Sub [Stop]()
            ticks2 = DateTime.Now
            stopped = True
        End Sub

        Private Shared Function TicksToMicroSeconds(t1 As DateTime, t2 As DateTime) As Integer
            Return TicksToMiliSeconds(t1, t2) * 1000
        End Function

        Private Shared Function TicksToMiliSeconds(t1 As DateTime, t2 As DateTime) As Integer
            Dim difference As TimeSpan = t2.Subtract(t1)
            Return (difference.Milliseconds + difference.Seconds * 1000 + difference.Minutes * 60000)
        End Function

        Private Shared Function TicksToSeconds(t1 As DateTime, t2 As DateTime) As Integer
            Dim difference As TimeSpan = t2.Subtract(t1)
            Return (difference.Seconds + difference.Minutes * 60)
        End Function

        Public Function GetMicroSeconds() As Integer
            If stopped Then
                Return TicksToMicroSeconds(ticks1, ticks2)
            End If
            Return TicksToMicroSeconds(ticks1, DateTime.Now)
        End Function

        Public Function GetMiliSeconds() As Integer
            If stopped Then
                Return TicksToMiliSeconds(ticks1, ticks2)
            End If
            Return TicksToMiliSeconds(ticks1, DateTime.Now)
        End Function

        Public Function GetSeconds() As Integer
            If stopped Then
                Return TicksToSeconds(ticks1, ticks2)
            End If
            Return TicksToSeconds(ticks1, DateTime.Now)
        End Function
    End Class

    Public Class BenchMark
        Private iterations As Integer
        Protected microSeconds As Integer

        Public Sub New(iterations As Integer)
            Me.iterations = iterations
        End Sub

        Public Function run() As Integer
            Dim self As BenchMark = Me
            For i As Integer = 0 To iterations - 1
                self.runOneIteration()
            Next
            Return Me.microSeconds
        End Function

        Public Overridable Function runOneIteration() As Object
            Return Nothing
        End Function
    End Class

    Public Class ArithmethicBenchmark
        Inherits BenchMark
        Public Sub New(iterations As Integer)
            MyBase.New(iterations)
        End Sub
        Public Overrides Function runOneIteration() As Object
            Dim chronometer As New Chronometer()
            Dim test As NumericSortTest = New JGFHeapSortBench()
            chronometer.Start()
            test.Test()
            chronometer.[Stop]()
            Me.microSeconds = Me.microSeconds + chronometer.GetMicroSeconds()
            Return Nothing
        End Function
    End Class

    Public MustInherit Class Test
        Public MustOverride Sub test()
    End Class

    Public MustInherit Class NumericSortTest
        Public array_rows
        Public TestArray
        Public Sub buildTestData()
            TestArray = New Integer(array_rows - 1) {}
            Dim rndnum
            rndnum = New Random(1729)
            Dim i
            i = 0
            While i < array_rows
                TestArray(i) = rndnum.[Next]()
                i = i + 1
            End While
        End Sub

        Public Sub [Do]()
            NumHeapSort()
        End Sub

        Private Sub NumSift(min, max)
            While (min + min) <= max
                Dim inc
                inc = False
                If (min + min) < max AndAlso TestArray(min + min) < TestArray(min + min + 1) Then
                    inc = True
                End If
                Dim k
                k = If(inc, min + min + 1, min + min)
                If TestArray(min) < TestArray(k) Then
                    Dim temp
                    temp = TestArray(k)
                    TestArray(k) = TestArray(min)
                    TestArray(min) = temp
                    min = k
                Else
                    min = max + 1
                End If
            End While
        End Sub

        Private Sub NumHeapSort()
            Dim temp

            Dim top
            top = array_rows - 1
            Dim i
            i = top / 2
            While i > 0
                NumSift(i, top)
                i = i - 1
            End While
            i = top
            While i > 0
                NumSift(0, i)
                temp = TestArray(0)
                TestArray(0) = TestArray(i)
                TestArray(i) = temp
                i = i - 1
            End While
        End Sub

        Public MustOverride Sub Test()
    End Class

    Public Class JGFHeapSortBench
        Inherits NumericSortTest
        Private size
        Private datasizes

        Public Sub New()
            datasizes = New Integer(2) {}
            datasizes(0) = 1000000
            datasizes(1) = 5000000
            datasizes(2) = 25000000
        End Sub

        Public Sub JGFsetsize(size)
            Me.size = size
        End Sub

        Public Sub JGFinitialise()
            array_rows = datasizes(size)
            buildTestData()
        End Sub

        Public Sub JGFkernel()
            [Do]()
        End Sub

        Public Sub JGFvalidate()
            Dim [error]
            [error] = False
            Dim i
            i = 1
            While i < array_rows
                [error] = (TestArray(i) < TestArray(i - 1))
                If [error] Then
                    'Console.WriteLine("Validation failed")
                    'Console.WriteLine(("Item " & Convert.ToString(i) & " = ") + TestArray(i).ToString())
                    'Console.WriteLine(("Item " & Convert.ToString((i - 1)) & " = ") + TestArray(i - 1).ToString())                    
                    'Exit While
                End If
                i = i + 1
            End While
        End Sub
        Public Overrides Sub Test()
            JGFsetsize(0)
            JGFinitialise()
            JGFkernel()
            JGFvalidate()
        End Sub
    End Class

    Public Class Program
        Public Shared Sub Main(args As String())
            If args.Length < 1 Then
                Console.[Error].WriteLine("You must pass the number of thousands iterations.")
                System.Environment.[Exit](-1)
            End If
            Dim iterations As Integer = Convert.ToInt32(args(0))
            Dim arith As New ArithmethicBenchmark(iterations)
            Console.WriteLine(arith.run())
        End Sub
    End Class
End Namespace
