Imports System.Collections

Namespace Pybench.Aritmethic
    Public Class Chronometer
        Private ticks1 As DateTime, ticks2 As DateTime
        Private stopped As Boolean

        Public Sub Start()
            ticks1 = DateTime.Now
            stopped = False
        End Sub
        Public Sub [Stop]()
            ticks2 = DateTime.Now
            stopped = True
        End Sub

        Private Shared Function TicksToMicroSeconds(t1 As DateTime, t2 As DateTime) As Integer
            Return TicksToMiliSeconds(t1, t2) * 1000
        End Function

        Private Shared Function TicksToMiliSeconds(t1 As DateTime, t2 As DateTime) As Integer
            Dim difference As TimeSpan = t2.Subtract(t1)
            Return (difference.Milliseconds + difference.Seconds * 1000 + difference.Minutes * 60000)
        End Function

        Private Shared Function TicksToSeconds(t1 As DateTime, t2 As DateTime) As Integer
            Dim difference As TimeSpan = t2.Subtract(t1)
            Return (difference.Seconds + difference.Minutes * 60)
        End Function

        Public Function GetMicroSeconds() As Integer
            If stopped Then
                Return TicksToMicroSeconds(ticks1, ticks2)
            End If
            Return TicksToMicroSeconds(ticks1, DateTime.Now)
        End Function

        Public Function GetMiliSeconds() As Integer
            If stopped Then
                Return TicksToMiliSeconds(ticks1, ticks2)
            End If
            Return TicksToMiliSeconds(ticks1, DateTime.Now)
        End Function

        Public Function GetSeconds() As Integer
            If stopped Then
                Return TicksToSeconds(ticks1, ticks2)
            End If
            Return TicksToSeconds(ticks1, DateTime.Now)
        End Function
    End Class

    Public Class BenchMark
        Private iterations As Integer
        Protected microSeconds As Integer

        Public Sub New(iterations As Integer)
            Me.iterations = iterations
        End Sub

        Public Function run() As Integer
            Dim self As BenchMark = Me
            For i As Integer = 0 To iterations - 1
                self.runOneIteration()
            Next
            Return Me.microSeconds
        End Function

        Public Overridable Function runOneIteration() As Object
            Return Nothing
        End Function
    End Class

    Public Class ArithmethicBenchmark
        Inherits BenchMark
        Public Sub New(iterations As Integer)
            MyBase.New(iterations)
        End Sub
        Public Overrides Function runOneIteration() As Object
            Dim chronometer As New Chronometer()
            Dim test As JGFRayTracerBench = New JGFRayTracerBench()
            chronometer.Start()
            test.Test()
            chronometer.[Stop]()
            Me.microSeconds = Me.microSeconds + chronometer.GetMicroSeconds()
            Return Nothing
        End Function
    End Class

    Public Class Vec
        Public x
        Public y
        Public z
        Public Sub New(a, b, c)
            x = a
            y = b
            z = c
        End Sub

        Public Sub New()
            x = 0.0
            y = 0.0
            z = 0.0
        End Sub

        Public Sub add1(a)
            x = a.x + Me.x
            y = a.y + Me.y
            z = a.z + Me.z
        End Sub

        Public Shared Function adds(s, a, b)
            Return New Vec(s * a.x + b.x, s * a.y + b.y, s * a.z + b.z)
        End Function

        Public Sub adds(s, b)
            Me.x = Me.x + (s * b.x)
            Me.y = Me.y + (s * b.y)
            Me.z = Me.z + (s * b.z)
        End Sub

        Public Shared Function sub1(a, b)
            Return New Vec(a.x - b.x, a.y - b.y, a.z - b.z)
        End Function

        Public Sub sub2(a, b)
            Me.x = a.x - b.x
            Me.y = a.y - b.y
            Me.z = a.z - b.z
        End Sub

        Public Shared Function mult1(a, b)
            Return New Vec(a.x * b.x, a.y * b.y, a.z * b.z)
        End Function

        Public Shared Function cross(a, b)
            Return New Vec(a.y * b.z - a.z * b.y, a.z * b.x - a.x * b.z, a.x * b.y - a.y * b.x)
        End Function

        Public Shared Function dot(a, b)
            Return a.x * b.x + a.y * b.y + a.z * b.z
        End Function

        Public Shared Function comb(a__1, A__2, b__3, B__4)
            Return New Vec(a__1 * A__2.x + b__3 * B__4.x, a__1 * A__2.y + b__3 * B__4.y, a__1 * A__2.z + b__3 * B__4.z)
        End Function

        Public Sub comb2(a__1, A__2, b__3, B__4)
            x = a__1 * A__2.x + b__3 * B__4.x
            y = a__1 * A__2.y + b__3 * B__4.y
            z = a__1 * A__2.z + b__3 * B__4.z
        End Sub

        Public Sub scale(t)
            x = t * x
            y = t * y
            z = t * z
        End Sub

        Public Sub negate()
            x = -1.0 * x
            y = -1.0 * y
            z = -1.0 * z
        End Sub

        Public Function normalize()
            Dim len = Math.Sqrt(x * x + y * y + z * z)
            While len > 0.0
                x = CDbl(x / len)
                y = CDbl(y / len)
                z = CDbl(z / len)
                len = 0.0
            End While
            Return len
        End Function

        Public Overrides Function ToString() As [String]
            Return "<" & Convert.ToString(x) & "," & Convert.ToString(y) & "," & Convert.ToString(z) & ">"
        End Function
    End Class

    Public Class View
        Public fromVec
        Public atVec
        Public upVec
        Public dist
        Public angle
        Public aspect

        Public Sub New(fromVec, atVec, upVec, dist, angle, aspect)
            Me.fromVec = fromVec
            Me.atVec = atVec
            Me.upVec = upVec
            Me.dist = dist
            Me.angle = angle
            Me.aspect = aspect
        End Sub
    End Class

    Public Class Ray
        Public P
        Public D

        Public Sub New(pnt, dir)
            P = New Vec()
            P.add1(pnt)
            D = New Vec()
            D.add1(dir)
            D.normalize()
        End Sub

        Public Sub New()
            P = New Vec()
            D = New Vec()
        End Sub

        Public Function point(t)
            Return New Vec(P.x + D.x * t, P.y + D.y * t, P.z + D.z * t)
        End Function

        Public Overrides Function ToString() As [String]
            Return "{" & P.ToString() & " -> " & D.ToString() & "}"
        End Function
    End Class

    Public Class Interval
        Public number
        Public width
        Public height
        Public yfrom
        Public yto
        Public total

        Public Sub New(number, width, height, yfrom, yto, total)
            Me.number = number
            Me.width = width
            Me.height = height
            Me.yfrom = yfrom
            Me.yto = yto
            Me.total = total
        End Sub
    End Class

    Public Class Isect
        Public t
        Public enter
        Public prim
        Public surf

        Public Overrides Function ToString() As [String]
            Return "{" & t.ToString() & "," & enter.ToString() & "," & prim.ToString() & "," & surf.ToString() & "}"
        End Function
    End Class

    Public Class Light
        Public pos
        Public brightness

        Public Sub New()
        End Sub

        Public Sub New(x, y, z, brightness)
            Me.pos = New Vec(x, y, z)
            Me.brightness = brightness
        End Sub
    End Class

    Public Class Surface
        Public color
        Public kd
        Public ks
        Public shine
        Public kt
        Public ior

        Public Sub New()
            color = New Vec(1.0, 0.0, 0.0)
            kd = 1.0
            ks = 0.0
            shine = 0.0
            kt = 0.0
            ior = 1.0
        End Sub

        Public Overrides Function ToString() As [String]
            Return "Surface { color=" & color.ToString() & "}"
        End Function
    End Class

    Public MustInherit Class Primitive
        Public surf

        Public Sub New()
            surf = New Surface()
        End Sub

        Public Sub setColor(r, g, b)
            surf.color = New Vec(r, g, b)
        End Sub

        Public MustOverride Function intersect(ry As Ray) As Isect
    End Class

    Public Class Sphere
        Inherits Primitive
        Private c
        Private r
        Private r2
        Private v
        Private b

        Public Sub New(center, radius)
            MyBase.New()
            c = center
            r = radius
            r2 = radius * radius
            v = New Vec()
            b = New Vec()
        End Sub

        Public Overrides Function intersect(ry As Ray) As Isect
            Dim p
            p = ry.P
            Dim d
            d = ry.D
            v.sub2(c, p)
            Dim b
            b = Vec.dot(v, d)
            Dim dotVV
            dotVV = Vec.dot(v, v)
            Dim disc
            disc = b * b - dotVV + r2
            If disc < 0.0 Then
                Return Nothing
            End If
            disc = Math.Sqrt(disc)
            Dim t
            t = If((b - disc < 0.000001), b + disc, b - disc)
            If t < 0.000001 Then
                Return Nothing
            End If
            Dim ip
            ip = New Isect()
            ip.t = t
            ip.enter = If(dotVV > r2 + 0.000001, 1.0, 0.0)
            ip.prim = Me
            ip.surf = Me.surf
            Return ip
        End Function

        Public Function normal(p)
            Dim r
            r = Vec.sub1(p, c)
            r.normalize()
            Return r
        End Function

        Public Overrides Function ToString() As [String]
            Return "Sphere {" & c.ToString() & "," & Convert.ToString(r) & "}"
        End Function

        Public Function getCenter()
            Return c
        End Function
        Public Sub setCenter(c)
            Me.c = c
        End Sub
    End Class

    Public Class Scene
        Public lights
        Public objects
        Private view

        Public Sub New()
            Me.lights = New ArrayList()
            Me.objects = New ArrayList()
        End Sub

        Public Sub addLight(l)
            Me.lights.Add(l)
        End Sub

        Public Sub addObject(myobject)
            Me.objects.Add(myobject)
        End Sub

        Public Sub setView(view)
            Me.view = view
        End Sub

        Public Function getView()
            Return Me.view
        End Function

        Public Function getLight(number)
            Return DirectCast(Me.lights.ToArray()(number), Light)
        End Function

        Public Function getObject(number)
            Return DirectCast(objects.ToArray()(number), Primitive)
        End Function

        Public Function getLights()
            Return Me.lights.Count
        End Function

        Public Function getObjects()
            Return Me.objects.Count
        End Function

        Public Sub setObject(myobject, pos)
            DirectCast(Me.objects, ArrayList)(pos) = myobject
        End Sub
    End Class

    Public Class RayTracer
        Public scene
        Public lights
        Public prim
        Public view
        Public tRay
        Public Shared alpha
        Public Shared voidVec
        Public L
        Public inter
        Public height
        Public width
        Public datasizes
        Public checksum
        Public size
        Public numobjects

        Public Sub New()
            datasizes = New Integer(9) {}
            datasizes(0) = 5
            datasizes(1) = 10
            datasizes(2) = 15
            datasizes(3) = 20
            datasizes(4) = 25
            datasizes(5) = 30
            datasizes(6) = 35
            datasizes(7) = 40
            datasizes(8) = 45
            datasizes(9) = 50
            RayTracer.alpha = 255 << 24
            tRay = New Ray()
            RayTracer.voidVec = New Vec()
            L = New Vec()
            inter = New Isect()
            checksum = 0
        End Sub

        Public Function createScene()
            Dim x
            x = 0.0
            Dim y
            y = 0.0
            Dim scene
            scene = New Scene()
            Dim p
            Dim nx
            nx = 4.0
            Dim ny
            ny = 4.0
            Dim nz
            nz = 4.0
            Dim i
            i = 0.0
            While i < nx
                Dim j
                j = 0.0
                While j < ny
                    Dim k
                    k = 0.0
                    While k < nz
                        Dim xx
                        xx = 20.0 / (nx - 1.0) * i - 10.0
                        Dim yy
                        yy = 20.0 / (ny - 1.0) * j - 10.0
                        Dim zz
                        zz = 20.0 / (nz - 1.0) * k - 10.0
                        p = New Sphere(New Vec(xx, yy, zz), 3.0)
                        p.setColor(0.0, 0.0, (CDbl(i + j) / CDbl(nx + ny - 2.0)))
                        p.surf.shine = 15.0
                        p.surf.ks = 1.5 - 1.0
                        p.surf.kt = 1.5 - 1.0
                        scene.addObject(p)
                        k = k + 1.0
                    End While
                    j = j + 1.0
                End While
                i = i + 1.0
            End While

            scene.addLight(New Light(100.0, 100.0, -50.0, 1.0))
            scene.addLight(New Light(-100.0, 100.0, -50.0, 1.0))
            scene.addLight(New Light(100.0, -100.0, -50.0, 1.0))
            scene.addLight(New Light(-100.0, -100.0, -50.0, 1.0))
            scene.addLight(New Light(200.0, 200.0, 0.0, 1.0))
            Dim v
            v = New View(New Vec(x, 20.0, -30.0), New Vec(x, y, 0.0), New Vec(0.0, 1.0, 0.0), 1.0, 35.0 * 3.14159265 / 180.0, 1.0)
            scene.setView(v)
            Return scene
        End Function

        Public Sub setScene(scene)
            Dim nLights
            nLights = scene.getLights()
            Dim nObjects
            nObjects = scene.getObjects()
            lights = New Light(nLights - 1) {}
            prim = New Primitive(nObjects - 1) {}
            Dim l
            l = 0
            While l < nLights
                lights(l) = scene.getLight(l)
                l = l + 1
            End While
            Dim o
            o = 0
            While o < nObjects
                prim(o) = scene.getObject(o)
                o = o + 1
            End While
            view = scene.getView()
        End Sub

        Public Function intersect(r, tmax)
            Dim nhits
            nhits = 0
            Dim temp
            temp = New Isect()
            temp.t = 1000000000.0
            Me.inter = temp
            Dim i
            i = 0
            While i < Me.prim.Length
                Dim tp
                tp = Me.prim(i).intersect(r)
                Dim sw
                sw = (tp IsNot Nothing)
                If sw Then
                    Dim isect
                    isect = New Isect()
                    isect.t = tp.t
                    isect.prim = tp.prim
                    isect.surf = tp.surf
                    isect.enter = tp.enter
                    Me.inter = isect
                    nhits = nhits + 1
                End If
                i = i + 1
            End While
            Return If(nhits > 0, True, False)
        End Function

        Public Function trace(level, weight, r)
            If level > 6 Then
                Return New Vec()
            End If

            Dim hit
            hit = intersect(r, 1000000.0)
            If hit Then
                Dim P
                P = r.point(inter.t)
                Dim N
                N = inter.prim.normal(P)
                If Vec.dot(r.D, N) >= 0.0 Then
                    N.negate()
                    'return shade(level, weight, P, N, r.D, inter);
                End If
            End If
            Return voidVec
        End Function

        Public Function SpecularDirection(I, N)
            Dim dotIN
            dotIN = Vec.dot(I, N)
            Dim abs
            abs = If(dotIN > 0, dotIN, (-1 * dotIN))
            Dim r
            r = Vec.comb(1.0 / abs, I, 2.0, N)
            r.normalize()
            Return r
        End Function

        Public Function TransDir(m1, m2, I, N)
            Dim n1
            n1 = If(m1 Is Nothing, 1.0, m1.ior)
            Dim n2
            n2 = If(m2 Is Nothing, 1.0, m2.ior)
            Dim eta
            eta = n1 / n2
            Dim c1
            c1 = -Vec.dot(I, N)
            Dim cs2
            cs2 = 1.0 - eta * eta * (1.0 - c1 * c1)
            If cs2 < 0.0 Then
                Return Nothing
            End If
            Dim r
            r = Vec.comb(eta, I, eta * c1 - Math.Sqrt(cs2), N)
            r.normalize()
            Return r
        End Function

        Public Function Shadow(r, tmax)
            If intersect(r, tmax) Then
                Return 0
            End If
            Return 1
        End Function

        Public Sub render(interval)
            Dim localView
            localView = Me.view
            Dim viewVec
            viewVec = Vec.sub1(localView.atVec, localView.fromVec)
            viewVec.normalize()
            Dim tmpVec
            tmpVec = New Vec(viewVec.x, viewVec.y, viewVec.z)
            Dim dotValue
            dotValue = Vec.dot(localView.upVec, viewVec)
            tmpVec.scale(dotValue)
            Dim upVec
            upVec = Vec.sub1(localView.upVec, tmpVec)
            upVec.normalize()
            Dim leftVec
            leftVec = Vec.cross(localView.upVec, viewVec)
            leftVec.normalize()
            Dim frustrumwidth
            frustrumwidth = localView.dist * Math.Tan(localView.angle)
            upVec.scale(-frustrumwidth)
            Dim scaleValue
            scaleValue = localView.aspect * frustrumwidth
            leftVec.scale(scaleValue)
            Dim r
            r = New Ray(localView.fromVec, RayTracer.voidVec)
            Dim y
            y = interval.yfrom
            Dim nhits
            nhits = 0
            While y < interval.yto
                Dim ylen
                ylen = (2.0 * y) / interval.width - 1.0
                Dim x
                x = 0
                While x < interval.width
                    Dim xlen
                    xlen = (2.0 * x) / interval.width - 1.0
                    r.D = Vec.comb(xlen, leftVec, ylen, upVec)
                    r.D.add1(viewVec)
                    r.D.normalize()
                    Dim hit
                    hit = Me.intersect(r, 1000000.0)
                    If hit Then
                        nhits = nhits + 1
                    End If
                    x = x + 1
                End While
                y = y + 1
            End While
            checksum = checksum + nhits
        End Sub
    End Class

    Public Class JGFRayTracerBench
        Inherits RayTracer
        Public Sub JGFsetsize(size)
            Me.size = size
        End Sub

        Public Sub JGFinitialise()
            height = datasizes(size)
            width = height
            scene = createScene()
            setScene(scene)
            numobjects = scene.getObjects()
        End Sub

        Public Sub JGFapplication()
            Dim interval
            interval = New Interval(0, width, height, 0, height, 1)
            render(interval)
        End Sub

        Public Sub JGFvalidate()
            Dim refval
            refval = New Integer(9) {}
            refval(0) = 12
            refval(1) = 40
            refval(2) = 104
            refval(3) = 170
            refval(4) = 278
            refval(5) = 390
            refval(6) = 552
            refval(7) = 692
            refval(8) = 894
            refval(9) = 1084
            Dim dev
            dev = checksum - refval(size)
            If dev <> 0 Then
                Console.WriteLine("Validation failed")
                Console.WriteLine("Pixel checksum = " & Convert.ToString(checksum))
                Console.WriteLine("Reference value = " + refval(size))
            End If
        End Sub

        Public Sub Test()
            JGFsetsize(4)
            JGFinitialise()
            JGFapplication()
            JGFvalidate()
        End Sub      
    End Class


    Public Class Program
        Public Shared Sub Main(args As String())
            If args.Length < 1 Then
                Console.[Error].WriteLine("You must pass the number of thousands iterations.")
                System.Environment.[Exit](-1)
            End If
            Dim iterations As Integer = Convert.ToInt32(args(0))
            Dim arith As New ArithmethicBenchmark(iterations)
            Console.WriteLine(arith.run())
        End Sub
    End Class
End Namespace
