Namespace Pybench.Aritmethic
    Public Class Chronometer
        Private ticks1 As DateTime, ticks2 As DateTime
        Private stopped As Boolean

        Public Sub Start()
            ticks1 = DateTime.Now
            stopped = False
        End Sub
        Public Sub [Stop]()
            ticks2 = DateTime.Now
            stopped = True
        End Sub

        Private Shared Function TicksToMicroSeconds(t1 As DateTime, t2 As DateTime) As Integer
            Return TicksToMiliSeconds(t1, t2) * 1000
        End Function

        Private Shared Function TicksToMiliSeconds(t1 As DateTime, t2 As DateTime) As Integer
            Dim difference As TimeSpan = t2.Subtract(t1)
            Return (difference.Milliseconds + difference.Seconds * 1000 + difference.Minutes * 60000)
        End Function

        Private Shared Function TicksToSeconds(t1 As DateTime, t2 As DateTime) As Integer
            Dim difference As TimeSpan = t2.Subtract(t1)
            Return (difference.Seconds + difference.Minutes * 60)
        End Function

        Public Function GetMicroSeconds() As Integer
            If stopped Then
                Return TicksToMicroSeconds(ticks1, ticks2)
            End If
            Return TicksToMicroSeconds(ticks1, DateTime.Now)
        End Function

        Public Function GetMiliSeconds() As Integer
            If stopped Then
                Return TicksToMiliSeconds(ticks1, ticks2)
            End If
            Return TicksToMiliSeconds(ticks1, DateTime.Now)
        End Function

        Public Function GetSeconds() As Integer
            If stopped Then
                Return TicksToSeconds(ticks1, ticks2)
            End If
            Return TicksToSeconds(ticks1, DateTime.Now)
        End Function
    End Class

    Public Class BenchMark
        Private iterations As Integer
        Protected microSeconds As Integer

        Public Sub New(iterations As Integer)
            Me.iterations = iterations
        End Sub

        Public Function run() As Integer
            Dim self As BenchMark = Me
            For i As Integer = 0 To iterations - 1
                self.runOneIteration()
            Next
            Return Me.microSeconds
        End Function

        Public Overridable Function runOneIteration() As Object
            Return Nothing
        End Function
    End Class

    Public Class ArithmethicBenchmark
        Inherits BenchMark
        Public Sub New(iterations As Integer)
            MyBase.New(iterations)
        End Sub
        Public Overrides Function runOneIteration() As Object
            Dim chronometer As New Chronometer()
            Dim test As Test = New IfThenElse()
            chronometer.Start()
            test.test()
            chronometer.[Stop]()
            Me.microSeconds = Me.microSeconds + chronometer.GetMicroSeconds()
            Return Nothing
        End Function
    End Class

    Public MustInherit Class Test
        Public MustOverride Sub test()
    End Class

    Public Class IfThenElse
        Inherits Test
        Public Overrides Sub test()
            Dim one
            one = 1
            Dim two
            two = 2
            Dim three
            three = 3
            Dim a
            a = one
            Dim b
            b = two
            Dim c
            c = three
            For x = 1 To 150000 * 3
                If a = 1 Then
                    If b = 2 Then
                        If c <> 3 Then
                            c = 3
                            b = 3
                        Else
                            c = 2
                        End If
                    Else
                        If b = 3 Then
                            b = 2
                            a = 2
                        End If
                    End If
                Else
                    If a = 2 Then
                        a = 3
                    Else
                        a = 1
                    End If
                End If

                If a = 1 Then
                    If b = 2 Then
                        If c <> 3 Then
                            c = 3
                            b = 3
                        Else
                            c = 2
                        End If
                    Else
                        If b = 3 Then
                            b = 2
                            a = 2
                        End If
                    End If
                Else
                    If a = 2 Then
                        a = 3
                    Else
                        a = 1
                    End If
                End If

                If a = 1 Then
                    If b = 2 Then
                        If c <> 3 Then
                            c = 3
                            b = 3
                        Else
                            c = 2
                        End If
                    Else
                        If b = 3 Then
                            b = 2
                            a = 2
                        End If
                    End If
                Else
                    If a = 2 Then
                        a = 3
                    Else
                        a = 1
                    End If
                End If

                If a = 1 Then
                    If b = 2 Then
                        If c <> 3 Then
                            c = 3
                            b = 3
                        Else
                            c = 2
                        End If
                    Else
                        If b = 3 Then
                            b = 2
                            a = 2
                        End If
                    End If
                Else
                    If a = 2 Then
                        a = 3
                    Else
                        a = 1
                    End If
                End If

                If a = 1 Then
                    If b = 2 Then
                        If c <> 3 Then
                            c = 3
                            b = 3
                        Else
                            c = 2
                        End If
                    Else
                        If b = 3 Then
                            b = 2
                            a = 2
                        End If
                    End If
                Else
                    If a = 2 Then
                        a = 3
                    Else
                        a = 1
                    End If
                End If
            Next
        End Sub
    End Class

    Public Class Program
        Public Shared Sub Main(args As String())
            If args.Length < 1 Then
                Console.[Error].WriteLine("You must pass the number of thousands iterations.")
                System.Environment.[Exit](-1)
            End If
            Dim iterations As Integer = Convert.ToInt32(args(0))
            Dim arith As New ArithmethicBenchmark(iterations)
            Console.WriteLine(arith.run())
        End Sub
    End Class
End Namespace
