Namespace Pybench.Aritmethic
    Public Class Chronometer
        Private ticks1 As DateTime, ticks2 As DateTime
        Private stopped As Boolean

        Public Sub Start()
            ticks1 = DateTime.Now
            stopped = False
        End Sub
        Public Sub [Stop]()
            ticks2 = DateTime.Now
            stopped = True
        End Sub

        Private Shared Function TicksToMicroSeconds(t1 As DateTime, t2 As DateTime) As Integer
            Return TicksToMiliSeconds(t1, t2) * 1000
        End Function

        Private Shared Function TicksToMiliSeconds(t1 As DateTime, t2 As DateTime) As Integer
            Dim difference As TimeSpan = t2.Subtract(t1)
            Return (difference.Milliseconds + difference.Seconds * 1000 + difference.Minutes * 60000)
        End Function

        Private Shared Function TicksToSeconds(t1 As DateTime, t2 As DateTime) As Integer
            Dim difference As TimeSpan = t2.Subtract(t1)
            Return (difference.Seconds + difference.Minutes * 60)
        End Function

        Public Function GetMicroSeconds() As Integer
            If stopped Then
                Return TicksToMicroSeconds(ticks1, ticks2)
            End If
            Return TicksToMicroSeconds(ticks1, DateTime.Now)
        End Function

        Public Function GetMiliSeconds() As Integer
            If stopped Then
                Return TicksToMiliSeconds(ticks1, ticks2)
            End If
            Return TicksToMiliSeconds(ticks1, DateTime.Now)
        End Function

        Public Function GetSeconds() As Integer
            If stopped Then
                Return TicksToSeconds(ticks1, ticks2)
            End If
            Return TicksToSeconds(ticks1, DateTime.Now)
        End Function
    End Class

    Public Class BenchMark
        Private iterations As Integer
        Protected microSeconds As Integer

        Public Sub New(iterations As Integer)
            Me.iterations = iterations
        End Sub

        Public Function run() As Integer
            Dim self As BenchMark = Me
            For i As Integer = 0 To iterations - 1
                self.runOneIteration()
            Next
            Return Me.microSeconds
        End Function

        Public Overridable Function runOneIteration() As Object
            Return Nothing
        End Function
    End Class

    Public Class ArithmethicBenchmark
        Inherits BenchMark
        Public Sub New(iterations As Integer)
            MyBase.New(iterations)
        End Sub
        Public Overrides Function runOneIteration() As Object
            Dim chronometer As New Chronometer()
            Dim test As Test = New DictWithFloatKeys()
            chronometer.Start()
            test.test()
            chronometer.[Stop]()
            Me.microSeconds = Me.microSeconds + chronometer.GetMicroSeconds()
            Return Nothing
        End Function
    End Class

    Public MustInherit Class Test
        Public MustOverride Sub test()
    End Class

    Public Class DictWithFloatKeys
        Inherits Test
        Public Shared SIZE = 50

        Public Shared Function Hash(value)
            Dim val
            val = value.GetHashCode()
            If val < 0 Then
                Return (0 - val) Mod SIZE
            End If
            Return val Mod SIZE
        End Function

        Public Overrides Sub test()
            Dim temp

            Dim hash1234
            Dim hash2345
            Dim hash3456
            Dim hash4567
            Dim hash5678
            Dim hash6789

            hash1234 = Hash(1.234)
            hash2345 = Hash(2.345)
            hash3456 = Hash(3.456)
            hash4567 = Hash(4.567)
            hash5678 = Hash(5.678)
            hash6789 = Hash(6.789)

            Dim one
            Dim two
            Dim three
            Dim four
            Dim five
            Dim six

            one = 1
            two = 2
            three = 3
            four = 4
            five = 5
            six = 6

            Dim d
            d = New Object(SIZE - 1) {}

            For i = 1 To 150000
                d(hash1234) = one
                d(hash2345) = two
                d(hash3456) = three
                d(hash4567) = four
                d(hash5678) = five
                d(hash6789) = six

                temp = d(hash1234)
                temp = d(hash2345)
                temp = d(hash3456)
                temp = d(hash4567)
                temp = d(hash5678)
                temp = d(hash6789)

                d(hash1234) = one
                d(hash2345) = two
                d(hash3456) = three
                d(hash4567) = four
                d(hash5678) = five
                d(hash6789) = six

                temp = d(hash1234)
                temp = d(hash2345)
                temp = d(hash3456)
                temp = d(hash4567)
                temp = d(hash5678)
                temp = d(hash6789)

                d(hash1234) = one
                d(hash2345) = two
                d(hash3456) = three
                d(hash4567) = four
                d(hash5678) = five
                d(hash6789) = six

                temp = d(hash1234)
                temp = d(hash2345)
                temp = d(hash3456)
                temp = d(hash4567)
                temp = d(hash5678)
                temp = d(hash6789)

                d(hash1234) = one
                d(hash2345) = two
                d(hash3456) = three
                d(hash4567) = four
                d(hash5678) = five
                d(hash6789) = six

                temp = d(hash1234)
                temp = d(hash2345)
                temp = d(hash3456)
                temp = d(hash4567)
                temp = d(hash5678)
                temp = d(hash6789)

                d(hash1234) = one
                d(hash2345) = two
                d(hash3456) = three
                d(hash4567) = four
                d(hash5678) = five
                d(hash6789) = six

                temp = d(hash1234)
                temp = d(hash2345)
                temp = d(hash3456)
                temp = d(hash4567)
                temp = d(hash5678)
                temp = d(hash6789)
            Next
        End Sub
    End Class

    Public Class Program
        Public Shared Sub Main(args As String())
            If args.Length < 1 Then
                Console.[Error].WriteLine("You must pass the number of thousands iterations.")
                System.Environment.[Exit](-1)
            End If
            Dim iterations As Integer = Convert.ToInt32(args(0))
            Dim arith As New ArithmethicBenchmark(iterations)
            Console.WriteLine(arith.run())
        End Sub
    End Class
End Namespace
