Namespace Pybench.Aritmethic
	Public Class Chronometer
		Private ticks1 As DateTime, ticks2 As DateTime
		Private stopped As Boolean

		Public Sub Start()
			ticks1 = DateTime.Now
			stopped = False
		End Sub
		Public Sub [Stop]()
			ticks2 = DateTime.Now
			stopped = True
		End Sub

		Private Shared Function TicksToMicroSeconds(t1 As DateTime, t2 As DateTime) As Integer
			Return TicksToMiliSeconds(t1, t2) * 1000
		End Function

		Private Shared Function TicksToMiliSeconds(t1 As DateTime, t2 As DateTime) As Integer
			Dim difference As TimeSpan = t2.Subtract(t1)
			Return (difference.Milliseconds + difference.Seconds * 1000 + difference.Minutes * 60000)
		End Function

		Private Shared Function TicksToSeconds(t1 As DateTime, t2 As DateTime) As Integer
			Dim difference As TimeSpan = t2.Subtract(t1)
			Return (difference.Seconds + difference.Minutes * 60)
		End Function

		Public Function GetMicroSeconds() As Integer
			If stopped Then
				Return TicksToMicroSeconds(ticks1, ticks2)
			End If
			Return TicksToMicroSeconds(ticks1, DateTime.Now)
		End Function

		Public Function GetMiliSeconds() As Integer
			If stopped Then
				Return TicksToMiliSeconds(ticks1, ticks2)
			End If
			Return TicksToMiliSeconds(ticks1, DateTime.Now)
		End Function

		Public Function GetSeconds() As Integer
			If stopped Then
				Return TicksToSeconds(ticks1, ticks2)
			End If
			Return TicksToSeconds(ticks1, DateTime.Now)
		End Function
	End Class

	Public Class BenchMark
		Private iterations As Integer
		Protected microSeconds As Integer

		Public Sub New(iterations As Integer)
			Me.iterations = iterations
		End Sub

		Public Function run() As Integer
			Dim self As BenchMark = Me
			For i As Integer = 0 To iterations - 1
				self.runOneIteration()
			Next
			Return Me.microSeconds
		End Function

		Public Overridable Function runOneIteration() As Object
			Return Nothing
		End Function
	End Class

	Public Class ArithmethicBenchmark
		Inherits BenchMark
		Public Sub New(iterations As Integer)
			MyBase.New(iterations)
		End Sub
		Public Overrides Function runOneIteration() As Object
			Dim chronometer As New Chronometer()
			Dim test As Test = New CreateNewInstances()
			chronometer.Start()
			test.test()
			chronometer.[Stop]()
			Me.microSeconds = Me.microSeconds + chronometer.GetMicroSeconds()
			Return Nothing
		End Function
	End Class

	Public MustInherit Class Test
		Public MustOverride Sub test()
	End Class

	Public Class Root
		Public a

		Public Sub New()
		End Sub

		Public Sub New(a)
			Me.a = a
		End Sub
	End Class

	Public Class C
		Inherits Root
		Public Shared b
		Public Shared c
	End Class

	Public Class D
		Inherits Root
		Public b
		Public c

		Public Sub New(a,b,c)
			MyBase.New(DirectCast(a, Object))
			Me.b = b
			Me.c = c
		End Sub
	End Class

	Public Class E
		Inherits Root
		Public b
		Public c
		Public d
		Public e
		Public f

		Public Sub New(a,b,c)
			MyBase.New(DirectCast(a, Object))
			Me.b = b
			Me.c = c
			Me.d = a
			Me.e = b
			Me.f = c
		End Sub
	End Class

	Public Class CreateNewInstances
		Inherits Test
		Public Overrides Sub test()
			Dim three 
			three = 3
			Dim four 
			four = 4
			Dim i
			For i = 1 To 800000
				Dim o 
				o = New C()
				Dim o1 
				o1 = New C()
				Dim o2 
				o2 = New C()
				Dim p 
				p = New D(i, i, three)
				Dim p1 
				p1 = New D(i, i, three)
				Dim p2 
				p2 = New D(i, three, three)
				Dim p3 
				p3 = New D(three, i, three)
				Dim p4 
				p4 = New D(i, i, i)
				Dim p5 
				p5 = New D(three, i, three)
				Dim p6 
				p6 = New D(i, i, i)
				Dim q 
				q = New E(i, i, three)
				Dim q1 
				q1 = New E(i, i, three)
				Dim q2 
				q2 = New E(i, i, three)
				Dim q3 
				q3 = New E(i, i, four)				
			Next
		End Sub
	End Class

	Public Class Program
		Public Shared Sub Main(args As String())
			If args.Length < 1 Then
				Console.[Error].WriteLine("You must pass the number of thousands iterations.")
				System.Environment.[Exit](-1)
			End If
			Dim iterations As Integer = Convert.ToInt32(args(0))
			Dim arith As New ArithmethicBenchmark(iterations)
			Console.WriteLine(arith.run())
		End Sub
	End Class
End Namespace
