Imports System

Namespace Points
	Public Class Chronometer
		Private ticks1 As DateTime, ticks2 As DateTime
		Private stopped As Boolean

		Public Sub Start()
			ticks1 = DateTime.Now
			stopped = False
		End Sub
		Public Sub [Stop]()
			ticks2 = DateTime.Now
			stopped = True
		End Sub

		Private Shared Function TicksToMicroSeconds(t1 As DateTime, t2 As DateTime) As Integer
			Return TicksToMiliSeconds(t1, t2) * 1000
		End Function

		Private Shared Function TicksToMiliSeconds(t1 As DateTime, t2 As DateTime) As Integer
			Dim difference As TimeSpan = t2.Subtract(t1)
			Return (difference.Milliseconds + difference.Seconds * 1000 + difference.Minutes * 60000)
		End Function

		Private Shared Function TicksToSeconds(t1 As DateTime, t2 As DateTime) As Integer
			Dim difference As TimeSpan = t2.Subtract(t1)
			Return (difference.Seconds + difference.Minutes * 60)
		End Function

		Public Function GetMicroSeconds() As Integer
			If stopped Then
				Return TicksToMicroSeconds(ticks1, ticks2)
			End If
			Return TicksToMicroSeconds(ticks1, DateTime.Now)
		End Function

		Public Function GetMiliSeconds() As Integer
			If stopped Then
				Return TicksToMiliSeconds(ticks1, ticks2)
			End If
			Return TicksToMiliSeconds(ticks1, DateTime.Now)
		End Function

		Public Function GetSeconds() As Integer
			If stopped Then
				Return TicksToSeconds(ticks1, ticks2)
			End If
			Return TicksToSeconds(ticks1, DateTime.Now)
		End Function
	End Class

	Public Class BenchMark
		Private iterations As Integer
		Protected microSeconds As Integer

		Public Sub New(iterations As Integer)
			Me.iterations = iterations
		End Sub

		Public Function run() As Integer
			Dim self As BenchMark = Me
			For i As Integer = 0 To iterations - 1
				self.runOneIteration()
			Next
			Return Me.microSeconds
		End Function

		Public Overridable Function runOneIteration() As Object
			Return Nothing
		End Function
	End Class

	Public Class ArithmethicBenchmark
		Inherits BenchMark
		Public Sub New(iterations As Integer)
			MyBase.New(iterations)
		End Sub
		Public Overrides Function runOneIteration() As Object
			Dim chronometer As New Chronometer()
			Dim test As Test = New PointsBench()
			chronometer.Start()
			test.test()
			chronometer.[Stop]()
			Me.microSeconds = Me.microSeconds + chronometer.GetMicroSeconds()
			Return Nothing
		End Function
	End Class

	Public MustInherit Class Test
		Public MustOverride Sub test()
	End Class


	Public Class Node
		Public data
		Public [next]
		Public Sub New(data, [next])
			Me.data = data
			Me.[next] = [next]
		End Sub
	End Class

	Public Class Point3D
		Public x
		Public y
		Public z
		Public dimensions
		Public Sub New(x, y, z, dimensions)
			Me.x = x
			Me.y = y
			Me.z = z
			Me.dimensions = dimensions
		End Sub
	End Class

	Public Class Point2D
		Public x
		Public y
		Public dimensions
		Public Sub New(x, y, dimensions)
			Me.x = x
			Me.y = y
			Me.dimensions = dimensions
		End Sub
	End Class

	Public Class PointsBench
		Inherits Test
		Private Function createPoint(dimensions, x, y, z)
			Dim point
			If dimensions = 2 Then
				point = New Point2D(x, y, dimensions)
			Else
				point = New Point3D(x, y, z, 3)
			End If
			Return point
		End Function


		Private Function createPoints(number)
			Dim i
			Dim list 
			Dim point
			i = 1
			point = createPoint(3, 0, 0, 0)
			list = New Node(point, 0)
			While i < number
				point = createPoint(i Mod 2 + 2, number / 2 - i, i, i)
				list = New Node(point, list)
				i = i + 1
			End While
			Return list
		End Function


		Private Function positiveX(list, n)
			Dim i
			Dim l
			Dim result
			i = 0
			result = 0
			l = list
			While i < n
				If l.data.x >= 0 Then
					result = New Node(l.data, result)
				End If
				l = l.[next]
				i = i + 1
			End While
			Return result
		End Function

		Private Function distance3D(point)
			Dim value
			value = 2147483647
			If point.dimensions = 3 Then
				value = point.x * point.x + point.y * point.y + point.z * point.z
			End If
			Return value
		End Function

		Private Function closestToOrigin3D(list, n)
			Dim i, minDistance
			Dim l
			Dim point3D

			point3D = createPoint(3, 0, 0, 0)
			minDistance = 2147483647
			l = list
			i = 0
			While i < n
				If distance3D(l.data) < minDistance Then
					minDistance = distance3D(l.data)
					point3D = l.data
				End If
				l = l.[next]
				i = i + 1
			End While
			Return point3D
		End Function

		Public Overrides Sub test()
			Dim numberOfPoints
			Dim list
			Dim positive 
			Dim point
			numberOfPoints = 10000
			list = createPoints(numberOfPoints)
			positive = positiveX(list, numberOfPoints)
			point = closestToOrigin3D(list, numberOfPoints)
		End Sub	
	End Class

	Public Class Program
		Public Shared Sub Main(args As String())
			If args.Length < 1 Then
				Console.[Error].WriteLine("You must pass the number of thousands iterations.")
				System.Environment.[Exit](-1)
			End If
			Dim iterations As Integer = Convert.ToInt32(args(0))
			Dim arith As New ArithmethicBenchmark(iterations)
			Console.WriteLine(arith.run())
		End Sub
	End Class
End Namespace
