'A port to StaDyn (dynamic) of the pystone benchmark using duck typing.


Imports System.Collections

Namespace StaDyn.Dynamic
    Public Class Chronometer
        Private ticks1 As DateTime, ticks2 As DateTime
        Private stopped As Boolean

        Public Sub Start()
            ticks1 = DateTime.Now
            stopped = False
        End Sub
        Public Sub [Stop]()
            ticks2 = DateTime.Now
            stopped = True
        End Sub

        Private Shared Function TicksToMicroSeconds(t1 As DateTime, t2 As DateTime) As Integer
            Return TicksToMiliSeconds(t1, t2) * 1000
        End Function

        Private Shared Function TicksToMiliSeconds(t1 As DateTime, t2 As DateTime) As Integer
            Dim difference As TimeSpan = t2.Subtract(t1)
            Return (difference.Milliseconds + difference.Seconds * 1000 + difference.Minutes * 60000)
        End Function

        Private Shared Function TicksToSeconds(t1 As DateTime, t2 As DateTime) As Integer
            Dim difference As TimeSpan = t2.Subtract(t1)
            Return (difference.Seconds + difference.Minutes * 60)
        End Function

        Public Function GetMicroSeconds() As Integer
            If stopped Then
                Return TicksToMicroSeconds(ticks1, ticks2)
            End If
            Return TicksToMicroSeconds(ticks1, DateTime.Now)
        End Function

        Public Function GetMiliSeconds() As Integer
            If stopped Then
                Return TicksToMiliSeconds(ticks1, ticks2)
            End If
            Return TicksToMiliSeconds(ticks1, DateTime.Now)
        End Function

        Public Function GetSeconds() As Integer
            If stopped Then
                Return TicksToSeconds(ticks1, ticks2)
            End If
            Return TicksToSeconds(ticks1, DateTime.Now)
        End Function
    End Class

    Public Class BenchMark
        Private iterations As Integer
        Protected microSeconds As Integer

        Public Sub New(iterations As Integer)
            Me.iterations = iterations
        End Sub

        Public Function run() As Integer
            Dim self As BenchMark = Me
            For i As Integer = 0 To iterations - 1
                self.runOneIteration()
            Next
            Return Me.microSeconds
        End Function

        Public Overridable Function runOneIteration() As Object
            Return Nothing
        End Function
    End Class

    Public Class ArithmethicBenchmark
        Inherits BenchMark
        Public Sub New(iterations As Integer)
            MyBase.New(iterations)
        End Sub
        Public Overrides Function runOneIteration() As Object
            Dim chronometer As New Chronometer()
            Dim pystone__1 As New Pystone()

            chronometer.Start()
            Dim value As Object = Pystone.pystones(5000)
            chronometer.[Stop]()

            Me.microSeconds = Me.microSeconds + chronometer.GetMicroSeconds()
            Return value
        End Function
    End Class

    Public Class Idents
        Public Shared Ident1 = 1
        Public Shared Ident2 = 2
        Public Shared Ident3 = 3
        Public Shared Ident4 = 4
        Public Shared Ident5 = 5
    End Class

    Public Class Record
        Public PtrComp
        Public Discr
        Public EnumComp
        Public IntComp
        Public StringComp

        Public Sub New(Discr, EnumComp, IntComp, StringComp)
            Me.Discr = Discr
            Me.EnumComp = EnumComp
            Me.IntComp = IntComp
            Me.StringComp = StringComp
        End Sub

        Public Function copy()
            Dim record = New Record(Me.Discr, Me.EnumComp, Me.IntComp, Me.StringComp)
            record.PtrComp = Me.PtrComp
            Return record
        End Function
    End Class

    Public Class Pystone
        Public Shared IntGlob = 0
        Public Shared BoolGlob = False
        Public Shared Char1Glob = ControlChars.NullChar
        Public Shared Char2Glob = ControlChars.NullChar
        Public Shared Array1Glob = New Integer(50) {}
        Public Shared Array2Glob = New Integer(50)() {}
        Public Shared PtrGlb
        Public Shared PtrGlbNext

        Public Sub New()
			Pystone.IntGlob = 0
			Pystone.BoolGlob = False
			Pystone.Char1Glob = ControlChars.NullChar
			Pystone.Char2Glob = ControlChars.NullChar
            For i As Integer = 0 To 50
                Pystone.Array2Glob(i) = New Integer(50) {}
            Next
        End Sub

        Public Shared Function pystones(loops)
            Return Pystone.Proc0(loops)
        End Function

        Public Shared Function Proc0(loops)
            Dim benchtime
			benchtime = 0
            Dim nulltime
			nulltime = 0
            Dim String1Loc
            Dim chronometer As New Chronometer()
            chronometer.Start()
            For i As Integer = 0 To loops - 1
				Next
            chronometer.Stop()
            nulltime = chronometer.GetMiliSeconds()

            PtrGlbNext = New Record(0, Idents.Ident1, 0, "")
            PtrGlb = New Record(0, Idents.Ident1, 0, "")
            PtrGlb.PtrComp = PtrGlbNext
            PtrGlb.Discr = Idents.Ident1
            PtrGlb.EnumComp = Idents.Ident3
            PtrGlb.IntComp = 40
            PtrGlb.StringComp = "DHRYSTONE PROGRAM, SOME STRING"
            String1Loc = "DHRYSTONE PROGRAM, 1'ST STRING"
            Array2Glob(8)(7) = 10

            chronometer.Start()
            For i As Integer = 0 To loops - 1
                Pystone.DoWork()
            Next
            chronometer.[Stop]()
            benchtime = chronometer.GetMiliSeconds()

            Dim result = New Double(1) {}
            result(0) = benchtime
            result(1) = If((benchtime <> 0), (loops / benchtime), 0)
            Return result
        End Function

        Private Shared Sub DoWork()
			Dim EnumLoc
            Dim String1Loc
			String1Loc = "DHRYSTONE PROGRAM, 1'ST STRING"
            Dim String2Loc
            Dim IntLoc1
            Dim IntLoc2
            Dim IntLoc3
            Dim CharIndex			
			EnumLoc = 0            
            IntLoc1 = 0
            IntLoc2 = 0
            IntLoc3 = 0
			CharIndex = "A"c
            

            Proc5()
            Proc4()
            IntLoc1 = 2
            IntLoc2 = 3
            String2Loc = "DHRYSTONE PROGRAM, 2'ND STRING"
            EnumLoc = Idents.Ident2
            BoolGlob = Not Func2(String1Loc, String2Loc)
            While IntLoc1 < IntLoc2
                IntLoc3 = 5 * IntLoc1 - IntLoc2
                IntLoc3 = Proc7(IntLoc1, IntLoc2)
                IntLoc1 = IntLoc1 + 1
            End While
            Proc8(Array1Glob, Array2Glob, IntLoc1, IntLoc3)
            PtrGlb = Proc1(PtrGlb)
            CharIndex = "A"c
            While CharIndex <= Char2Glob
                If EnumLoc = Func1(CharIndex, "C"c) Then
                    EnumLoc = Proc6(Idents.Ident1)
                End If                
                CharIndex = ChrW(AscW(CharIndex) + 1)
            End While
            IntLoc3 = IntLoc2 * IntLoc1
            IntLoc2 = IntLoc3 / IntLoc1
            IntLoc2 = 7 * (IntLoc3 - IntLoc2) - IntLoc1
            IntLoc1 = Proc2(IntLoc1)
        End Sub

        Public Shared Function Proc1(PtrParIn)
            Dim NextRecord 
			NextRecord = PtrGlb.copy()
            PtrParIn.PtrComp = NextRecord
            PtrParIn.IntComp = 5
            NextRecord.IntComp = PtrParIn.IntComp
            NextRecord.PtrComp = PtrParIn.PtrComp
            NextRecord.PtrComp = Proc3(NextRecord.PtrComp)
            If NextRecord.Discr = Idents.Ident1 Then
                NextRecord.IntComp = 6
                NextRecord.EnumComp = Proc6(PtrParIn.EnumComp)
                NextRecord.PtrComp = PtrGlb.PtrComp
                NextRecord.IntComp = Proc7(NextRecord.IntComp, 10)
            Else
                PtrParIn = NextRecord.copy()
            End If
            NextRecord.PtrComp = Nothing
            Return PtrParIn
        End Function

        Public Shared Function Proc2(IntParIO)
            Dim IntLoc
            Dim EnumLoc
			EnumLoc = Idents.Ident5

            IntLoc = IntParIO + 10
            While True
                If Pystone.Char1Glob = "A"c Then
                    IntLoc = IntLoc - 1
                    IntParIO = IntLoc - Pystone.IntGlob
                    EnumLoc = Idents.Ident1
                End If
                If EnumLoc = Idents.Ident1 Then
                    Return IntParIO
                End If
            End While
            Return IntParIO
        End Function

        Public Shared Function Proc3(PtrParOut)
            If PtrGlb IsNot Nothing Then
                PtrParOut = PtrGlb.PtrComp
            Else
                IntGlob = 100
            End If
            PtrGlb.IntComp = Proc7(10, IntGlob)
            Return PtrParOut
        End Function

        Public Shared Sub Proc4()		
            Dim BoolLoc
			BoolLoc = (Char1Glob = "A"c)
            BoolLoc = (BoolLoc OrElse BoolGlob)
            Char2Glob = "B"c
        End Sub

        Public Shared Sub Proc5()
            Char1Glob = "A"c
            BoolGlob = False
        End Sub

        Public Shared Function Proc6(EnumParIn)
            Dim EnumParOut
            EnumParOut = EnumParIn
            If Not Func3(EnumParIn) Then
                EnumParOut = Idents.Ident4
            End If
            If EnumParIn = Idents.Ident1 Then
                EnumParOut = Idents.Ident1
            Else
                If EnumParIn = Idents.Ident2 Then
                    If IntGlob > 100 Then
                        EnumParOut = Idents.Ident1
                    Else
                        EnumParOut = Idents.Ident4
                    End If
                Else
                    If EnumParIn = Idents.Ident3 Then
                        EnumParOut = Idents.Ident2
                    Else
                        If EnumParIn = Idents.Ident4 Then
                        Else
                            If EnumParIn = Idents.Ident5 Then
                                EnumParOut = Idents.Ident3
                            End If
                        End If
                    End If
                End If
            End If
            Return EnumParOut
        End Function

        Public Shared Function Proc7(IntParI1, IntParI2)
            Dim IntParOut
            Dim IntLoc
            IntLoc = IntParI1 + 2
            IntParOut = IntParI2 + IntLoc
            Return IntParOut
        End Function

        Public Shared Sub Proc8(Array1Par, Array2Par, IntParI1, IntParI2)
            Dim IntLoc
            IntLoc = IntParI1 + 5
            Array1Par(IntLoc) = IntParI2
            Array1Par(IntLoc + 1) = Array1Par(IntLoc)
            Array1Par(IntLoc + 30) = IntLoc
            For IntIndex As Integer = IntLoc To IntLoc + 1
                Array2Par(IntLoc)(IntIndex) = IntLoc
            Next
            Array2Par(IntLoc)(IntLoc - 1) = Array2Par(IntLoc)(IntLoc - 1) + 1
            Array2Par(IntLoc + 20)(IntLoc) = Array1Par(IntLoc)
            Pystone.IntGlob = 5
        End Sub

        Public Shared Function Func1(CharPar1, CharPar2)
            Dim CharLoc1
            Dim CharLoc2
            CharLoc1 = CharPar1
            CharLoc2 = CharLoc1
            If CharLoc2 <> CharPar2 Then
                Return Idents.Ident1
            Else
                Return Idents.Ident2
            End If
        End Function

        Public Shared Function Func2(StrParI1, StrParI2)
            Dim IntLoc
            Dim CharLoc
			CharLoc = " "c
            IntLoc = 1
            While IntLoc <= 1
                If Func1(StrParI1.ToCharArray()(IntLoc), StrParI2.ToCharArray()(IntLoc + 1)) = Idents.Ident1 Then
                    CharLoc = "A"c
                    IntLoc = IntLoc + 1
                End If
            End While
            If (CharLoc >= "W"c) AndAlso (CharLoc <= "Z"c) Then
                IntLoc = 7
            End If
            If CharLoc = "X"c Then
                Return True
            Else
                If StrParI1.CompareTo(StrParI2) > 0 Then
                    IntLoc = IntLoc + 7
                    Return True
                Else
                    Return False
                End If
            End If
        End Function

        Public Shared Function Func3(EnumParIn)
            Dim EnumLoc
            EnumLoc = EnumParIn
            If EnumLoc = Idents.Ident3 Then
                Return True
            End If
            Return False
        End Function
    End Class

    Public Class Program
        Public Shared Sub Main(args As String())
            If args.Length < 1 Then
                Console.[Error].WriteLine("You must pass the number of thousands iterations.")
                System.Environment.[Exit](-1)
            End If
            Dim iterations As Integer = Convert.ToInt32(args(0))
            Dim arith As New ArithmethicBenchmark(iterations)
            Console.WriteLine(arith.run())
        End Sub
    End Class
End Namespace
