import sys, time, random, math
from time import clock
TRUE = 1
FALSE = 0

class Vec:
	def __init__(self,a=None,b=None,c=None):
		if a == None:
			self.x = 0.0
			self.y = 0.0
			self.z = 0.0
		else:			
			self.x = a
			self.y = b
			self.z = c
	
	def add1(self,a):
		self.x = a.x + self.x
		self.y = a.y + self.y
		self.z = a.z + self.z
	
	@staticmethod
	def adds(s,a,b):
		return Vec(s * a.x + b.x, s * a.y + b.y, s * a.z + b.z)	

	def adds(self,s,b):	
		self.x = self.x + (s * b.x)
		self.y = self.y + (s * b.y)
		self.z = self.z + (s * b.z)

	@staticmethod
	def sub1(a,b):
		return Vec(a.x - b.x, a.y - b.y, a.z - b.z)
	
	def sub2(self,a,b):
		self.x = a.x - b.x
		self.y = a.y - b.y
		self.z = a.z - b.z	
	
	@staticmethod
	def mult1(a,b):
		return Vec(a.x * b.x, a.y * b.y, a.z * b.z)

	@staticmethod
	def cross(a,b):
		return Vec(a.y * b.z - a.z * b.y, a.z * b.x - a.x * b.z, a.x * b.y - a.y * b.x)
	
	@staticmethod
	def dot(a,b):
		return a.x * b.x + a.y * b.y + a.z * b.z
	
	@staticmethod
	def comb(a,A,b,B):
		return Vec(a * A.x + b * B.x, a * A.y + b * B.y, a * A.z + b * B.z)
	
	def comb2(self,a,A,b,B):
		self.x = a * A.x + b * B.x
		self.y = a * A.y + b * B.y
		self.z = a * A.z + b * B.z	

	def scale(self,t):
		self.x = t * self.x
		self.y = t * self.y
		self.z = t * self.z
	
	def negate(self):
		self.x = -1.0 * self.x
		self.y = -1.0 * self.y
		self.z = -1.0 * self.z
	
	def normalize(self):
		length = math.sqrt(self.x * self.x + self.y * self.y + self.z * self.z)
		while length > 0.0:
			self.x = self.x / length
			self.y = self.y / length
			self.z = self.z / length
			length = 0.0
		return length
		
	def __str__(self):
		return "<" + str(self.x) + "," + str(self.y) + "," + str(self.z) + ">"

class View:
	def __init__(self,fromVec,atVec,upVec,dist,angle,aspect):
		self.fromVec = fromVec
		self.atVec = atVec
		self.upVec = upVec
		self.dist = dist
		self.angle = angle
		self.aspect = aspect

class Ray:
	def __init__(self,pnt=None,dir=None):
		if pnt == None:
			self.P = Vec()
			self.D = Vec()
		else:
			self.P = Vec()
			self.P.add1(pnt)
			self.D = Vec()
			self.D.add1(dir)
			self.D.normalize()
			
	def point(self,t):
		return Vec(self.P.x + self.D.x * t, self.P.y + self.D.y * t, self.P.z + self.D.z * t)
	
	def __str__(self):
		return "" + str(self.P) + " -> " + str(self.P) + ""

class Interval:	
	def __init__(self,number,width,height,yfrom,yto,total):
		self.number = number
		self.width = width
		self.height = height
		self.yfrom = yfrom
		self.yto = yto
		self.total = total
	
class Isect:
	def __init__(self):
		self.t = None
		self.enter = None
		self.prim = None
		self.surf = None
		
	def __str__(self):
		return "" + str(self.t) + "," + str(self.enter) + "," + str(self.prim) + "," + str(self.surf) + ""
		
class Light:
	def __init__(self,x=None,y=None,z=None,brightness=None):
		if x == None:
			self.pos = None
			self.brightness = None
		else:
			self.pos = Vec(x,y,z)
			self.brightness = brightness
		
class Surface:
	def __init__(self):
		self.color = Vec(1.0, 0.0, 0.0)
		self.kd = 1.0
		self.ks = 0.0
		self.shine = 0.0
		self.kt = 0.0
		self.ior = 1.0

	def __str__(self):
		return "Surface  color=" + str(color) + ""

class Sphere:
	def __init__(self,center,radius):		
		self.c = center
		self.r = radius
		self.r2 = radius * radius
		self.v = Vec()
		self.b = Vec()
		self.surf = Surface()
	
	def setColor(self,r,g,b):
		self.surf.color = Vec(r, g, b)
		
	def intersect(self,ry):
		p = ry.P
		d = ry.D	   			
		self.v.sub2(self.c, p)		
		b = Vec.dot(self.v, d)
		dotVV = Vec.dot(self.v, self.v)
		disc = b * b - dotVV + self.r2				
		if disc < 0.0:
			return None														
		disc = math.sqrt(disc)
		t = (b + disc if (b - disc) < 0.000001 else b - disc)
		if t < 0.000001:
			return None
		ip = Isect()
		ip.t = t						
		ip.enter = (1.0 if dotVV > (self.r2 + 0.000001) else 0.0)
		ip.prim = self
		ip.surf = self.surf
		return ip

	def normal(self,p):
		r = Vec.sub1(p, self.c)
		r.normalize()
		return r	

	def getCenter(self):
		return self.c
		
	def setCenter(self,c):
		self.c = c
		
	def __str__(self):
		return "Sphere " + str(self.c) + "," + str(self.r) + ""

class Scene:
	def __init__(self):
		self.lights = []
		self.objects = []
		self.view = None

	def addLight(self,l):
		self.lights.append(l)
	
	def addObject(self,myobject):
		self.objects.append(myobject)
	
	def setView(self,view):	
		self.view = view
	
	def getView(self):
		return self.view
	
	def getLight(self,number):
		return self.lights[number]
	
	def getObject(self,number):
		return self.objects[number]	

	def getLights(self):
		return len(self.lights)

	def getObjects(self):
		return len(self.objects)
	
	def setObject(self,myobject,pos):
		self.objects[pos] = myobject

class RayTracer:	
	def __init__(self):
		self.datasizes = []
		self.datasizes.append(5)
		self.datasizes.append(10)
		self.datasizes.append(15)
		self.datasizes.append(20)
		self.datasizes.append(25)
		self.datasizes.append(30)
		self.datasizes.append(35)
		self.datasizes.append(40)
		self.datasizes.append(45)
		self.datasizes.append(50)
		self.datasizes.append(150)
		RayTracer.alpha = 255 << 24
		self.tRay = Ray()
		RayTracer.voidVec = Vec()
		self.L = Vec()
		self.inter = Isect()
		self.checksum = 0
		
		self.size = None
		self.numobjects = None
		self.height = None
		self.width = None
		self.view = None
		self.prim = None
		self.lights = None
		self.scene = None
		
	def createScene(self):
		x = 0.0
		y = 0.0
		scene = Scene()
		nx = 4.0
		ny = 4.0
		nz = 4.0
		i = 0.0			
		while i < nx:		
			j = 0.0
			while j < ny:			
				k = 0.0
				while k < nz:				
					xx = 20.0 / (nx - 1.0) * i - 10.0
					yy = 20.0 / (ny - 1.0) * j - 10.0
					zz = 20.0 / (nz - 1.0) * k - 10.0												
					p = Sphere(Vec(xx, yy, zz), 3.0)					
					p.setColor(0.0, 0.0, (i + j) / (nx + ny - 2.0))					
					p.surf.shine = 15.0
					p.surf.ks = 1.5 - 1.0
					p.surf.kt = 1.5 - 1.0						
					scene.addObject(p)						
					k = k + 1.0				
				j = j + 1.0			
			i = i + 1.0		

		scene.addLight(Light(100.0, 100.0, -50.0, 1.0))
		scene.addLight(Light(-100.0, 100.0, -50.0, 1.0))
		scene.addLight(Light(100.0, -100.0, -50.0, 1.0))
		scene.addLight(Light(-100.0, -100.0, -50.0, 1.0))
		scene.addLight(Light(200.0, 200.0, 0.0, 1.0))
		v = View(Vec(x, 20.0, -30.0), Vec(x, y, 0.0), Vec(0.0, 1.0, 0.0), 1.0, 35.0 * 3.14159265 / 180.0, 1.0)
		scene.setView(v)
		return scene
		
	def setScene(self,scene):		
		nLights = scene.getLights()
		nObjects = scene.getObjects()		
		self.lights = {}
		self.prim = {}
		l = 0
		while l < nLights:
			self.lights[l] = scene.getLight(l)
			l = l + 1		
		o = 0
		while o < nObjects:		
			self.prim[o] = scene.getObject(o)
			o = o + 1		
		self.view = scene.getView()
		
	def intersect(self,r,tmax):		
		nhits = 0
		temp =  Isect()			
		temp.t = 1e9	
		self.inter = temp					
		i = 0											
		while i < len(self.prim):
			tp = self.prim[i].intersect(r)
			sw = (tp != None)					
			if(sw == TRUE):
				isect = Isect()							
				isect.t = tp.t															
				isect.prim = tp.prim
				isect.surf = tp.surf
				isect.enter = tp.enter														
				self.inter = isect							
				nhits = nhits + 1											
			i = i + 1
		return (TRUE if nhits > 0 else FALSE)
		
	def trace(self,level,weight,r):
		if level > 6:
			return Vec()
		hit = self.intersect(r, 1e6)
		if hit == TRUE:
			P = r.point(self.inter.t)
			N = self.inter.prim.normal(P)
			if Vec.dot(r.D, N) >= 0.0:
				N.negate()
		return voidVec
		
	def SpecularDirection(self,I,N):
		dotIN = Vec.dot(I, N)
		abs =  (dotIN if dotIN > 0 else -1 * dotIN)
		r = Vec.comb(1.0 / abs, I, 2.0, N)
		r.normalize()
		return r
		
	def TransDir(self,m1,m2,I,N):
		n1 = (1.0 if m1 == None else m1.ior)
		n2 = (1.0 if m2 == None else m2.ior)
		eta = n1 / n2
		c1 = -1 * Vec.dot(I, N)
		cs2 = 1.0 - eta * eta * (1.0 - c1 * c1)
		if cs2 < 0.0:
			return None
		r = Vec.comb(eta, I, eta * c1 - math.sqrt(cs2), N)
		r.normalize()
		return r

	def Shadow(self,r,tmax):
		if intersect(r,tmax) == TRUE:
			return 0
		return 1
	
	def render(self,interval):
		localView = self.view			
		viewVec = Vec.sub1(localView.atVec, localView.fromVec)											
		viewVec.normalize()			
		tmpVec = Vec(viewVec.x, viewVec.y, viewVec.z)			
		dotValue = Vec.dot(localView.upVec, viewVec)			
		tmpVec.scale(dotValue)			
		upVec = Vec.sub1(localView.upVec, tmpVec)
		upVec.normalize()
		leftVec = Vec.cross(localView.upVec, viewVec)
		leftVec.normalize()		
		frustrumwidth = localView.dist * math.tan(localView.angle)
		upVec.scale(-frustrumwidth)			
		scaleValue = localView.aspect * frustrumwidth
		leftVec.scale(scaleValue)						
		r = Ray(localView.fromVec, RayTracer.voidVec)
		y = interval.yfrom					
		nhits = 0					
		while y < interval.yto:
			ylen = (2.0 * y) / interval.width - 1.0
			x = 0
			while x < interval.width:
				xlen = (2.0 * x) / interval.width - 1.0					
				r.D = Vec.comb(xlen, leftVec, ylen, upVec)	 					
				r.D.add1(viewVec) 									
				r.D.normalize()																
				hit = self.intersect(r, 1e6)										
				if hit == TRUE:
					nhits = nhits + 1
				x = x + 1			
			y = y + 1		
		self.checksum = self.checksum + nhits	
		
		
class JGFRayTracerBench(RayTracer):
	def JGsetsize(self,size):
		self.size = size
		
	def JGinitialise(self):
		self.height = self.datasizes[self.size]
		self.width = self.height
		self.scene = self.createScene()
		self.setScene(self.scene)
		self.numobjects = self.scene.getObjects()
			
	def JGapplication(self):
		interval = Interval(0, self.width, self.height, 0, self.height, 1)
		self.render(interval)	  
	
	def JGvalidate(self):		
		refval =  {}
		refval[0] = 12
		refval[1] = 40			
		refval[2] = 104			
		refval[3] = 170	 
		refval[4] = 278			
		refval[5] = 390
		refval[6] = 552			
		refval[7] = 692			
		refval[8] = 894	 
		refval[9] = 1084			
		refval[10] = 9834			
		dev = self.checksum - refval[self.size]		
		if dev != 0:			
			print ("Validation failed")
			print ("Pixel checksum = " + str(self.checksum))
			print ("Reference value = " + str(refval[self.size]))
			
	def test(self):
		self.JGsetsize(10)
		self.JGinitialise()
		self.JGapplication()
		self.JGvalidate()	
	
	
class Chronometer:
	ticks1 = 0
	ticks2 = 0
	stopped = TRUE
	
	def start(self):
		self.ticks1 = time.time()
		self.stopped = FALSE

	def stop(self):
		self.ticks2 = time.time()
		self.stopped = TRUE

	def getSeconds(self):
		if self.stopped:
			return self.ticks2 - self.ticks1
		return time.time() - self.ticks1		
	def getMiliSeconds(self):
		return int(self.getSeconds() * 1000)
	def getMicroSeconds(self):
		return int(self.getSeconds() * 1000000)
	
class BenchMark:		
	def __init__(self,iterations):
		self.microSeconds = 0
		self.iterations = iterations

	def runOneIteration(self): 
		chronometer = Chronometer()							
		chronometer.start()				
		JGFRayTracerBench().test()
		chronometer.stop()		
		self.microSeconds = self.microSeconds + chronometer.getMicroSeconds()		
		

	def run(self): 
		for i in range(self.iterations):
			self.runOneIteration()
		return self.microSeconds
		
if __name__ == '__main__':
	import sys
	if len(sys.argv) < 2: 
		print ("You must pass the number of thousands iterations.")
		sys.exit(-1)  
	iterations = int(sys.argv[1])
	bench = BenchMark(iterations)	
	print(str(int(bench.run())).replace(".",","))