import sys, time, random, math
from time import clock
TRUE = 1
FALSE = 0

class JGSparseMatmultBench:
	def __init__(self):
		JGSparseMatmultBench.SPARSE_NUM_ITER = 10
		JGSparseMatmultBench.ytotal = 0.0
				
	def JGFkernel(self, val, x, y, row, col):
		for reps in range(JGSparseMatmultBench.SPARSE_NUM_ITER):		
			for i in range(len(val)):
				y[row[i]] += x[col[i]] * val[i]				
		for i in range(len(val)):
			JGSparseMatmultBench.ytotal += y[row[i]]   
	
	def JGFvalidate(self,size):		
		refval = {}
		refval[0] = 75.02484945753453
		refval[1] = 150.0130719633895
		refval[2] = 749.5245870753752
		dev = JGSparseMatmultBench.ytotal - refval[size]		
		dev = (dev if dev >= 0 else -1 * dev)
		if dev > 0.000000000001:
			#print ("Validation failed")
			#print ("ytotal = " + str(JGSparseMatmultBench.ytotal) + "  " + str(dev) + "  " + str(self.size))
			pass
			
	def test(self):
		datasizes_M = {}
		datasizes_M[0] = 50000
		datasizes_M[1] = 100000
		datasizes_M[2] = 500000

		datasizes_N = {}
		datasizes_N[0] = 50000
		datasizes_N[1] = 100000
		datasizes_N[2] = 500000

		datasizes_nz = {}
		datasizes_nz[0] = 250000
		datasizes_nz[1] = 500000
		datasizes_nz[2] = 2500000	
		
		size = 0
		R = random
		R.seed(1010)
		
		x = self.RandomVector(datasizes_N[size], R)
		y = [0.0]*datasizes_M[size]
		
		val = {}
		col = {}
		row = {}

		for i in range(datasizes_nz[size]):
			temp1 = R.randint(0,2147483647) % datasizes_M[size]
			temp2 = R.randint(0,2147483647) % datasizes_N[size]			
			row[i] = (temp1 if temp1 >= 0 else -1 * temp1)			
			col[i] = (temp2 if temp2 >= 0 else -1 * temp2)
			val[i] = R.random()
			
		self.JGFkernel(val,x,y,row,col)
		self.JGFvalidate(size)
	
	def RandomVector(self,N,R):		
		a = {}
		for i in range(N):		
			a[i] = R.random() * 0.000001
		return a
	
class Chronometer:
	ticks1 = 0
	ticks2 = 0
	stopped = TRUE
	
	def start(self):
		self.ticks1 = time.time()
		self.stopped = FALSE

	def stop(self):
		self.ticks2 = time.time()
		self.stopped = TRUE

	def getSeconds(self):
		if self.stopped:
			return self.ticks2 - self.ticks1
		return time.time() - self.ticks1		
	def getMiliSeconds(self):
		return int(self.getSeconds() * 1000)
	def getMicroSeconds(self):
		return int(self.getSeconds() * 1000000)
	
class BenchMark:		
	def __init__(self,iterations):
		self.microSeconds = 0
		self.iterations = iterations

	def runOneIteration(self): 
		chronometer = Chronometer()							
		chronometer.start()				
		JGSparseMatmultBench().test()
		chronometer.stop()		
		self.microSeconds = self.microSeconds + chronometer.getMicroSeconds()		
		
	def run(self): 
		for i in range(self.iterations):
			self.runOneIteration()
		return self.microSeconds
		
if __name__ == '__main__':
	import sys
	if len(sys.argv) < 2: 
		print ("You must pass the number of thousands iterations.")
		sys.exit(-1)  
	iterations = int(sys.argv[1])
	bench = BenchMark(iterations)	
	print(str(int(bench.run())).replace(".",","))