function JGFSparseMatmultBench() { 
	this.SPARSE_NUM_ITER = 10;
	this.ytotal = 0.0;
}

JGFSparseMatmultBench.prototype.RandomVector = function(N,R) {     
	var a = new Array(N);            
	for (var i = 0; i < N; i = i + 1)
		a[i] = R.random() * 1e-6;        
	return a;
}

function getRandomInt(R) {
	var min = 0;
	var max = 2147483647;
	return R.floor(R.random() * (max - min + 1)) + min;
}

JGFSparseMatmultBench.prototype.JGFkernel = function(val,x,y,row,col) {		
	for (var reps = 0; reps < this.SPARSE_NUM_ITER; reps = reps + 1)
	{				
		for (var i = 0; i < val.length ; i = i + 1)
		{								
			y[row[i]] += x[col[i]] * val[i];
		}
	}							
	for (var i = 0; i < val.length ; i = i + 1)
	{
		this.ytotal += y[row[i]];                
	}	
}

JGFSparseMatmultBench.prototype.JGFvalidate = function(size) {
	var refval = new Array(3);
	refval[0] = 75.02484945753453;
	refval[1] = 150.0130719633895;
	refval[2] = 749.5245870753752;
	var dev = this.ytotal - refval[size];		
	dev = dev >= 0 ? dev : -1 * dev;	
	if (dev > 1.0e-12)
	{
		//print("Validation failed");
		//print("ytotal = " + this.ytotal + "  " + dev + "  " + this.size);
	}	
}

JGFSparseMatmultBench.prototype.test = function() {
	var datasizes_M = new Array(3);            
	datasizes_M[0] = 50000;
	datasizes_M[1] = 100000;
	datasizes_M[2] = 500000;  	
	
	var datasizes_N = new Array(3);            
	datasizes_N[0] = 50000;
	datasizes_N[1] = 100000;
	datasizes_N[2] = 500000; 
	
	var datasizes_nz = new Array(3);            
	datasizes_nz[0] = 250000;
	datasizes_nz[1] = 500000;
	datasizes_nz[2] = 2500000; 
	
	var size = 0;
	
	var R = Math;
	var x = this.RandomVector(datasizes_N[size], R);
	var y = new Array(datasizes_M[size]);
    var val = new Array(datasizes_nz[size]);
    var col = new Array(datasizes_nz[size]);
    var row = new Array(datasizes_nz[size]);
	for (var i = 0; i < datasizes_nz[size]; i++){
		var temp1 = getRandomInt(R) % datasizes_M[size];
		var temp2 = getRandomInt(R) % datasizes_N[size]; 
		y[i] = 0;				
		row[i] = temp1 >= 0 ? temp1 : -1 * temp1;
		col[i] = temp2 >= 0 ? temp2 : -1 * temp2;
		val[i] = R.random();                   
	}
	
	this.JGFkernel(val,x,y,row,col);
	this.JGFvalidate(size);   	
}

function BenchMark(iterations) {
    this.microSeconds = 0;
	this.iterations = iterations;
}

BenchMark.prototype.runOneIteration = function() {		
	var before = new Date();
	new JGFSparseMatmultBench().test();
	var after = new Date();	
	this.microSeconds =  this.microSeconds + ((after-before) * 1000);
}

BenchMark.prototype.run = function() {	
	for(var i = 0 ; i < this.iterations ; i++)
		this.runOneIteration();	
	return this.microSeconds;
}	

function range(n) {
	var result = new Array();
	for(var i=0; i<n; i++)
		result[i] = i;
	return result;
}

if (scriptArgs.length == 1) {		
	var bench = new BenchMark(scriptArgs[0]);
	print(bench.run());
}
else {
	print("I need the name of the test.");
	quit();
}