import System

class Chronometer:
	private ticks1 as DateTime
	private ticks2 as DateTime
	
	private stopped as bool

	public def start():
		self.ticks1 = DateTime.Now
		self.stopped = false

	public def stop():
		self.ticks2 = DateTime.Now
		self.stopped = true

	private def ticksToMicroSeconds(t1 as DateTime, t2 as DateTime) as int:
		difference as TimeSpan
		difference = t2.Subtract(t1)
		return (difference.Milliseconds + difference.Seconds * 1000 + difference.Minutes * 60000) * 1000

	public def getMicroSeconds() as int:
		if self.stopped:
			return self.ticksToMicroSeconds(ticks1, ticks2)
		return self.ticksToMicroSeconds(ticks1, DateTime.Now)
		
class BenchMark:
	protected iterations as int
	protected microSeconds as int
	
	public def run() as int:
		for i in range(iterations):
			self.runOneIteration()
		return self.microSeconds
		
	public virtual def runOneIteration() as object:
		return null

class ArithmethicBenchmark(BenchMark):
	public def constructor(iterations as int):
		self.iterations = iterations
		
	public def runOneIteration() as object:		
		chronometer as Chronometer
		chronometer = Chronometer()	
		test = JGFFTBench()		
		chronometer.start()
		test.test()
		chronometer.stop()		
		self.microSeconds = self.microSeconds + chronometer.getMicroSeconds()
		return null	

class Test:
	pass

class JGFFTBench(Test):
	public datasizes as (int,1)
	public size as int
	public R as Random
	
	public def constructor():
		self.datasizes = array(int,3)
		#datasizes[0] = 2097152;
		self.datasizes[0] = 65536		
		self.datasizes[1] = 8388608
		self.datasizes[2] = 16777216	
	
	public def JGsetsize(size as int):
		self.size = size  
		
	public def JGinitialise():
		self.R = Random(1010)
			
	public def JGFkernel():
		x as (double,1)
		x = JGFFTBench.RandomVector(2 * (self.datasizes[self.size]), self.R)
		FFT.transform(x)
		FFT.inverse(x) 
	
	public def JGFvalidate():				
		refval = array(double, 3)
		refval[0] = 1.726962988395339
		refval[1] = 6.907851953579193
		refval[2] = 13.815703907167297
		refvali = array(double, 3)	
		refvali[0] = 2.0974756152524314
		refvali[1] = 8.389142211032294
		refvali[2] = 16.778094422092604
		dev as double
		dev = FFT.JDKtotal - refval[self.size]
		dev = (dev if dev >= 0 else -1 * dev)
		if dev > 0.000000000001:
			#print "Validation failed"
			#print "JDKtotal = " + str(FFT.JDKtotal) + "  " + str(dev) + "  " + str(self.size)
			pass
		devi as double
		devi = FFT.JDKtotali - refvali[self.size]
		devi = (devi if devi >= 0 else -1 * devi)
		if dev > 0.000000000001:
			#print "Validation failed"
			#print "JDKtotali = " + str(FFT.JDKtotali) + "  " + str(devi) + "  " + str(self.size)
			pass
	
	public static def RandomVector(N as int,R as Random) as (double,1):
		a as (double,1)
		a = array(double,N)
		for i in range(N):		
			a[i] = R.NextDouble() * 0.000001
		return a
				
	public def test():
		self.JGsetsize(0)
		self.JGinitialise()
		self.JGFkernel()
		self.JGFvalidate()

class FFT:
	public static JDKtotal = 0.0
	public static JDKtotali = 0.0
	
	public static def log2(n as int) as int:
		log as int
		log = 0
		k as int
		k = 1
		while k < n:
			k = k * 2
			log = log + 1			
		if n != (1 << log):
			print "FFT: Data Length is not a power of 2!: " + n.ToString()
		return log
		
	public static def makeRandom(n as int) as (double,1):
		rand as Random
		rand = Random()
		nd as int
		nd = 2 * n
		data as (double,1)		
		data = array(double, nd)
		for i in range(nd):
			data[i] = rand.NextDouble()
		return data
		
	public static def bitreverse(data as (double,1)):
		n as int
		n = len(data) / 2		
		j as int
		j = 0
		i as int
		i = 0		
		while i < (n - 1):
			ii as int
			ii = 2 * i
			jj as int
			jj = 2 * j
			k as int
			k = n / 2
			if i < j:
				tmp_real as double
				tmp_real = data[ii]
				tmp_imag as double
				tmp_imag = data[ii + 1]
				data[ii] = data[jj]
				data[ii + 1] = data[jj + 1]
				data[jj] = tmp_real
				data[jj + 1] = tmp_imag
			while k <= j:				
				j = j - k
				k = k / 2				
			j = j + k
			i = i + 1
	
	public static def transform_internal(data as (double,1), direction as int):	  
		n as int
		n = len(data) / 2
		if n == 1: 
			return		
		logn as int
		logn = FFT.log2(n)			
		FFT.bitreverse(data)
		dual as int
		dual = 1
		bit as int
		bit = 0
		while bit < logn:
			w_real as double
			w_real = 1.0
			w_imag as double
			w_imag = 0.0
			theta as double
			theta = 2.0 * direction * 3.14159265 / (2.0 * dual)
			s as double
			s = Math.Sin(theta)
			t as double
			t = Math.Sin(theta / 2.0)
			s2 as double
			s2 = 2.0 * t * t
			iterations as int
			iterations = 0
			while iterations < n:
				i as int
				i = 2 * iterations
				j as int
				j = 2 * (iterations + dual)
				wd_real as double
				wd_real = data[j]
				wd_imag as double
				wd_imag = data[j + 1]
				data[j] = data[i] - wd_real
				data[j + 1] = data[i + 1] - wd_imag
				data[i] = data[i] + wd_real
				data[i + 1] = data[i + 1] + wd_imag
				iterations = iterations + (2*dual)
			iterations = 1
			while iterations < dual:	
				tmp_real as double
				tmp_real = w_real - s * w_imag - s2 * w_real
				tmp_imag as double
				tmp_imag = w_imag + s * w_real - s2 * w_imag
				w_real = tmp_real
				w_imag = tmp_imag

				b as int
				b = 0
				while b < n:				
					i = 2 * (b + iterations)
					j = 2 * (b + iterations + dual)

					z1_real as double
					z1_real = data[j]
					z1_imag as double
					z1_imag = data[j + 1]

					wd_real = w_real * z1_real - w_imag * z1_imag
					wd_imag = w_real * z1_imag + w_imag * z1_real

					data[j] = data[i] - wd_real
					data[j + 1] = data[i + 1] - wd_imag
					data[i] = data[i] + wd_real
					data[i + 1] = data[i + 1] + wd_imag
					b = b + (2 * dual)
				iterations = iterations + 1							
			dual = dual * 2
			bit = bit + 1
	
	public static def transform(data as (double,1)):
		FFT.transform_internal(data, -1)
		JDKrange as int
		JDKrange = len(data)
		for i in range(JDKrange):			
			FFT.JDKtotal = FFT.JDKtotal + data[i]	
	
	public static def inverse(data as (double,1)):
		FFT.transform_internal(data, 1)
		nd as int
		nd = len(data)
		n as int
		n = nd / 2
		norm as double
		norm = 1 / n
		for i in range(nd):
			data[i] = data[i] * norm
		for i in range(nd):			
			FFT.JDKtotali = FFT.JDKtotali + data[i]
	
args = argv
def main():
	if args.Length < 1: 
		print ("You must pass the number of thousands iterations.")
		System.Environment.Exit(-1)
	
	iterations as int
	iterations = Convert.ToInt32(args[0])
	arith as ArithmethicBenchmark
	arith = ArithmethicBenchmark(iterations)
	Console.WriteLine(arith.run())	

main()