import System

class Chronometer:
	private ticks1 as DateTime
	private ticks2 as DateTime
	
	private stopped as bool

	public def start():
		self.ticks1 = DateTime.Now
		self.stopped = false

	public def stop():
		self.ticks2 = DateTime.Now
		self.stopped = true

	private def ticksToMicroSeconds(t1 as DateTime, t2 as DateTime) as int:
		difference as TimeSpan
		difference = t2.Subtract(t1)
		return (difference.Milliseconds + difference.Seconds * 1000 + difference.Minutes * 60000) * 1000

	public def getMicroSeconds() as int:
		if self.stopped:
			return self.ticksToMicroSeconds(ticks1, ticks2)
		return self.ticksToMicroSeconds(ticks1, DateTime.Now)
		
class BenchMark:
	protected iterations as int
	protected microSeconds as int
	
	public def run() as int:
		for i in range(iterations):
			self.runOneIteration()
		return self.microSeconds
		
	public virtual def runOneIteration() as object:
		return null

class ArithmethicBenchmark(BenchMark):
	public def constructor(iterations as int):
		self.iterations = iterations
		
	public def runOneIteration() as object:		
		chronometer as Chronometer
		chronometer = Chronometer()	
		test = JGFHeapSortBench()		
		chronometer.start()
		test.test()
		chronometer.stop()		
		self.microSeconds = self.microSeconds + chronometer.getMicroSeconds()
		return null	

class NumericSortTest():	
	public array_rows as int
	public TestArray as (int,1)
	
	public def buildTestData():	
		rndnum as Random
		i as int
		self.TestArray = array(int,self.array_rows)
		rndnum = Random(1729)		
		i =	0
		while i < self.array_rows:
			self.TestArray[i] = rndnum.Next()
			i = i + 1
	
	public def Do():
		self.NumHeapSort()
	
	def NumSift(min as int ,max as int):
		inc as bool
		k as int
		temp as int
		while (min + min) <= max:
			inc = false
			if (min + min) < max and self.TestArray[min + min] < self.TestArray[min + min + 1]:
				inc = true
			k =  (min + min + 1 if inc else min + min)
			if self.TestArray[min] < self.TestArray[k]:
				temp = self.TestArray[k]
				self.TestArray[k] = self.TestArray[min]
				self.TestArray[min] = temp
				min = k				
			else:
				min = max + 1

	def NumHeapSort():	
		top as int
		i as int
		temp as int
		top = self.array_rows - 1
		i = top / 2
		while i > 0:
			self.NumSift(i, top)
			i = i - 1
		i = top
		while i > 0:
			self.NumSift(0, i)
			temp = self.TestArray[0]
			self.TestArray[0] = self.TestArray[i]
			self.TestArray[i] = temp
			i = i - 1
	
class JGFHeapSortBench(NumericSortTest):
	size as int
	datasizes as (int,1)
	def constructor():		
		self.datasizes = array(int,3)
		self.datasizes[0] = 1000000
		self.datasizes[1] = 5000000
		self.datasizes[2] = 25000000

	def JGsetsize(size as int):
		self.size = size  

	def JGinitialise():
		self.array_rows = self.datasizes[self.size]
		self.buildTestData()
		
	def JGFkernel():
		self.Do()
	
	def JGFvalidate():
		error as bool
		i as int
		error = false
		i = 1
		while i < self.array_rows:
			error = self.TestArray[i] < self.TestArray[i - 1]
			if error:
				#print ("Validation failed")
				#print ("Item " + i + " = " + self.TestArray[i])
				#print ("Item " + (i - 1) + " = " + self.TestArray[i - 1])
				pass
			i = i + 1
	
	def test():
		self.JGsetsize(0)
		self.JGinitialise()
		self.JGFkernel()
		self.JGFvalidate()

args = argv
def main():
	if args.Length < 1: 
		print ("You must pass the number of thousands iterations.")
		System.Environment.Exit(-1)
	
	iterations as int
	iterations = Convert.ToInt32(args[0])
	arith as ArithmethicBenchmark
	arith = ArithmethicBenchmark(iterations)
	Console.WriteLine(arith.run())	

main()