import System

class Chronometer:
	private ticks1 as DateTime
	private ticks2 as DateTime
	
	private stopped as bool

	public public def start():
		self.ticks1 = DateTime.Now
		self.stopped = false

	public public def stop():
		self.ticks2 = DateTime.Now
		self.stopped = true

	private public def ticksToMicroSeconds(t1 as DateTime, t2 as DateTime) as int:
		difference as TimeSpan
		difference = t2.Subtract(t1)
		return (difference.Milliseconds + difference.Seconds * 1000 + difference.Minutes * 60000) * 1000

	public public def getMicroSeconds() as int:
		if self.stopped:
			return self.ticksToMicroSeconds(ticks1, ticks2)
		return self.ticksToMicroSeconds(ticks1, DateTime.Now)
		
class BenchMark:
	protected iterations as int
	protected microSeconds as int
	
	public public def run() as int:
		for i in range(iterations):
			self.runullIteration()
		return self.microSeconds
		
	public virtual public def runullIteration() as object:
		return null

class ArithmethicBenchmark(BenchMark):
	public public def constructor(iterations as int):
		self.iterations = iterations
		
	public public def runullIteration() as object:		
		chronometer as Chronometer
		chronometer = Chronometer()	
		test = JGFRayTracerBench()		
		chronometer.start()
		test.Test()		
		chronometer.stop()		
		self.microSeconds = self.microSeconds + chronometer.getMicroSeconds()
		return null	

class Vec:
	public x as double
	public y as double
	public z as double
	public def constructor():		
		self.x = 0.0
		self.y = 0.0
		self.z = 0.0
		
	public def constructor(a as double,b as double,c as double):		
		self.x = a
		self.y = b
		self.z = c
	
	public def add1(a as Vec):
		self.x = a.x + self.x
		self.y = a.y + self.y
		self.z = a.z + self.z
	
	public static def adds(s as double,a as Vec,b as Vec) as Vec:	
		return Vec(s * a.x + b.x, s * a.y + b.y, s * a.z + b.z)	

	public def adds(s as double,b as Vec):	
		self.x = self.x + (s * b.x)
		self.y = self.y + (s * b.y)
		self.z = self.z + (s * b.z)

	public static def sub1(a as Vec,b as Vec) as Vec:		
		return Vec(a.x - b.x, a.y - b.y, a.z - b.z)
	
	public def sub2(a as Vec,b as Vec):
		self.x = a.x - b.x
		self.y = a.y - b.y
		self.z = a.z - b.z	
	
	public static def mult1(a as Vec,b as Vec) as Vec:
		return Vec(a.x * b.x, a.y * b.y, a.z * b.z)

	public static def cross(a as Vec,b as Vec) as Vec:
		return Vec(a.y * b.z - a.z * b.y, a.z * b.x - a.x * b.z, a.x * b.y - a.y * b.x)
	
	public static def dot(a as Vec,b as Vec) as double:
		return a.x * b.x + a.y * b.y + a.z * b.z
	
	public static def comb(a as double,A as Vec,b as double,B as Vec) as Vec:
		return Vec(a * A.x + b * B.x, a * A.y + b * B.y, a * A.z + b * B.z)
	
	public def comb2(a as double,A as Vec,b as double,B as Vec):
		self.x = a * A.x + b * B.x
		self.y = a * A.y + b * B.y
		self.z = a * A.z + b * B.z	

	public def scale(t as double):
		self.x = t * self.x
		self.y = t * self.y
		self.z = t * self.z
	
	public def negate():
		self.x = -1.0 * self.x
		self.y = -1.0 * self.y
		self.z = -1.0 * self.z
	
	public def normalize() as double:
		length as double
		length = Math.Sqrt(self.x * self.x + self.y * self.y + self.z * self.z)
		while length > 0.0:
			self.x = self.x / length
			self.y = self.y / length
			self.z = self.z / length
			length = 0.0			
		return length
		
	public override def ToString() as string:
		return "<" + self.x.ToString() + "," + self.y.ToString() + "," + self.z.ToString() + ">"

class View:
	public fromVec as Vec
	public atVec as Vec
	public upVec as Vec
	public dist as double
	public angle as double
	public aspect as double
	
	public def constructor(fromVec as Vec,atVec as Vec,upVec as Vec,dist as double,angle as double,aspect as double):
		self.fromVec = fromVec
		self.atVec = atVec
		self.upVec = upVec
		self.dist = dist
		self.angle = angle
		self.aspect = aspect

class Ray:	
	public P as Vec
	public D as Vec
	
	public def constructor():
		self.P = Vec()			
		self.D = Vec()			
		
	public def constructor(pnt as Vec,dir as Vec):
		self.P = Vec()			
		self.P.add1(pnt)
		self.D = Vec()			
		self.D.add1(dir)
		self.D.normalize()

	public def point(t as double) as Vec:
		return Vec(self.P.x + self.D.x * t, self.P.y + self.D.y * t, self.P.z + self.D.z * t)
	
	public override def ToString() as string:	
		return "" + self.P.ToString() + " -> " + self.P.ToString() + ""

class Interval:	
	public number as int
	public width as int
	public height as int
	public yfrom as int
	public yto as int
	public total as int
		
	public def constructor(number as int,width as int,height as int,yfrom as int,yto as int,total as int):
		self.number = number
		self.width = width
		self.height = height
		self.yfrom = yfrom
		self.yto = yto
		self.total = total
	
class Isect:
	public t as double
	public enter as int
	public prim as Primitive
	public surf as Surface	
	
	public def constructor():
		self.t = 0.0
		self.enter = 0
		self.prim = null
		self.surf = null
		
	public override def ToString() as string:
		return "" + self.t.ToString() + "," + self.enter.ToString() + "," + self.prim.ToString() + "," + self.surf.ToString() + ""
		
class Light:
	public pos as Vec
	public brightness as double
		
	public def constructor():
		self.pos = null
		self.brightness = 0.0
		
	public def constructor(x as double,y as double,z as double,brightness as double):
		self.pos = Vec(x,y,z)		
		self.brightness = brightness
		
class Surface:
	public color as Vec
	public kd as double 
	public ks as double 
	public shine as double 
	public kt as double 
	public ior as double
		
	public def constructor():
		self.color = Vec(1.0, 0.0, 0.0)
		self.kd = 1.0
		self.ks = 0.0
		self.shine = 0.0
		self.kt = 0.0
		self.ior = 1.0

	public override def ToString() as string:
		return "Surface  color=" + color.ToString() + ""

class Primitive:
	public surf as Surface
	
	public def constructor():
		self.surf = Surface()

	public def setColor(r as double,g as double,b as double):
		self.surf.color = Vec(r, g, b)
		
	public virtual def intersect(ry as Ray) as Isect:
		pass
	public virtual def normal(pnt as Vec) as Vec:
		pass
		
class Sphere(Primitive):
	public c as Vec
	public r as double
	public r2 as double
	public v as Vec
	public b as Vec
		
	public def constructor(center as Vec,radius as double):		
		self.c = center
		self.r = radius
		self.r2 = radius * radius
		self.v = Vec()
		self.b = Vec()
	
	public override def intersect(ry as Ray) as Isect:
		p as Vec	
		p = ry.P
		d as Vec		
		d = ry.D	   			
		self.v.sub2(self.c, p)		
		b as double
		b = Vec.dot(self.v, d)
		dotVV as double
		dotVV = Vec.dot(self.v, self.v)
		disc as double
		disc = b * b - dotVV + self.r2				
		if disc < 0.0:
			return null														
		disc = Math.Sqrt(disc)
		t as double
		t = (b + disc if (b - disc) < 0.000001 else b - disc)
		if t < 0.000001:
			return null
		ip as Isect
		ip = Isect()
		ip.t = t						
		ip.enter = (1.0 if dotVV > (self.r2 + 0.000001) else 0.0)
		ip.prim = self
		ip.surf = self.surf
		return ip			
	
	public override def normal(p as Vec) as Vec:
		r as Vec
		r = Vec.sub1(p, self.c)
		r.normalize()
		return r	

	public def getCenter() as Vec:
		return self.c
		
	public def setCenter(c as Vec):
		self.c = c

	public override def ToString() as string:
		return "Sphere " + self.c.ToString() + "," + self.r.ToString() + ""
		
class Scene:
	public lights as List
	public objects as List
	public view as View
	public def constructor():
		self.lights = []
		self.objects = []
		self.view = null

	public def addLight(l as Light):
		self.lights.Add(l)
	
	public def addObject(myobject as Primitive):
		self.objects.Add(myobject)
	
	public def setView(view as View):	
		self.view = view
	
	public def getView() as View:
		return self.view
	
	public def getLight(number as int) as Light:
		return self.lights[number]
	
	public def getObject(number as int) as Primitive:
		return self.objects[number]	

	public def getLights() as int:
		return len(self.lights)

	public def getObjects() as int:
		return len(self.objects)
	
	public def setObject(myobject as Primitive,pos as int):
		self.objects[pos] = myobject
		
class RayTracer:	
	public static alpha as int
	public static voidVec as Vec
	public datasizes as List
	public tRay as Ray
	public L as Vec
	public inter as Isect
	public checksum as int
	public size as int
	public numobjects as int
	public height as int
	public width as int
	public view as View
	public prim as (Primitive,1)
	public lights as (Light,1)
	public scene as Scene
	
	public def constructor():
		self.datasizes = []
		self.datasizes.Add(5)
		self.datasizes.Add(10)
		self.datasizes.Add(15)
		self.datasizes.Add(20)
		self.datasizes.Add(25)
		self.datasizes.Add(30)
		self.datasizes.Add(35)
		self.datasizes.Add(40)
		self.datasizes.Add(45)
		self.datasizes.Add(50)
		RayTracer.alpha = (255 << 24) cast int		
		self.tRay = Ray()
		RayTracer.voidVec = Vec()
		self.L = Vec()
		self.inter = Isect()
		self.checksum = 0
		
		self.size = 0
		self.numobjects = 0
		self.height = 0
		self.width = 0
		self.view = null
		self.prim = null
		self.lights = null
		self.scene = null

	public def createScene() as Scene:
		x as double
		x = 0.0
		y as double
		y = 0.0
		scene as Scene
		scene = Scene()
		nx as double
		nx = 4.0
		ny as double
		ny = 4.0
		nz as double
		nz = 4.0
		i as double
		i = 0.0			
		while i < nx:		
			j as double
			j = 0.0
			while j < ny:			
				k as double
				k = 0.0
				while k < nz:				
					xx as double
					xx = 20.0 / (nx - 1.0) * i - 10.0
					yy as double
					yy = 20.0 / (ny - 1.0) * j - 10.0
					zz as double
					zz = 20.0 / (nz - 1.0) * k - 10.0												
					p as Sphere
					p = Sphere(Vec(xx, yy, zz), 3.0)					
					p.setColor(0.0, 0.0, (i + j) / (nx + ny - 2.0))					
					p.surf.shine = 15.0
					p.surf.ks = 1.5 - 1.0
					p.surf.kt = 1.5 - 1.0						
					scene.addObject(p)						
					k = k + 1.0				
				j = j + 1.0			
			i = i + 1.0		

		scene.addLight(Light(100.0, 100.0, -50.0, 1.0))
		scene.addLight(Light(-100.0, 100.0, -50.0, 1.0))
		scene.addLight(Light(100.0, -100.0, -50.0, 1.0))
		scene.addLight(Light(-100.0, -100.0, -50.0, 1.0))
		scene.addLight(Light(200.0, 200.0, 0.0, 1.0))
		v as View
		v = View(Vec(x, 20.0, -30.0), Vec(x, y, 0.0), Vec(0.0, 1.0, 0.0), 1.0, 35.0 * 3.14159265 / 180.0, 1.0)
		scene.setView(v)
		return scene

	public def setScene(scene as Scene):		
		nLights as int
		nLights = scene.getLights()
		nObjects as int
		nObjects = scene.getObjects()		
		self.lights = array(Light,nLights)		
		self.prim = array(Primitive,nObjects)
		l as int
		l = 0
		while l < nLights:
			self.lights[l] = scene.getLight(l)
			l = l + 1		
		o as int
		o = 0
		while o < nObjects:		
			self.prim[o] = scene.getObject(o)
			o = o + 1		
		self.view = scene.getView()

	public def intersect(r as Ray,tmax as double) as bool:		
		nhits as int
		nhits = 0
		temp as Isect
		temp =  Isect()			
		temp.t = 1e9	
		self.inter = temp					
		i as int
		i = 0											
		while i < len(self.prim):
			tp as Isect
			tp = self.prim[i].intersect(r)
			sw as bool
			sw = (tp != null)					
			if(sw == true):
				isect as Isect
				isect = Isect()							
				isect.t = tp.t															
				isect.prim = tp.prim
				isect.surf = tp.surf
				isect.enter = tp.enter														
				self.inter = isect							
				nhits = nhits + 1											
			i = i + 1
		return (true if nhits > 0 else false)
		
	public def trace(level as int,weight as double,r as Ray) as Vec:
		if level > 6:
			return Vec()
		hit as bool
		hit = self.intersect(r, 1e6)
		if hit == true:
			P as Vec
			P = r.point(self.inter.t)
			N as Vec
			N = self.inter.prim.normal(P)
			if Vec.dot(r.D, N) >= 0.0:
				N.negate()
			#return shade(level, weight, P, N, r.D, inter)	
		return voidVec
		
	public def SpecularDirection(I as Vec,N as Vec) as Vec:
		dotIN as double
		dotIN = Vec.dot(I, N)
		abs as double
		abs =  (dotIN if dotIN > 0 else -1 * dotIN)
		r as Vec
		r = Vec.comb(1.0 / abs, I, 2.0, N)
		r.normalize()
		return r
		
	public def TransDir(m1 as Surface,m2 as Surface,I as Vec,N as Vec) as Vec:
		n1 as double		
		n1 = (1.0 if m1 == null else m1.ior)
		n2 as double
		n2 = (1.0 if m2 == null else m2.ior)
		eta as double
		eta = n1 / n2
		c1 as double
		c1 = -1 * Vec.dot(I, N)
		cs2 as double
		cs2 = 1.0 - eta * eta * (1.0 - c1 * c1)
		if cs2 < 0.0:
			return null
		r as Vec
		r = Vec.comb(eta, I, eta * c1 - Math.Sqrt(cs2), N)
		r.normalize()
		return r

	public def Shadow(r as Ray,tmax as double) as int:
		if intersect(r,tmax) == true:
			return 0
		return 1

	public def render(interval as Interval):
		localView as View
		localView = self.view			
		viewVec as Vec
		viewVec = Vec.sub1(localView.atVec, localView.fromVec)											
		viewVec.normalize()			
		tmpVec as Vec
		tmpVec = Vec(viewVec.x, viewVec.y, viewVec.z)			
		dotValue as double
		dotValue = Vec.dot(localView.upVec, viewVec)			
		tmpVec.scale(dotValue)			
		upVec as Vec
		upVec = Vec.sub1(localView.upVec, tmpVec)
		upVec.normalize()
		leftVec as Vec
		leftVec = Vec.cross(localView.upVec, viewVec)
		leftVec.normalize()		
		frustrumwidth as double
		frustrumwidth = localView.dist * Math.Tan(localView.angle)						
		upVec.scale(-frustrumwidth)			
		scaleValue as double
		scaleValue = localView.aspect * frustrumwidth
		leftVec.scale(scaleValue)						
		r as Ray
		r = Ray(localView.fromVec, RayTracer.voidVec)
		y as int
		y = interval.yfrom					
		nhits as int
		nhits = 0					
		while y < interval.yto:
			ylen as double
			ylen = (2.0 * y) / interval.width - 1.0
			x as int
			x = 0
			while x < interval.width:
				xlen as double
				xlen = (2.0 * x) / interval.width - 1.0					
				r.D = Vec.comb(xlen, leftVec, ylen, upVec)	 					
				r.D.add1(viewVec) 									
				r.D.normalize()																
				hit as bool
				hit = self.intersect(r, 1e6)										
				if hit == true:
					nhits = nhits + 1
				x = x + 1			
			y = y + 1		
		self.checksum = self.checksum + nhits		

class JGFRayTracerBench(RayTracer):
	public def JGFsetsize(size as int):
		self.size = size
		
	public def JGFinitialise():		
		self.height = self.datasizes[self.size]
		self.width = self.height
		self.scene = self.createScene()
		self.setScene(self.scene)
		self.numobjects = self.scene.getObjects()
		
	public def JGFapplication():
		interval = Interval(0, self.width, self.height, 0, self.height, 1)
		self.render(interval)		

	public def JGFvalidate():	
		refval as (int,1)
		refval = array(int, 10)
		refval[0] = 12
		refval[1] = 40			
		refval[2] = 104			
		refval[3] = 170	 
		refval[4] = 278			
		refval[5] = 390
		refval[6] = 552			
		refval[7] = 692			
		refval[8] = 894	 
		refval[9] = 1084
		dev as double		
		dev = self.checksum - refval[self.size]
		if dev != 0:			
			print "Validation failed"
			print "Pixel checksum = " + self.checksum.ToString()
			print "Reference value = " + (refval[self.size])
			
	public def Test():
		self.JGFsetsize(4)
		self.JGFinitialise()
		self.JGFapplication()
		self.JGFvalidate()			
		
args = argv
def main():
	if args.Length < 1: 
		print ("You must pass the number of thousands iterations.")
		System.Environment.Exit(-1)
	
	iterations as int
	iterations = Convert.ToInt32(args[0])
	arith as ArithmethicBenchmark
	arith = ArithmethicBenchmark(iterations)
	Console.WriteLine(arith.run())	

main()