import System

class Chronometer:
	private ticks1 as DateTime
	private ticks2 as DateTime
	
	private stopped as bool

	public def start():
		self.ticks1 = DateTime.Now
		self.stopped = false

	public def stop():
		self.ticks2 = DateTime.Now
		self.stopped = true

	private def ticksToMicroSeconds(t1 as DateTime, t2 as DateTime) as int:
		difference as TimeSpan
		difference = t2.Subtract(t1)
		return (difference.Milliseconds + difference.Seconds * 1000 + difference.Minutes * 60000) * 1000

	public def getMicroSeconds() as int:
		if self.stopped:
			return self.ticksToMicroSeconds(ticks1, ticks2)
		return self.ticksToMicroSeconds(ticks1, DateTime.Now)
		
class BenchMark:
	protected iterations as int
	protected microSeconds as int
	
	public def run() as int:
		for i in range(iterations):
			self.runOneIteration()
		return self.microSeconds
		
	public virtual def runOneIteration() as object:
		return null

class ArithmethicBenchmark(BenchMark):
	public def constructor(iterations as int):
		self.iterations = iterations
		
	public def runOneIteration() as object:		
		chronometer as Chronometer
		chronometer = Chronometer()	
		test = JGSparseMatmultBench()		
		chronometer.start()
		test.test()
		chronometer.stop()		
		self.microSeconds = self.microSeconds + chronometer.getMicroSeconds()
		return null	

class Test:
	pass

class JGSparseMatmultBench(Test):
	public static datasizes_M as (int,1)
	public static datasizes_N as (int,1)
	public static datasizes_nz as (int,1)
	public static SPARSE_NUM_ITER as int
	public static ytotal as double
	public size as int
	public R as Random
	public x as (double,1)
	public y as (double,1)
	public val as (double,1)
	public col as (int,1)
	public row as (int,1)
	
	public def constructor():
		JGSparseMatmultBench.datasizes_M = array(int,3)
		JGSparseMatmultBench.datasizes_M[0] = 50000
		JGSparseMatmultBench.datasizes_M[1] = 100000
		JGSparseMatmultBench.datasizes_M[2] = 500000

		JGSparseMatmultBench.datasizes_N = array(int,3)
		JGSparseMatmultBench.datasizes_N[0] = 50000
		JGSparseMatmultBench.datasizes_N[1] = 100000
		JGSparseMatmultBench.datasizes_N[2] = 500000

		JGSparseMatmultBench.datasizes_nz = array(int,3)
		JGSparseMatmultBench.datasizes_nz[0] = 250000
		JGSparseMatmultBench.datasizes_nz[1] = 500000
		JGSparseMatmultBench.datasizes_nz[2] = 2500000
		
		JGSparseMatmultBench.SPARSE_NUM_ITER = 10
		JGSparseMatmultBench.ytotal = 0.0
	
	public def JGsetsize(size as int):
		self.size = size  
		
	public def JGinitialise():
		self.R = Random(1010)
		
		self.x = self.RandomVector(JGSparseMatmultBench.datasizes_N[self.size], self.R)
		self.y = array(double, JGSparseMatmultBench.datasizes_M[self.size])
		
		self.val = array(double,JGSparseMatmultBench.datasizes_nz[self.size])
		self.col = array(int,JGSparseMatmultBench.datasizes_nz[self.size])
		self.row = array(int,JGSparseMatmultBench.datasizes_nz[self.size])

		for i in range(JGSparseMatmultBench.datasizes_nz[self.size]):
			temp1 as int
			temp2 as int
			temp1 = self.R.Next() % JGSparseMatmultBench.datasizes_M[self.size]
			temp2 = self.R.Next() % JGSparseMatmultBench.datasizes_N[self.size]
			self.row[i] = (temp1 if temp1 >= 0 else -1 * temp1)
			self.col[i] = (temp2 if temp2 >= 0 else -1 * temp2)
			self.val[i] = self.R.NextDouble()
			
	public def JGFkernel():
		for reps in range(JGSparseMatmultBench.SPARSE_NUM_ITER):
			for i in range(len(self.val)):
				self.y[self.row[i]] += self.x[self.col[i]] * self.val[i]
		for i in range(len(self.val)):
			JGSparseMatmultBench.ytotal += self.y[self.row[i]]   
	
	public def JGFvalidate():        
		refval = array(double, 3)
		refval[0] = 75.02484945753453
		refval[1] = 150.0130719633895
		refval[2] = 749.5245870753752
		dev as double
		dev = JGSparseMatmultBench.ytotal - refval[self.size]
		dev = (dev if dev >= 0 else -1 * dev)
		if dev > 0.000000000001:
			#print "Validation failed"
			#print "ytotal = " + JGSparseMatmultBench.ytotal.ToString() + "  " + dev.ToString() + "  " + self.size.ToString()
			pass
	
	public def RandomVector(N as int,R as Random) as (double,1):
		a as (double,1)
		a = array(double,N cast int)
		for i in range(N):		
			a[i] = R.NextDouble() * 0.000001
		return a
		
	public def test():
		self.JGsetsize(0)
		self.JGinitialise()
		self.JGFkernel()
		self.JGFvalidate()

args = argv
def main():
	if args.Length < 1: 
		print ("You must pass the number of thousands iterations.")
		System.Environment.Exit(-1)
	
	iterations as int
	iterations = Convert.ToInt32(args[0])
	arith as ArithmethicBenchmark
	arith = ArithmethicBenchmark(iterations)
	Console.WriteLine(arith.run())	

main()