public class Chronometer 
{
  private Int ticks1;
  private Int ticks2;
  private Bool stopped;
  
  public Void start() 
  {
    this.ticks1 = Duration.nowTicks();
    this.stopped = false;
  }

  public Void stop() {
    this.ticks2 = Duration.nowTicks();
    this.stopped = true;
  }
  
  public Int getMicroSeconds() {
    return (this.ticks2 - this.ticks1)/1000;
  } 
}

class Random
{
  public Int next()
  {
    return Int.random;
  }
  
  public Float nextFloat()
  {
    return Float.random;
  }
}

public abstract class Benchmark 
{
  private Int iterations;
  protected Int microSeconds;
  
  new make(Int iterations) 
  {
    this.iterations = iterations;
  } 
  
  public Int run() {
    for(i := 0 ; i < this.iterations ; i++)
      this.runOneIteration();
    return this.microSeconds;
  }
  
  public virtual Obj? runOneIteration() {
    return null;
  }   
}


class ArithmethicBenchmark : Benchmark 
{
  new make(Int iterations): super(iterations) {}

  public override Obj? runOneIteration() 
  {
    chronometer := Chronometer(); 
    
    test := JGFFFTBench();
    
    chronometer.start();
    test.test();
    chronometer.stop();   
    this.microSeconds = this.microSeconds + chronometer.getMicroSeconds();
    return null;
  }
}

public abstract class TestPyBench {
        public abstract Void test();
    }


 public class JGFFFTBench : TestPyBench
  {
    Int size;
    Int?[]  datasizes;
    Random? R;
    FFT? fft;
    
    new make()
    {
      datasizes = Int?[,];
      datasizes.size = 3;
      datasizes[0] = 65536;      
      datasizes[1] = 8388608;
      datasizes[2] = 16777216;
      fft = FFT();
      
    }
    
    public override Void test() 
    {   
      JGFsetsize(0);
      JGFinitialise();
      JGFkernel();
      JGFvalidate();     
    }
    
    public Void JGFsetsize(Int size)
    {
      this.size = size;
    }
    
    public Void JGFinitialise()
    {      
      R = Random();     
    }
    
    private static Float?[] RandomVector(Int N, Random? R)
    {      
      Float?[] a := Float?[,];
      a.size = N;
      for (Int i := 0; i < N; i = i + 1)
        a[i] = R.nextFloat() * 0.000001f;                
      return a;
    }
  
    public Void JGFkernel()
    {       
      Float[] x := JGFFFTBench.RandomVector(datasizes[size] * 2, R);
      fft.transform(x);      
      fft.inverse(x); 
    }
  
    public Void JGFvalidate()
    {
      Float?[] refval := Float?[,];
      refval.size = 3;
      refval[0] = 1.726962988395339f;
      refval[1] = 6.907851953579193f;
      refval[2] = 13.815703907167297f;
      Float?[] refvali := Float?[,];
      refvali.size = 3;
      refvali[0] = 2.0974756152524314f;
      refvali[1] = 8.389142211032294f;
      refvali[2] = 16.778094422092604f;
      
      Float dev := fft.JDKtotal - refval[size];
      dev = (dev >= 0.0f ? dev : dev * -1.0f);
      if (dev > 0.000000000001f)
      {
        //echo("Validation failed");
        //echo("FFT.JDKtotal = " + FFT.JDKtotal + "  " + dev + "  " + size);
      }
      
      Float devi := fft.JDKtotali - refvali[size];
      devi = (devi >= 0.0f ? devi : devi * -1.0f);
      if (devi > 0.000000000001f)
      {
        //echo("Validation failed");
        //echo("FFT.JDKtotali = " + FFT.JDKtotali + "  " + devi + "  " + size);
      }
    }    
  }
  
  public class FFT
  {
    public Float JDKtotal := 0.0f;
    public Float JDKtotali := 0.0f;
    
    public static Int log2(Int n)
    {
      Int log;
      log = 0;
      Int k;
      k = 1;        
      while(k < n)
      {                 
        k = k * 2;                
        log = log + 1;                
      }     
      if (n != (1.shiftl(log)))
        echo("FFT: Data Length is not a power of 2!: " + n);
      return log;       
    }
    
    public static Float?[] makeRandom(Int n)
    {
      Random? rand;
      rand = Random();
      Int nd;
      nd = n * 2;
      Float?[] data := Float?[,];
      data.size = nd;
      for (Int i := 0; i < nd; i = i + 1)
        data[i] = rand.nextFloat();                
      return data;
    }    
    
    public static Void bitreverse(Float?[] data)
    {     
      Int n;
      n = data.size / 2;
      Int j;
      j = 0;
      Int i;
      i = 0;          
      while(i < (n - 1))
      {     
        Int ii;
        ii = i * 2;
        Int jj;
        jj = j * 2;
        Int k;
        k = n / 2;
        if (i < j)
        {                     
            Float tmp_real;
            tmp_real = data[ii];                    
            Float tmp_imag;
            tmp_imag = data[ii + 1];
            data[ii] = data[jj];
            data[ii + 1] = data[jj + 1];
            data[jj] = tmp_real;
            data[jj + 1] = tmp_imag;        
        }             
        while (k <= j)
        {                   
            j = j - k;
            k = k / 2;        
        }       
        j = j + k;
        i = i + 1;                        
      }           
    }
    
    public static Void transform_internal(Float?[] data, Int direction)
    {     
      Int n;
      n = data.size / 2;
      if (n == 1) 
        return;     
      Int logn;
      logn = log2(n);     
      bitreverse(data);             
      Int dual;
      dual = 1; 
      Int bit;
      bit = 0;
      while(bit < logn)            
      {                            
          Float w_real;
          w_real = 1.0f;
          Float w_imag;
          w_imag = 0.0f;
          Float theta;
          theta = 2.0f * direction * 3.14159265f / (2.0f * dual);     
          Float s;
          s = theta.sin;
          Float t;
          t = (theta /2.0f).sin;      
          Float s2;
          s2 = t * t * 2.0f;
          Int iterations;
          iterations = 0;
          while (iterations < n)
          {                    
            Int i := 2 * iterations;
            Int j := 2 * (iterations + dual);
            Float wd_real := data[j];
            Float wd_imag := data[j + 1];                    
            data[j] = data[i] - wd_real;
            data[j + 1] = data[i + 1] - wd_imag;
            data[i] = data[i] + wd_real;
            data[i + 1] = data[i + 1] + wd_imag;                    
            iterations = iterations + (2*dual);
          }             
          iterations = 1;
        while (iterations < dual)
        {
          Float tmp_real := w_real - s * w_imag - s2 * w_real;
          Float tmp_imag := w_imag + s * w_real - s2 * w_imag;
          w_real = tmp_real;
          w_imag = tmp_imag;
        
          Int b;
          b = 0;
          while (b < n)
          {                        
            Int i := 2 * (b + iterations);
            Int j := 2 * (b + iterations + dual);

            Float z1_real := data[j];
            Float z1_imag := data[j + 1];

            Float wd_real := w_real * z1_real - w_imag * z1_imag;
            Float wd_imag := w_real * z1_imag + w_imag * z1_real;

            data[j] = data[i] - wd_real;
            data[j + 1] = data[i + 1] - wd_imag;
            data[i] = data[i] + wd_real;
            data[i + 1] = data[i + 1] + wd_imag;
            b = b + (2 * dual);
          }                    
          iterations = iterations + 1;
        }
          dual = dual * 2;
          bit = bit + 1;
      }
    }
    
    public Void transform(Float?[] data)
    {
        Int JDKrange;
        FFT.transform_internal(data, -1);
        JDKrange = data.size;
        for (Int i := 0; i < JDKrange; i = i + 1)          
          this.JDKtotal = data[i] + this.JDKtotal;            
    }
    
    public Void inverse(Float?[] data)
    {
        FFT.transform_internal(data, 1);
        Int nd;
        nd = data.size;
        Int n;
        n = nd / 2;
        Float norm;            
        norm = 1.0f / n.toFloat();   
        for (Int i := 0; i < nd; i = i + 1)  
            data[i] = data[i] * norm;   
        for (Int i := 0; i < nd; i = i + 1)  
            this.JDKtotali = data[i] + this.JDKtotali;
    }    
  }

public class Main
{
  public static Void main(Str[] args)
  {   
    if (args.size < 1)
    {   
      echo("You must pass the number of thousands iterations.");
    }
    else
    {   
      iterations := Int.fromStr(args[0]);   
      arith := ArithmethicBenchmark(iterations)
      echo(arith.run());
    }    
  } 
}