public class Chronometer 
{
  private Int ticks1;
  private Int ticks2;
  private Bool stopped;
  
  public Void start() 
  {
    this.ticks1 = Duration.nowTicks();
    this.stopped = false;
  }

  public Void stop() {
    this.ticks2 = Duration.nowTicks();
    this.stopped = true;
  }
  
  public Int getMicroSeconds() {
    return (this.ticks2 - this.ticks1)/1000;
  } 
}

public abstract class Benchmark 
{
  private Int iterations;
  protected Int microSeconds;
  
  new make(Int iterations) 
  {
    this.iterations = iterations;
  } 
  
  public Int run() {
    for(i := 0 ; i < this.iterations ; i++)
      this.runOneIteration();
    return this.microSeconds;
  }
  
  public virtual Obj? runOneIteration() {
    return null;
  }   
}

class Random
{
  public Int next()
  {
    return Int.random;
  }
  
  public Float nextFloat()
  {
    return Float.random;
  }
}

class MathmulBenchmark : Benchmark 
{
  new make(Int iterations): super(iterations) {}

  public override Obj? runOneIteration() 
  {
    chronometer := Chronometer(); 
    
    test := JGFHeapSortBench();
    
    chronometer.start();
    test.test();
    chronometer.stop();   
    this.microSeconds = this.microSeconds + chronometer.getMicroSeconds();
    return null;
  }
}

public abstract class NumericSortTest {
  public Int array_rows;
  public Int?[] TestArray; 

  public abstract Void test();
  
  new make()
  {
    TestArray = Int?[,];
  }
  
  public Void buildTestData()
  {
    TestArray.size = array_rows;
    Random? rndnum;
    rndnum = Random(); //Set seed = 1729??? 
    Int i;
    i = 0;
    while(i < array_rows)
    {
      TestArray[i] = rndnum.next();
      i = i + 1;
    }     
  }
  
  public Void Do()
  {
      NumHeapSort();
  }

  private Void NumSift(Int min, Int max)    
  {   
    while (min + min <= max)
    {                
      Bool inc;
      inc = false;
      if (min+ min < max && TestArray[min+ min] < TestArray[min+ min + 1])
        inc = true;
      Int k;
      k = inc ? min + min + 1 : min + min;                
      if (TestArray[min] < TestArray[k])
      {                   
        Int temp;
        temp = TestArray[k];
        TestArray[k] = TestArray[min];
        TestArray[min] = temp;                 
        min = k;                    
      }
      else
        min = max + 1;
    }   
  }

  private Void NumHeapSort()
  {
    Int temp;         
    Int top;
    top = array_rows - 1;     
    Int i;
    i = top / 2;
    while( i > 0)
    {               
      NumSift(i, top);
      i = i - 1;
    } 
    i = top;  
    while( i > 0)
    {
      NumSift(0, i);
      temp = TestArray[0];
      TestArray[0] = TestArray[i];
      TestArray[i] = temp;
      i = i - 1;
    }     
  } 
}


 public class JGFHeapSortBench : NumericSortTest
  {
    Int size;
    Int?[]  datasizes;
    
    new make()
    {
      datasizes = Int?[,];
      datasizes.size = 3; 
      datasizes[0] = 1000000;
      datasizes[1] = 5000000;
      datasizes[2] = 25000000;    
    }
    
    public override Void test() {
      JGFsetsize(0);
      JGFinitialise();
      JGFkernel();
      JGFvalidate();
    }
    
    public Void JGFsetsize(Int size)
    {
      this.size = size;
    }

    public Void JGFinitialise()
    {
      array_rows = datasizes[size];
      buildTestData();
    }
    
    public Void JGFkernel()
    {
      Do();
    }
    
    public Void JGFvalidate()
    {
      Bool error;
      error = false;
      Int i;
      i = 1;
      while (i < array_rows)
      {
        error = (TestArray[i] < TestArray[i - 1]);        
        if (error)
        {
          //echo("Validation failed");
          //echo("Item " + i + " = " + TestArray[i]);
          //echo("Item " + (i - 1) + " = " + TestArray[i - 1]);          
        }
        i = i + 1;
      }
    }  
  }

public class Main
{
  public static Void main(Str[] args)
  {   
    if (args.size < 1)
    {   
      echo("You must pass the number of thousands iterations.");
    }
    else
    {   
      iterations := Int.fromStr(args[0]);   
      arith := MathmulBenchmark(iterations)
      echo(arith.run());
    }    
  } 
}