public class Chronometer 
{
  private Int ticks1;
  private Int ticks2;
  private Bool stopped;
  
  public Void start() 
  {
    this.ticks1 = Duration.nowTicks();
    this.stopped = false;
  }

  public Void stop() {
    this.ticks2 = Duration.nowTicks();
    this.stopped = true;
  }
  
  public Int getMicroSeconds() {
    return (this.ticks2 - this.ticks1)/1000;
  } 
}

public abstract class Benchmark 
{
  private Int iterations;
  protected Int microSeconds;
  
  new make(Int iterations) 
  {
    this.iterations = iterations;
  } 
  
  public Int run() {
    for(i := 0 ; i < this.iterations ; i++)
      this.runOneIteration();
    return this.microSeconds;
  }
  
  public virtual Obj? runOneIteration() {
    return null;
  }   
}


class ArithmethicBenchmark : Benchmark 
{
  new make(Int iterations): super(iterations) {}

  public override Obj? runOneIteration() 
  {
    chronometer := Chronometer(); 
    test := JGFRayTracerBench();
    chronometer.start();
    test.Test();
    chronometer.stop();   
    this.microSeconds = this.microSeconds + chronometer.getMicroSeconds();
    return null;
  }
  
  
}

public class Vec
{
    public Float x;
    public Float y;
    public Float z;
  
    new make()
    {
      x = 0.0f;
      y = 0.0f;
      z = 0.0f;      
    }
  
    new makeComponents(Float a, Float b, Float c)
    {
      x = a;
      y = b;
      z = c;
    }
  
    public Void add1(Vec? a)
    {
      x = a.x + this.x;
      y = a.y + this.y;
      z = a.z + this.z;
    }
  
    public static Vec? addsStatic(Float s, Vec? a, Vec? b)
    {
      return Vec(s * a.x + b.x, s * a.y + b.y, s * a.z + b.z);
    }
  
    public Void adds(Float s, Vec? b)
    {
      this.x = this.x + (s * b.x);
      this.y = this.y + (s * b.y);
      this.z = this.z + (s * b.z);
    }
  
    public static Vec? sub1(Vec? a, Vec? b)
    {             
      return Vec(a.x - b.x, a.y - b.y, a.z - b.z);                    
    }
  
    public Void sub2(Vec? a, Vec? b)
    {     
      this.x = a.x - b.x;
      this.y = a.y - b.y;
      this.z = a.z - b.z;
    }
  
    public static Vec? mult1(Vec? a, Vec? b)
    {
       return Vec(a.x * b.x, a.y * b.y, a.z * b.z);
    }
  
    public static Vec? cross(Vec? a, Vec? b)
    {           
      return Vec(a.y * b.z - a.z * b.y, a.z * b.x - a.x * b.z, a.x * b.y - a.y * b.x);                        
    }
  
    public static Float dot(Vec? a, Vec? b)
    {     
      return a.x * b.x + a.y * b.y + a.z * b.z;
    }
  
    public static Vec? comb(Float a, Vec? A, Float b, Vec? B)
    {     
      return Vec(a * A.x + b * B.x, a * A.y + b * B.y, a * A.z + b * B.z);
    }
  
    public Void comb2(Float a, Vec? A, Float b, Vec? B)
    {
        x = a * A.x + b * B.x;
        y = a * A.y + b * B.y;
        z = a * A.z + b * B.z;
    }
  
    public Void scale(Float t)
    {     
        x = t * x;
        y = t * y;
        z = t * z;
    }
  
    public Void negate()
    {
        x = -1.0f * x;
        y = -1.0f * y;
        z = -1.0f * z;
    }
  
    public Float normalize()
    {                 
      Float len;
      len = (x * x + y * y + z * z).sqrt;    
      while (len > 0.0f)
      {
        x = x / len;
        y = y / len;
        z = z / len;
        len = 0.0f;
      }         
      return len;
    }
  
    public override Str toStr()
    {
      return "<" + x + "," + y + "," + z + ">";
    }   
}

public class View
{
    public Vec? fromVec;
    public Vec? atVec;
    public Vec? upVec;
    public Float dist;
    public Float angle;
    public Float aspect;
  
    new make(Vec? fromVec, Vec? atVec, Vec? upVec, Float dist, Float angle, Float aspect)
    {
      this.fromVec = fromVec;
      this.atVec = atVec;
      this.upVec = upVec;
      this.dist = dist;
      this.angle = angle;
      this.aspect = aspect;
    }
}

public class Ray
{
    public Vec? P;
    public Vec? D;

    new makeRay(Vec? pnt, Vec? dir)
    {                 
      P = Vec();      
      P.add1(pnt);
      D = Vec();      
      D.add1(dir);
      D.normalize();
    }

    new make()
    {
      P = Vec();
      D = Vec();
    }

    public Vec? point(Float t)
    {
      return Vec(P.x + D.x * t, P.y + D.y * t, P.z + D.z * t);
    }

    public override Str toStr()
    {
      return "{" + P.toStr() + " -> " + D.toStr() + "}";
    }
}

public class Interval
{
    public Int number;
    public Int width;
    public Int height;
    public Int yfrom;
    public Int yto;
    public Int total;

    new make(Int number, Int width, Int height, Int yfrom, Int yto, Int total)
    {
        this->number = number;
        this->width = width;
        this->height = height;
        this->yfrom = yfrom;
        this->yto = yto;
        this->total = total;
    }
}
  
public class Isect
{
    public Float t;
    public Int enter;
    public Primitive? prim;
    public Surface? surf;
  
    public override Str toStr()
    {
      return "{" + t.toStr() + "," + enter.toStr() + "," + prim.toStr() + "," + surf.toStr() + "}";
    }
}
  
public class Light
{
    public Vec? pos;
    public Float brightness;

    new make() {}

    new makeComponents(Float x, Float y, Float z, Float brightness)
    {
        this.pos = Vec(x,y,z);
        this.brightness = brightness;
    }
}
  
public class Surface
{
    public Vec? color;
    public Float kd;
    public Float ks;
    public Float shine;
    public Float kt;
    public Float ior;

    new make()
    {
        color = Vec(1.0f, 0.0f, 0.0f);
        kd = 1.0f;
        ks = 0.0f;
        shine = 0.0f;
        kt = 0.0f;
        ior = 1.0f;
    }

    public override Str toStr()
    {
        return "Surface { color=" + color.toStr() + "}";
    }
}
  
public abstract class Primitive
{
    public Surface? surf;

    new make()
    {
      surf = Surface();
    }

    public Void setColor(Float r, Float g, Float b)
    {
        surf->color = Vec(r, g, b);
    }

    public abstract Isect? intersect(Ray? ry);   
    public abstract Vec? normal(Vec? pnt);
}


public class Sphere : Primitive
{
    Vec? c;
    Float r;
    Float r2;
    Vec? v;
    Vec? b;

    new make(Vec? center, Float radius):super()
    {     
        c = center;
        r = radius;
        r2 = radius * radius;
        v = Vec();
        b = Vec();
    }

    public override Isect? intersect(Ray? ry) 
    {   
        Vec? p;
        p = ry.P;
        Vec? d;
        d = ry.D;            
        v.sub2(c, p);          
        Float b;
        b = Vec.dot(v, d);
        Float dotVV;
        dotVV = Vec.dot(v, v);
        Float disc;
        disc = b* b - dotVV + r2;        
        if (disc < 0.0f)                  
          return null;                                  
        disc = disc.sqrt();                              
        Float t;
        t = (b - disc < 0.000001f) ? b + disc : b - disc;            
        if (t < 0.000001f)            
          return null;          
        Isect? ip;
        ip = Isect();
        ip.t = t;            
        ip.enter = dotVV > r2 + 1000001f ? 1 : 0;
        ip.prim = this;
        ip.surf = this.surf;
        return ip;          
    }

    public override Vec? normal(Vec? p)
    {
        Vec? r;
        r = Vec.sub1(p, c);
        r.normalize();
        return r;
    }

    public override Str toStr()
    {
        return "Sphere {" + c.toStr() + "," + r + "}";
    }

    public Vec? getCenter()
    {
        return c;
    }
    public Void setCenter(Vec? c)
    {
        this->c = c;
    }
}

public class Scene
{
    public Light?[] lights;
    public Primitive?[] objects;
    private View? view;

    new make()
    {
        this.lights = [,];
        this.objects = [,];
    }

    public Void addLight(Light? l)
    {
        this.lights.add(l);
    }

    public Void addObject(Primitive? myobject)
    {
        this.objects.add(myobject);
    }

    public Void setView(View? view)
    {
        this.view = view;
    }

    public View? getView()
    {
        return this.view;
    }

    public Light? getLight(Int number)
    {
        return this.lights[number];
    }

    public Primitive? getObject(Int number)
    {
        return objects[number];
    }

    public Int getLights()
    {
        return this.lights.size;
    }

    public Int getObjects()
    {
        return this.objects.size;
    }

    public Void setObject(Primitive? myobject, Int pos)
    {
        this.objects[pos] = myobject;
    }
}


  public class RayTracer
    {
        public Scene? scene;
        public Light?[]? lights;
        public Primitive?[]? prim;
        public View? view;
        public Ray? tRay;
        public Int alpha;
        public Vec? voidVec;
        public Vec? L;
        public Isect? inter;
        public Int height;
        public Int width;
        public Int?[] datasizes;
        public Int checksum;
        public Int size;
        public Int numobjects;       
      
        new make()
        {
          datasizes = Int?[,];
          datasizes.size = 10;
          datasizes[0] = 5;  
          datasizes[1] = 10;        
          datasizes[2] = 15;            
          datasizes[3] = 20;        
          datasizes[4] = 25;
          datasizes[5] = 30;  
          datasizes[6] = 35;        
          datasizes[7] = 40;            
          datasizes[8] = 45;        
          datasizes[9] = 50;
          
          alpha = 255.shiftl(24);
          voidVec = Vec();
          
          tRay = Ray();
          
          L = Vec();
          inter = Isect();
          checksum = 0;
        }
  
    public Obj? createScene()
    {
        Float x;
        x = 0.0f;
        Float y;
        y = 0.0f;
        Scene? scene;
        scene = Scene();
        Primitive? p;
        Float nx;
        nx = 4.0f;
        Float ny;
        ny = 4.0f;
        Float nz;
        nz = 4.0f;
        Float i;
        i = 0.0f;     
        while (i < nx)
        {
            Float j;
            j = 0.0f;
            while (j < ny)
            {
                Float k;
                k = 0.0f;
                while (k < nz)
                {
                    Float xx;
                    xx = 20.0f / (nx - 1.0f) * i - 10.0f; 
                    Float yy;
                    yy = 20.0f / (ny - 1.0f) * j - 10.0f;
                    Float zz;
                    zz = 20.0f / (nz - 1.0f) * k - 10.0f;                                                
                    p = Sphere(Vec(xx, yy, zz), 3.0f);            
                    p.setColor(0.0f, 0.0f, ((Float)(i + j) / (Float)(nx + ny - 2.0f)));
                    p.surf.shine = 15.0f;
                    p.surf.ks = 1.5f - 1.0f;
                    p.surf.kt = 1.5f - 1.0f;
                    scene.addObject(p);            
                    k = k + 1.0f;
                }
                j = j + 1.0f;
            }
            i = i + 1.0f;
        }
        scene->addLight(Light(100.0f, 100.0f, -50.0f, 1.0f));
        scene->addLight(Light(-100.0f, 100.0f, -50.0f, 1.0f));
        scene->addLight(Light(100.0f, -100.0f, -50.0f, 1.0f));
        scene->addLight(Light(-100.0f, -100.0f, -50.0f, 1.0f));
        scene->addLight(Light(200.0f, 200.0f, 0.0f, 1.0f));
        View? v;
        v = View(Vec(x, 20.0f, -30.0f), Vec(x, y, 0.0f), Vec(0.0f, 1.0f, 0.0f), 1.0f, 35.0f * 3.14159265f / 180.0f, 1.0f);
        scene.setView(v);
        return scene;
    }
  
    public Void setScene(Scene? scene)
    {     
      Int nLights;
      nLights = scene.getLights();
      Int nObjects;
      nObjects = scene.getObjects();      
      lights = [,];
      lights.size = nLights;
      prim = [,];
      prim.size = nObjects;
      Int l;
      l = 0;      
      while(l < nLights)
      {
        lights[l] = scene.getLight(l);     
        l = l + 1;
      }
      Int o;
      o = 0;
      while (o < nObjects)
      {
        prim[o] = scene->getObject(o);        
        o = o + 1;
      }
      view = scene.getView();      
    } 
   
    public Bool intersect(Ray? r, Float tmax)
    {     
      Int nhits;
      nhits = 0;
      Isect? temp;
      temp = Isect();             
      temp.t = 1e9f;  
      this.inter = temp;         
      Int i;
      i = 0;
      while (i < this.prim.size)
      {
        Isect? tp;
        tp = this.prim[i].intersect(r);                             
        Bool sw;
        sw = (tp != null);         
        if(sw)
        {                   
          Isect? isect;
          isect = Isect();              
          isect.t = tp.t;                             
          isect.prim = tp.prim;
          isect.surf = tp.surf;
          isect.enter = tp.enter;                           
          this.inter = isect;              
          nhits = nhits + 1;                      
        }
        i = i + 1;
      }
      return nhits > 0 ? true : false;
    }


    public Vec? trace(Int level, Float weight, Ray? r)
    {                        
      if (level > 6)
        return Vec();

      Bool hit;
      hit = intersect(r, 1e6f);
      if (hit)
      {
        Vec? P;
        P = r->point(inter.t);
        Vec? N;
        N = inter.prim.normal(P);
        if (Vec.dot(r.D, N) >= 0.0f)
          N.negate();
        //return shade(level, weight, P, N, r->D, inter);
      }
      return voidVec;
    }
    
    public Vec? SpecularDirection(Vec? I, Vec? N)
    {
      Float dotIN;
      dotIN = Vec.dot(I, N);
      Float abs;
      abs = dotIN > 0.0f ? dotIN : -1.0f * dotIN;
      Vec? r;
      r = Vec.comb(1.0f / abs, I, 2.0f, N);
      r.normalize();
      return r;
    }

    public Vec? TransDir(Surface? m1, Surface? m2, Vec? I, Vec? N)
    {            
      Float n1;
      n1 = m1 == null ? 1.0f : m1.ior;
      Float n2;
      n2 = m2 == null ? 1.0f : m2.ior;
      Float eta;
      eta = n1/n2;
      Float c1;
      c1 = Vec.dot(I, N) * -1;
      Float cs2;
      cs2 = 1.0f - eta * eta * (1.0f - c1 * c1);
      if (cs2 < 0.0f)
          return null;
      Vec? r;
      r = Vec.comb(eta, I, eta * c1 - cs2.sqrt(), N);
      r->normalize();
      return r;
    }
    
    public Int Shadow(Ray? r, Float tmax)
    {
      if (intersect(r,tmax))
          return 0;
      return 1;
    }
    
    public Void render(Interval? interval)
    {     
      View? localView;
      localView = this.view;     
      Vec? viewVec;
      viewVec = Vec.sub1(localView.atVec, localView.fromVec);                      
      viewVec.normalize();     
      Vec? tmpVec;
      tmpVec = Vec(viewVec.x, viewVec.y, viewVec.z);     
      Float dotValue;
      dotValue = Vec.dot(localView.upVec, viewVec);     
      tmpVec.scale(dotValue);      
      Vec? upVec;
      upVec = Vec.sub1(localView.upVec, tmpVec);
      upVec.normalize();
      Vec? leftVec;
      leftVec = Vec.cross(localView.upVec, viewVec);
      leftVec.normalize();
      Float frustrumwidth;
      frustrumwidth = localView.dist * localView.angle.tan();            
      upVec.scale(frustrumwidth * -1.0f);         
      Float scaleValue;
      scaleValue = localView.aspect * frustrumwidth;
      leftVec.scale(scaleValue);         
      Ray? r;
      r = Ray(localView.fromVec, this.voidVec);    
      Int y;
      y = interval.yfrom;      
      Int nhits;
      nhits = 0;      
      while (y < interval.yto)
      {              
        Float ylen;
        ylen = (2.0f * y.toFloat()) / interval.width.toFloat() - 1.0f;        
        Int x;
        x = 0;
        while (x < interval->width)
        {               
          Float xlen;
          xlen = (2.0f * x.toFloat()) / interval.width.toFloat() - 1.0f;                        
          r.D = Vec.comb(xlen, leftVec, ylen, upVec);               
          r.D->add1(viewVec);                  
          r.D->normalize();                                
          Bool? hit;
          hit = this.intersect(r, 1e6f);                    
          if(hit)
            nhits = nhits + 1;
          x = x + 1;
        }
          y = y + 1;
      }     
      checksum = checksum + nhits;      
    }  
}
    
    
public class JGFRayTracerBench : RayTracer
{
    public Void JGFsetsize(Int size)
    {
        this.size = size;
    }

    public Void JGFinitialise()
    {
        width = height = datasizes[size];
        scene = createScene();
        setScene(scene);
        numobjects = scene.getObjects();
    }

    public Void JGFapplication()
    {
        Interval? interval;
        interval = Interval(0, width, height, 0, height, 1);
        render(interval);
    }

    public Void JGFvalidate()
    {
        Int?[] refval := Int?[,];
        refval.size = 10;
        refval[0] = 12;
        refval[1] = 40;            
        refval[2] = 104;            
        refval[3] = 170;     
        refval[4] = 278;            
        refval[5] = 390;
        refval[6] = 552;            
        refval[7] = 692;            
        refval[8] = 894;     
        refval[9] = 1084;           
        Int dev;
        dev = checksum - refval[size];
        if (dev != 0)
        {
            echo("Validation failed");
            echo("Pixel checksum = " + checksum);
            echo("Reference value = " + refval->get(size));
          }
      }

      public Void Test()
      {
          JGFsetsize(4);
          JGFinitialise();
          JGFapplication();
          JGFvalidate();            
      }
}  

public class Main
{
  public static Void main(Str[] args)
  {   
    if (args.size < 1)
    {   
      echo("You must pass the number of thousands iterations.");
    }
    else
    {   
      iterations := Int.fromStr(args[0]);   
      arith := ArithmethicBenchmark(iterations)
      echo(arith.run());
    }    
  } 
}