public class Chronometer 
{
  private Int ticks1;
  private Int ticks2;
  private Bool stopped;
  
  public Void start() 
  {
    this.ticks1 = Duration.nowTicks();
    this.stopped = false;
  }

  public Void stop() {
    this.ticks2 = Duration.nowTicks();
    this.stopped = true;
  }
  
  public Int getMicroSeconds() {
    return (this.ticks2 - this.ticks1)/1000;
  } 
}

public abstract class Benchmark 
{
  private Int iterations;
  protected Int microSeconds;
  
  new make(Int iterations) 
  {
    this.iterations = iterations;
  } 
  
  public Int run() {
    for(i := 0 ; i < this.iterations ; i++)
      this.runOneIteration();
    return this.microSeconds;
  }
  
  public virtual Obj? runOneIteration() {
    return null;
  }   
}

class Random
{
  public Int next()
  {
    return Int.random;
  }
  
  public Float nextFloat()
  {
    return Float.random;
  }
}

class ArithmethicBenchmark : Benchmark 
{
  new make(Int iterations): super(iterations) {}

  public override Obj? runOneIteration() 
  {
    chronometer := Chronometer(); 
    
    test := JGFSparseMatmultBench();
    
    chronometer.start();
    test.test();
    chronometer.stop();   
    this.microSeconds = this.microSeconds + chronometer.getMicroSeconds();
    return null;
  }
}

public abstract class TestPyBench {
        public abstract Void test();
    }


 public class JGFSparseMatmultBench : TestPyBench
  {
    Int size;
    
    //Atributos de clase con inicializacion estatica
    Int?[]  datasizes_M;
    Int?[]  datasizes_N;
    Int?[]  datasizes_nz;
    Int SPARSE_NUM_ITER;
         
    //Atributos de instancia
    Random? R;
    
    Float?[]? x;
    Float?[]? y;
    
    Float?[]? val;
    Int?[]? col;
    Int?[]? row;
    
    Float ytotal;
    
    new make()
    {
      ytotal = 0.0f;
      datasizes_M = Int?[,];
      datasizes_M.size = 3; 
      datasizes_M[0] = 50000;
      datasizes_M[1] = 100000;
      datasizes_M[2] = 500000;
      datasizes_N = Int?[,];
      datasizes_N.size = 3;      
      datasizes_N[0] = 50000;
      datasizes_N[1] = 100000;
      datasizes_N[2] = 500000;
      datasizes_nz = Int?[,];
      datasizes_nz.size = 3;
      datasizes_nz[0] = 250000;
      datasizes_nz[1] = 500000;
      datasizes_nz[2] = 2500000;
      SPARSE_NUM_ITER = 10;
    }
    
    public override Void test() 
    {   
      JGFsetsize(0);
      JGFinitialise();
      JGFkernel();
      JGFvalidate();     
    }
    
    public Void JGFsetsize(Int size)
    {
      this.size = size;
    }
    
    public Void JGFinitialise()
    {      
      R = Random();

      x = RandomVector(datasizes_N[size], R);
      y = Float?[,];
      y.size = datasizes_M[size];
      for (Int i := 0; i < datasizes_M[size]; i++)
      {
        y[i] = 0.0f;
      }
      val = Float?[,];
      val.size = datasizes_nz[size];
      col = Int?[,];
      col.size = datasizes_nz[size];
      row = Int?[,];
      row.size = datasizes_nz[size];
      
      for (Int i := 0; i < datasizes_nz[size]; i = i + 1)
      {
        Int temp1;
        temp1 = R.next().mod(datasizes_M[size]);
        Int temp2;
        temp2 = R.next().mod(datasizes_N[size]);                            
        row[i] = temp1 >= 0 ? temp1 : -1 * temp1;
        col[i] = temp2 >= 0 ? temp2 : -1 * temp2;
        val[i] = R.nextFloat();                   
      }
    }
    
    private static Float?[]? RandomVector(Int N, Random? R)
    {
      Float?[]? a;
      a = Float?[,];
      a.size = N;
      for (Int i := 0; i < N; i = i + 1)
        a[i] = R.nextFloat() * 0.000001f;                
      return a;
    }
  
    public Void JGFkernel()
    {       
      for (Int reps := 0; reps < SPARSE_NUM_ITER; reps = reps + 1)
      {      
        for (Int i := 0; i < val.size ; i = i + 1)
        {    
            y[row[i]] = y[row[i]] + (x[col[i]] * val[i]);          
        }
      }          
      for (Int i := 0; i < val.size ; i = i + 1)
      {
        ytotal = ytotal + y[row[i]];                  
      }     
    }
  
    public Void JGFvalidate()
    {
      Float?[]? refval;
      refval = Float?[,];
      refval.size = 3;
      refval[0] = 75.02484945753453f;
      refval[1] = 150.0130719633895f;
      refval[2] = 749.5245870753752f;
      Float dev;
      dev = ytotal - refval[size];
      dev = dev >= 0.0f ? dev : dev * -1.0f;
      if (dev > 0.000000000001f)
      {
        //echo("Validation failed");
        //echo("ytotal = " + ytotal + "  " + dev + "  " + size);
      }
    }    
  }

public class Main
{
  public static Void main(Str[] args)
  {   
    if (args.size < 1)
    {   
      echo("You must pass the number of thousands iterations.");
    }
    else
    {   
      iterations := Int.fromStr(args[0]);   
      arith := ArithmethicBenchmark(iterations)
      echo(arith.run());
    }    
  } 
}