public class Chronometer 
{
  private Int ticks1;
  private Int ticks2;
  private Bool stopped;
  
  public Void start() 
  {
    this.ticks1 = Duration.nowTicks();
    this.stopped = false;
  }

  public Void stop() {
    this.ticks2 = Duration.nowTicks();
    this.stopped = true;
  }
  
  public Int getMicroSeconds() {
    return (this.ticks2 - this.ticks1)/1000;
  } 
}

public abstract class Benchmark 
{
  private Int iterations;
  protected Int microSeconds;
  
  new make(Int iterations) 
  {
    this.iterations = iterations;
  } 
  
  public Int run() {
    for(i := 0 ; i < this.iterations ; i++)
      this.runOneIteration();
    return this.microSeconds;
  }
  
  public virtual Obj? runOneIteration() {
    return null;
  }   
}


class ArithmethicBenchmark : Benchmark 
{
  new make(Int iterations): super(iterations) {}

  public override Obj? runOneIteration() 
  {
    chronometer := Chronometer();   
    pystone := Pystone();    
    chronometer.start();
    value := pystone.Proc0(5000);
    chronometer.stop();   
    this.microSeconds = this.microSeconds + chronometer.getMicroSeconds();
    return value;
  }
}


public class Idents
{
  public static const Int Ident1 := 1;
  public static const Int Ident2 := 2;
  public static const Int Ident3 := 3;
  public static const Int Ident4 := 4;
  public static const Int Ident5 := 5;
}
    
public class Record
{
    public Record? PtrComp;        
    public Int Discr;
    public Int EnumComp;
    public Int IntComp;
    public Str StringComp;

    new make(Int Discr, Int EnumComp, Int IntComp, Str StringComp)
    {            
        this.Discr = Discr;  
        this.EnumComp = EnumComp;
        this.IntComp = IntComp;
        this.StringComp = StringComp;
    }        

    public Record? copy()
    {
        Record? record := Record(this.Discr, this.EnumComp, this.IntComp, this.StringComp);
        record.PtrComp = this.PtrComp;
        return record;
    }
}
    
public class Pystone
{
    public Int IntGlob;
    public Bool BoolGlob;
    public Int Char1Glob;
    public Int Char2Glob;
    public Int?[] Array1Glob;
    public Int?[][] Array2Glob;
    public Record? PtrGlb;
    public Record? PtrGlbNext;    

    new make()
    {     
      this.IntGlob = 0;
      this.BoolGlob = false;
      this.Char1Glob = ' ';
      this.Char2Glob = ' ';
      this.Array1Glob = Int?[,];
      this.Array1Glob.size = 51;
      for (Int i := 0; i < 51; i++)
        this.Array1Glob[i] = 0;
      this.Array2Glob = [,];
      this.Array2Glob.size = 51;
      for (Int i := 0; i < 51; i++)
      {        
        row := Int?[,];
        row.size = 51;
        for (Int j := 0; j < 51; j++)
          row[j] = 0;
        this.Array2Glob[i] = row;
      }        
      this.PtrGlb = null;
      this.PtrGlbNext = null;
    }
              
    public Float?[] pystones(Int loops)
    {            
      return this.Proc0(loops);
    }  
  
    public Float?[] Proc0(Int loops)
    {       
      Int benchtime := 0;
      Int nulltime := 0;
      Str String1Loc := "";
         
      Chronometer? chronometer := Chronometer(); 
      chronometer.start();
      for (Int i := 0; i < loops; i++)
        continue;
      chronometer.stop();
      nulltime = chronometer.getMicroSeconds();
      
      
      this.PtrGlbNext = Record(0, Idents.Ident1, 0, "");
      this.PtrGlb = Record(0, Idents.Ident1, 0, "");
      this.PtrGlb.PtrComp = this.PtrGlbNext;
      this.PtrGlb.Discr = Idents.Ident1;
      this.PtrGlb.EnumComp = Idents.Ident3;
      this.PtrGlb.IntComp = 40;
      this.PtrGlb.StringComp = "DHRYSTONE PROGRAM, SOME STRING";
      String1Loc = "DHRYSTONE PROGRAM, 1'ST STRING";
      this.Array2Glob.get(8).set(7,10);
            
      chronometer.start();      
      for (Int i := 0; i < loops; i++)            
        this.DoWork();     
      chronometer.stop();
      benchtime = chronometer.getMicroSeconds();
      
      result := Float?[,];
      result.size = 2;
      result[0] = benchtime.toFloat;
            
      
      Float loopsPerBenchtime := 0f
      if(benchtime > 0)
        loopsPerBenchtime = (loops / benchtime).toFloat;      
      result[1] = loopsPerBenchtime;     
      return result;
    }

   private Void DoWork()
    {
      Int EnumLoc := 0;            
      Str String1Loc := "DHRYSTONE PROGRAM, 1'ST STRING";
      Str String2Loc := "";
      Int IntLoc1 := 0;
      Int IntLoc2 := 0;
      Int IntLoc3 := 0;
      Int CharIndex := 'A';      
      
      Proc5();
      Proc4();
      IntLoc1 = 2;
      IntLoc2 = 3;
      String2Loc = "DHRYSTONE PROGRAM, 2'ND STRING";
      EnumLoc = Idents.Ident2;
      BoolGlob = !Func2(String1Loc, String2Loc);
      while (IntLoc1 < IntLoc2)
      {
        IntLoc3 = 5 * IntLoc1 - IntLoc2;
        IntLoc3 = Proc7(IntLoc1, IntLoc2);
        IntLoc1 = IntLoc1 + 1;
      }
      Proc8(Array1Glob, Array2Glob, IntLoc1, IntLoc3);                
      PtrGlb = Proc1(PtrGlb);             
      CharIndex = 'A';              
      while (CharIndex <= Char2Glob) {                                              
        if (EnumLoc == Func1(CharIndex, 'C')){            
          EnumLoc = Proc6(Idents.Ident1);
        }       
        CharIndex = CharIndex + 1;
      }     
      IntLoc3 = IntLoc2 * IntLoc1;
      IntLoc2 = IntLoc3 / IntLoc1;
      IntLoc2 = 7 * (IntLoc3 - IntLoc2) - IntLoc1;      
      IntLoc1 = Proc2(IntLoc1);
    }
  
    public Record? Proc1(Record? PtrParIn)
    {     
        Record? NextRecord := PtrGlb.copy();   
        PtrParIn.PtrComp = NextRecord;      
        PtrParIn.IntComp = 5;     
        NextRecord.IntComp = PtrParIn.IntComp;      
        NextRecord.PtrComp = PtrParIn.PtrComp;      
        NextRecord.PtrComp = Proc3(NextRecord.PtrComp);     
        if (NextRecord.Discr == Idents.Ident1)
        {       
            NextRecord.IntComp = 6;       
            NextRecord.EnumComp = Proc6(PtrParIn.EnumComp);       
            NextRecord.PtrComp = this.PtrGlb.PtrComp;
            NextRecord.IntComp = Proc7(NextRecord.IntComp, 10);
        }
        else 
          PtrParIn = NextRecord.copy();
        NextRecord.PtrComp = null;
        return PtrParIn;
    }
  
    public Int Proc2(Int IntParIO) 
    {        
        Int EnumLoc := Idents.Ident5;
        Int IntLoc := IntParIO + 10;            
        while (true) 
        {
            if (this.Char1Glob == 'A') 
            {
                IntLoc = IntLoc - 1;
                IntParIO = IntLoc - this.IntGlob;
                EnumLoc = Idents.Ident1;
            }
            if (EnumLoc == Idents.Ident1)
              return IntParIO;
        }
        return IntParIO;
    }
  
    public Record? Proc3(Record? PtrParOut)
    {
        if (PtrGlb != null)        
            PtrParOut = PtrGlb.PtrComp;        
        else 
          IntGlob = 100;
        PtrGlb.IntComp = Proc7(10, IntGlob);
        return PtrParOut;
    }  
  
    public Void Proc4()
    {
        Bool BoolLoc := (Char1Glob == 'A');
        BoolLoc = (BoolLoc || BoolGlob);
        Char2Glob = 'B';
    }  
  
    public Void Proc5()
    {
      Char1Glob = 'A';
      BoolGlob = false;
    }
  
    public Int Proc6(Int EnumParIn)
    {        
        Int EnumParOut := EnumParIn;
        if (!Func3(EnumParIn))
            EnumParOut = Idents.Ident4;
        if (EnumParIn == Idents.Ident1)
            EnumParOut = Idents.Ident1;
        else
        {
            if (EnumParIn == Idents.Ident2)
            {
                if (IntGlob > 100) 
                  EnumParOut = Idents.Ident1;
                else
                  EnumParOut = Idents.Ident4;
            }
            else
            {
                if (EnumParIn == Idents.Ident3)
                    EnumParOut = Idents.Ident2;
                else
                {
                    if (EnumParIn == Idents.Ident4)
                    {
                    }
                    else
                    {
                        if (EnumParIn == Idents.Ident5)
                            EnumParOut = Idents.Ident3;
                    }
                }
            }
        }
        return EnumParOut;
    }
  
    public Int Proc7(Int IntParI1, Int IntParI2)
    {     
        Int IntLoc := IntParI1 + 2;
        Int IntParOut := IntParI2 + IntLoc;
        return IntParOut;
    }
  
    public Void Proc8(Int?[] Array1Par, Int?[][] Array2Par, Int IntParI1, Int IntParI2)
    {
      Int IntLoc := IntParI1 + 5;
      Array1Par[IntLoc] = IntParI2;
      Array1Par[IntLoc + 1] = Array1Par[IntLoc];
      Array1Par[IntLoc + 30] = IntLoc;
      for (Int IntIndex := IntLoc ; IntIndex < IntLoc + 2 ; IntIndex++)
          Array2Par[IntLoc][IntIndex] = IntLoc;  
      Array2Par[IntLoc][IntLoc - 1] = Array2Par[IntLoc][IntLoc - 1] + 1;
      Array2Par[IntLoc + 20][IntLoc] = Array1Par[IntLoc];
      this.IntGlob = 5;
    }  
  
    public static Int Func1(Int CharPar1, Int CharPar2)
    {                             
      Int CharLoc1 := CharPar1;
      Int CharLoc2 := CharLoc1;      
      if (CharLoc2 != CharPar2)
        return Idents.Ident1;
      else 
        return Idents.Ident2;
    }
  
    public Bool Func2(Str StrParI1, Str StrParI2)
    {
      Int CharLoc := ' ';
      Int IntLoc := 1;
      while(IntLoc <= 1)
      {
          if (Func1(StrParI1[IntLoc], StrParI2[IntLoc + 1]) == Idents.Ident1)
          {
              CharLoc = 'A';
              IntLoc = IntLoc + 1;
          }
      }
      if (CharLoc >= 'W' && CharLoc <= 'Z')
          IntLoc = 7;
      if (CharLoc == 'X')
              return true;
      else
      {
          if (StrParI1.compare(StrParI2) > 0)
          {
              IntLoc = IntLoc.plus(7);
              return true;
          }
          else 
            return false;
      }         
    }
  
  
    public Bool Func3(Int EnumParIn)
    {        
        Int EnumLoc := EnumParIn;
        if (EnumLoc == Idents.Ident3) return true;
        return false;
    }
}


public class Main
{
  public static Void main(Str[] args)
  {   
    if (args.size < 1)
    {   
      echo("You must pass the number of thousands iterations.");
    }
    else
    {   
      iterations := Int.fromStr(args[0]);   
      arith := ArithmethicBenchmark(iterations)
      echo(arith.run());
    }    
  } 
}