Namespace JG.SparseMaumult
    Public Class Chronometer
        Private ticks1 As DateTime, ticks2 As DateTime
        Private stopped As Boolean

        Public Sub Start()
            ticks1 = DateTime.Now
            stopped = False
        End Sub
        Public Sub [Stop]()
            ticks2 = DateTime.Now
            stopped = True
        End Sub

        Private Shared Function TicksToMicroSeconds(t1 As DateTime, t2 As DateTime) As Integer
            Return TicksToMiliSeconds(t1, t2) * 1000
        End Function

        Private Shared Function TicksToMiliSeconds(t1 As DateTime, t2 As DateTime) As Integer
            Dim difference As TimeSpan = t2.Subtract(t1)
            Return (difference.Milliseconds + difference.Seconds * 1000 + difference.Minutes * 60000)
        End Function

        Private Shared Function TicksToSeconds(t1 As DateTime, t2 As DateTime) As Integer
            Dim difference As TimeSpan = t2.Subtract(t1)
            Return (difference.Seconds + difference.Minutes * 60)
        End Function

        Public Function GetMicroSeconds() As Integer
            If stopped Then
                Return TicksToMicroSeconds(ticks1, ticks2)
            End If
            Return TicksToMicroSeconds(ticks1, DateTime.Now)
        End Function

        Public Function GetMiliSeconds() As Integer
            If stopped Then
                Return TicksToMiliSeconds(ticks1, ticks2)
            End If
            Return TicksToMiliSeconds(ticks1, DateTime.Now)
        End Function

        Public Function GetSeconds() As Integer
            If stopped Then
                Return TicksToSeconds(ticks1, ticks2)
            End If
            Return TicksToSeconds(ticks1, DateTime.Now)
        End Function
    End Class

    Public Class BenchMark
        Private iterations As Integer
        Protected microSeconds As Integer

        Public Sub New(iterations As Integer)
            Me.iterations = iterations
        End Sub

        Public Function run() As Integer
            Dim self As BenchMark = Me
            For i As Integer = 0 To iterations - 1
                self.runOneIteration()
            Next
            Return Me.microSeconds
        End Function

        Public Overridable Function runOneIteration() As Object
            Return Nothing
        End Function
    End Class

    Public Class ArithmethicBenchmark
        Inherits BenchMark
        Public Sub New(iterations As Integer)
            MyBase.New(iterations)
        End Sub
        Public Overrides Function runOneIteration() As Object
            Dim chronometer As New Chronometer()
            Dim test As Test = New JGFSparseMatmultBench()
            chronometer.Start()
            test.test()
            chronometer.[Stop]()
            Me.microSeconds = Me.microSeconds + chronometer.GetMicroSeconds()
            Return Nothing
        End Function
    End Class

    Public MustInherit Class Test
        Public MustOverride Sub test()
    End Class

    Public Class JGFSparseMatmultBench
        Inherits Test
        Private size As Integer

        Private Shared datasizes_M As Integer()
        Private Shared datasizes_N As Integer()
        Private Shared datasizes_nz As Integer()
        Private Shared SPARSE_NUM_ITER As Integer
        Public Shared ytotal As Double

        Private R As Random

        Private x As Double()
        Private y As Double()
        Private val As Double()
        Private col As Integer()
        Private row As Integer()

        Public Sub New()
            JGFSparseMatmultBench.SPARSE_NUM_ITER = 10
            JGFSparseMatmultBench.ytotal = 0.0

            JGFSparseMatmultBench.datasizes_M = New Integer(2) {}
            JGFSparseMatmultBench.datasizes_M(0) = 50000
            JGFSparseMatmultBench.datasizes_M(1) = 100000
            JGFSparseMatmultBench.datasizes_M(2) = 500000

            JGFSparseMatmultBench.datasizes_N = New Integer(2) {}
            JGFSparseMatmultBench.datasizes_N(0) = 50000
            JGFSparseMatmultBench.datasizes_N(1) = 100000
            JGFSparseMatmultBench.datasizes_N(2) = 500000

            JGFSparseMatmultBench.datasizes_nz = New Integer(2) {}
            JGFSparseMatmultBench.datasizes_nz(0) = 250000
            JGFSparseMatmultBench.datasizes_nz(1) = 500000
            JGFSparseMatmultBench.datasizes_nz(2) = 2500000
        End Sub

        Public Sub JGFsetsize(size As Integer)
            Me.size = size
        End Sub

        Public Sub JGFinitialise()
            R = New Random(1010)

            x = RandomVector(datasizes_N(size), R)
            y = New Double(datasizes_M(size) - 1) {}

            val = New Double(datasizes_nz(size) - 1) {}
            col = New Integer(datasizes_nz(size) - 1) {}
            row = New Integer(datasizes_nz(size) - 1) {}

            For i As Integer = 0 To datasizes_nz(size) - 1
                Dim temp1 As Integer
                temp1 = R.[Next]() Mod datasizes_M(size)
                Dim temp2 As Integer
                temp2 = R.[Next]() Mod datasizes_N(size)
                row(i) = If(temp1 >= 0, temp1, -1 * temp1)
                col(i) = If(temp2 >= 0, temp2, -1 * temp2)
                val(i) = R.NextDouble()
            Next
        End Sub

        Public Sub JGFkernel()
            For reps As Integer = 1 To SPARSE_NUM_ITER
                For i As Integer = 0 To val.Length() - 1
                    y(row(i)) += x(col(i)) * val(i)
                Next
            Next
            For i As Integer = 0 To val.Length() - 1
                ytotal += y(row(i))
            Next
        End Sub

        Public Sub JGFvalidate()
            Dim refval As Double()
            refval = New Double(2) {}
            refval(0) = 75.0248494575345
            refval(1) = 150.01307196339
            refval(2) = 749.524587075375
            Dim dev As Double
            dev = ytotal - refval(size)
            dev = If(dev >= 0, dev, -1 * dev)
            If dev > 0.000000000001 Then
                'Console.WriteLine("Validation failed");
                'Console.WriteLine("ytotal = " + ytotal + "  " + dev + "  " + size);
            End If
        End Sub

        Private Shared Function RandomVector(N As Integer, R As Random) As Double()
            Dim a As Double()
            a = New Double(N - 1) {}
            For i As Integer = 0 To N - 1
                a(i) = R.NextDouble() * 0.000001
            Next
            Return a
        End Function

        Public Overrides Sub test()
            JGFsetsize(0)
            JGFinitialise()
            JGFkernel()
            JGFvalidate()
        End Sub
    End Class

    Public Class Program
        Public Shared Sub Main(args As String())
            If args.Length < 1 Then
                Console.[Error].WriteLine("You must pass the number of thousands iterations.")
                System.Environment.[Exit](-1)
            End If
            Dim iterations As Integer = Convert.ToInt32(args(0))
            Dim arith As New ArithmethicBenchmark(iterations)
            Console.WriteLine(arith.run())
        End Sub
    End Class
End Namespace
